﻿--[[
Name: StatLogic-1.0
Description: A Library for stat conversion, calculation and summarization.
Revision: $Revision: 31848 $
Author: Whitetooth@Cenarius (hotdogee@bahamut.twbbs.org)
LastUpdate: $Date: 2007-04-04 08:25:16 -0400 (Wed, 04 Apr 2007) $
Website:
Documentation:
SVN: $URL: http://svn.wowace.com/wowace/trunk/StatLogicLib/StatLogic-1.0/StatLogic-1.0.lua $
Dependencies: AceLibrary, AceLocale-2.2, Babble-Spell-2.2
License: LGPL v2.1
Design Aim: Indexing algorithm instead of calling strfind for every stat
]]

-- This library is still in early development

local MAJOR_VERSION = "StatLogic-1.0"
local MINOR_VERSION = tonumber(("$Revision: 31848 $"):sub(12, -3))

if not AceLibrary then error(MAJOR_VERSION.." requires AceLibrary") end
if not AceLibrary:IsNewVersion(MAJOR_VERSION, MINOR_VERSION) then return end


--------------
-- Librarys --
--------------
local BS = AceLibrary("Babble-Spell-2.2")
local L = AceLibrary("AceLocale-2.2"):new(MAJOR_VERSION)


-------------------------
-- Localization Tables --
-------------------------
-- uncomment below to print out print out every missing translation for each locale
-- L:EnableDebugging()
L:RegisterTranslations("enUS", function()
	return {
		["Locale"] = "enUS",
		["tonumber"] = tonumber,
		--["Dual Wield"] = "Dual Wield", -- Babble-Spell-2.2.lua
		-------------------
		-- Exclude Table --
		-------------------
		-- By looking at the first ExcludeLen letters of a line we can exclude a lot of lines
		["ExcludeLen"] = 5, -- using string.utf8len
		["Exclude"] = {
			[""] = true,
			[" \n"] = true,
			["Binds when equipped"] = true,
			["Binds when picked up"] = true,
			["Binds when used"] = true,
			["Red Socket"] = true,
			["Yellow Socket"] = true,
			["Blue Socket"] = true,
			["Meta Socket"] = true,
			["Quest Item"] = true,
			["This Item Begins a Quest"] = true,
			["Uniqu"] = true, -- Unique (20)
			["Requi"] = true, -- Requires Level xx
			["\nRequ"] = true, -- Requires Level xx
			["Class"] = true, -- Classes: xx
			["Races:"] = true, -- Races: xx (vendor mounts)
			["Use: "] = true, -- Use:
			["Chanc"] = true, -- Chance On Hit:
			-- Set Bonuses
			["Set: "] = true,
			["(2) S"] = true,
			["(3) S"] = true,
			["(4) S"] = true,
			["(5) S"] = true,
			["(6) S"] = true,
			["(7) S"] = true,
			["(8) S"] = true,
			-- Equip type
			["Projectile"] = true, -- Ice Threaded Arrow ID:19316
			[INVTYPE_AMMO] = true,
			[INVTYPE_HEAD] = true,
			[INVTYPE_NECK] = true,
			[INVTYPE_SHOULDER] = true,
			[INVTYPE_BODY] = true,
			[INVTYPE_CHEST] = true,
			[INVTYPE_ROBE] = true,
			[INVTYPE_WAIST] = true,
			[INVTYPE_LEGS] = true,
			[INVTYPE_FEET] = true,
			[INVTYPE_WRIST] = true,
			[INVTYPE_HAND] = true,
			[INVTYPE_FINGER] = true,
			[INVTYPE_TRINKET] = true,
			[INVTYPE_CLOAK] = true,
			[INVTYPE_WEAPON] = true,
			[INVTYPE_SHIELD] = true,
			[INVTYPE_2HWEAPON] = true,
			[INVTYPE_WEAPONMAINHAND] = true,
			[INVTYPE_WEAPONOFFHAND] = true,
			[INVTYPE_HOLDABLE] = true,
			[INVTYPE_RANGED] = true,
			[INVTYPE_THROWN] = true,
			[INVTYPE_RANGEDRIGHT] = true,
			[INVTYPE_RELIC] = true,
			[INVTYPE_TABARD] = true,
			[INVTYPE_BAG] = true,
		},
		--[[
		textTable = {
			"Spell Damage +6 and Spell Hit Rating +5",
			"+3 Stamina, +4 Critical Strike Rating",
			"+26 Healing Spells & 2% Reduced Threat",
			"+3 Stamina/+4 Critical Strike Rating",
			"Socket Bonus: 2 mana per 5 sec.",
			"Equip: Increases damage and healing done by magical spells and effects by up to 150.",
			"Equip: Increases the spell critical strike rating of all party members within 30 yards by 28.",
			"Equip: Increases damage and healing done by magical spells and effects of all party members within 30 yards by up to 33.",
			"Equip: Increases healing done by magical spells and effects of all party members within 30 yards by up to 62.",
			"Equip: Increases your spell damage by up to 120 and your healing by up to 300.",
			"Equip: Restores 11 mana per 5 seconds to all party members within 30 yards.",
			"Equip: Increases healing done by spells and effects by up to 300.",
			"Equip: Increases attack power by 420 in Cat, Bear, Dire Bear, and Moonkin forms only.",
			"+10 Defense Rating/+10 Stamina/+15 Block Value", -- ZG Enchant
			"+26 Attack Power and +14 Critical Strike Rating", -- Swift Windfire Diamond ID:28556
			"+26 Healing Spells & 2% Reduced Threat", -- Bracing Earthstorm Diamond ID:25897
			"+6 Spell Damage, +5 Spell Crit Rating", -- Potent Ornate Topaz ID: 28123
			----
			"Critical Rating +6 and Dodge Rating +5", -- Assassin's Fire Opal ID:30565
			"Healing +11 and 2 mana per 5 sec.", -- Royal Tanzanite ID: 30603
		}
		--]]
		-----------------------
		-- Whole Text Lookup --
		-----------------------
		-- Mainly used for enchants that doesn't have numbers in the text
		["WholeTextLookup"] = {
			["Savagery"] = {["AP"] = 70}, -- 
	
			["Minor Wizard Oil"] = {["SPELL_DMG"] = 8, ["HEAL"] = 8}, -- 
			["Lesser Wizard Oil"] = {["SPELL_DMG"] = 16, ["HEAL"] = 16}, -- 
			["Wizard Oil"] = {["SPELL_DMG"] = 24, ["HEAL"] = 24}, -- 
			["Brilliant Wizard Oil"] = {["SPELL_DMG"] = 36, ["HEAL"] = 36, ["CR_CRIT_SPELL"] = 14}, -- 
			
			["Superior Mana Oil"] = {["MANA_REG"] = 14}, -- 
			["Minor Mana Oil"] = {["MANA_REG"] = 4}, -- 
			["Lesser Mana Oil"] = {["MANA_REG"] = 8}, -- 
			["Brilliant Mana Oil"] = {["MANA_REG"] = 12, ["HEAL"] = 25}, -- 
			["Superior Wizard Oil"] = {["SPELL_DMG"] = 42, ["HEAL"] = 42}, -- 
			
			["Eternium Line"] = {["FISHING"] = 5}, -- 
			["Vitality"] = {["MANA_REG"] = 4, ["HEALTH_REG"] = 4}, -- 
			["Soulfrost"] = {["SHADOW_SPELL_DMG"] = 54, ["FROST_SPELL_DMG"] = 54}, -- 
			["Sunfire"] = {["ARCANE_SPELL_DMG"] = 50, ["FIRE_SPELL_DMG"] = 50}, -- 
			
			["Mithril Spurs"] = {["MOUNT_SPEED%"] = 4}, -- Mithril Spurs
			["Minor Mount Speed Increase"] = {["MOUNT_SPEED%"] = 2}, -- Enchant Gloves - Riding Skill
			["Equip: Run speed increased slightly."] = {["RUN_SPEED%"] = 8}, -- [Highlander's Plate Greaves] ID: 20048
			["Run speed increased slightly"] = {["RUN_SPEED%"] = 8}, --
			["Minor Speed Increase"] = {["RUN_SPEED%"] = 8}, -- Enchant Boots - Minor Speed
			["Equip: Allows underwater breathing."] = false, -- [Band of Icy Depths] ID: 21526
			["Allows underwater breathing"] = false, --
			["Equip: Immune to Disarm."] = false, -- [Stronghold Gauntlets] ID: 12639
			["Immune to Disarm"] = false, --
			["Crusader"] = false, -- Enchant Crusader
			["Lifestealing"] = false, -- Enchant Crusader
		},
		----------------------------
		-- Single Plus Stat Check --
		----------------------------
		-- depending on locale, it may be
		-- +19 Stamina = "^%+(%d+) ([%a ]+%a)$"
		-- Stamina +19 = "^([%a ]+%a) %+(%d+)$"
		-- +19 耐力 = "^%+(%d+) (.-)$"
		--["SinglePlusStatCheck"] = "^%+(%d+) ([%a ]+%a)$",
		["SinglePlusStatCheck"] = "^%+(%d+) (.-)$",
		-----------------------------
		-- Single Equip Stat Check --
		-----------------------------
		-- stat1, value, stat2 = strfind
		-- stat = stat1..stat2
		-- "^Equip: (.-) by u?p? ?t?o? ?(%d+) ?(.-)%.$"
		["SingleEquipStatCheck"] = "^Equip: (.-) by u?p? ?t?o? ?(%d+) ?(.-)%.$",
		-------------
		-- PreScan --
		-------------
		-- Special cases that need to be dealt with before base scan
		["PreScanPatterns"] = {
			--["^Equip: Increases attack power by (%d+) in Cat"] = "FERAL_AP",
			--["^Equip: Increases attack power by (%d+) when fighting Undead"] = "AP_UNDEAD", -- Seal of the Dawn ID:13029
			["^(%d+) Block$"] = "BLOCK_VALUE",
			["^(%d+) Armor$"] = "ARMOR",
			["Reinforced %(%+(%d+) Armor%)"] = "ARMOR_BONUS",
			["Mana Regen (%d+) per 5 sec%.$"] = "MANA_REG",
			["^%+?%d+ %- (%d+) .-Damage$"] = "MAX_DAMAGE",
			["^%(([%d%.]+) damage per second%)$"] = "DPS",
			-- Exclude
			["^(%d+) Slot"] = false, -- Set Name (0/9)
			["^[%a '%-]+%((%d+)/%d+%)$"] = false, -- Set Name (0/9)
			["|cff808080"] = false, -- Gray text "  |cff808080Requires at least 2 Yellow gems|r\n  |cff808080Requires at least 1 Red gem|r"
			-- Procs
			["[Cc]hance"] = false, -- [Mark of Defiance] ID:27924 -- [Staff of the Qiraji Prophets] ID:21128
			["[Ss]ometimes"] = false, -- [Darkmoon Card: Heroism] ID:19287
			["[Ww]hen struck in combat"] = false, -- [Essence of the Pure Flame] ID: 18815
		},
		--------------
		-- DeepScan --
		--------------
		-- Strip leading "Equip: ", "Socket Bonus: "
		["Equip: "] = "Equip: ",
		["Socket Bonus: "] = "Socket Bonus: ",
		-- Strip trailing "."
		["."] = ".",
		["DeepScanSeparators"] = {
			" and ", -- "Critical Rating +6 and Dodge Rating +5": Assassin's Fire Opal ID:30565
			"/", -- "+10 Defense Rating/+10 Stamina/+15 Block Value": ZG Enchant
			" & ", -- "+26 Healing Spells & 2% Reduced Threat": Bracing Earthstorm Diamond ID:25897
			", ", -- "+6 Spell Damage, +5 Spell Crit Rating": Potent Ornate Topaz ID: 28123
			"%. ", -- "Equip: Increases attack power by 81 when fighting Undead. It also allows the acquisition of Scourgestones on behalf of the Argent Dawn.": Seal of the Dawn
		},
		["DeepScanPatterns"] = {
			"^(.-) by u?p? ?t?o? ?(%d+) ?(.-)$", -- "xxx by up to 22 xxx" (scan first)
			"^(.-) ?%+(%d+) ?(.-)$", -- "xxx xxx +22" or "+22 xxx xxx" or "xxx +22 xxx" (scan 2ed)
			"^(.-) ?([%d%.]+) ?(.-)$", -- 22.22 xxx xxx (scan last)
		},
		-----------------------
		-- Stat Lookup Table --
		-----------------------
		["StatIDLookup"] = {
	
			["Increases your effective stealth level"] = {"STEALTH_LEVEL"}, -- [Nightscape Boots] ID: 8197
			["Weapon Damage"] = {"MELEE_DMG"}, -- Enchant
			["Increases mount speed%"] = {"MOUNT_SPEED%"}, -- [Highlander's Plate Greaves] ID: 20048
	
			["All Stats"] = {"STR", "AGI", "STA", "INT", "SPI",},
			["Strength"] = {"STR",},
			["Agility"] = {"AGI",},
			["Stamina"] = {"STA",},
			["Intellect"] = {"INT",},
			["Spirit"] = {"SPI",},
	
			["Arcane Resistance"] = {"ARCANE_RES",},
			["Fire Resistance"] = {"FIRE_RES",},
			["Nature Resistance"] = {"NATURE_RES",},
			["Frost Resistance"] = {"FROST_RES",},
			["Shadow Resistance"] = {"SHADOW_RES",},
			["Shadow resistance"] = {"SHADOW_RES",}, -- Demons Blood ID: 10779
			["All Resistances"] = {"ARCANE_RES", "FIRE_RES", "FROST_RES", "NATURE_RES", "SHADOW_RES",},
			["Resist All"] = {"ARCANE_RES", "FIRE_RES", "FROST_RES", "NATURE_RES", "SHADOW_RES",},
	
			["Fishing"] = {"FISHING",}, -- Fishing enchant ID:846
			["Fishing Skill"] = {"FISHING",}, -- Fishing lure
			["Increased Fishing"] = {"FISHING",}, -- Equip: Increased Fishing +20.
			["Mining"] = {"MINING",}, -- Mining enchant ID:844
			["Herbalism"] = {"HERBALISM",}, -- Heabalism enchant ID:845
			["Skinning"] = {"SKINNING",}, -- Skinning enchant ID:865
	
			["Armor"] = {"ARMOR",},
			["Defense"] = {"DEFENSE",},
			["Increased Defense"] = {"DEFENSE",},
			["Block"] = {"BLOCK_VALUE",}, -- +22 Block Value
			["Block Value"] = {"BLOCK_VALUE",}, -- +22 Block Value
			["Increases the block value of your shield"] = {"BLOCK_VALUE",},
	
			["Health"] = {"HEALTH",},
			["HP"] = {"HEALTH",},
			["Mana"] = {"MANA",},
	
			["Attack Power"] = {"AP",},
			["Increases attack power"] = {"AP",},
			["Attack Power when fighting Undead"] = {"AP_UNDEAD",},
			["Increases attack powerwhen fighting Undead"] = {"AP_UNDEAD",},
			["Increases attack powerwhen fighting Undead.  It also allows the acquisition of Scourgestones on behalf of the Argent Dawn"] = {"AP_UNDEAD",},
			["Increases attack powerwhen fighting Demons"] = {"AP_DEMON",},
			["Attack Power in Cat, Bear, and Dire Bear forms only"] = {"FERAL_AP",},
			["Increases attack powerin Cat, Bear, Dire Bear, and Moonkin forms only"] = {"FERAL_AP",},
			["Ranged Attack Power"] = {"RANGED_AP",},
			["Increases ranged attack power"] = {"RANGED_AP",}, -- [High Warlord's Crossbow] ID: 18837
	
			["Health Regen"] = {"MANA_REG",},
			["Health per"] = {"HEALTH_REG",},
			["health per"] = {"HEALTH_REG",}, -- Frostwolf Insignia Rank 6 ID:17909
			["Health every"] = {"MANA_REG",},
			["health every"] = {"HEALTH_REG",}, -- [Resurgence Rod] ID:17743
			["your normal health regeneration"] = {"HEALTH_REG",}, -- Demons Blood ID: 10779
			["Restoreshealth per 5 sec"] = {"HEALTH_REG",}, -- [Onyxia Blood Talisman] ID: 18406
			["Restoreshealth every 5 sec"] = {"HEALTH_REG",}, -- [Resurgence Rod] ID:17743
			["Mana Regen"] = {"MANA_REG",},
			["Mana per"] = {"MANA_REG",},
			["mana per"] = {"MANA_REG",}, -- Resurgence Rod ID:17743 Most common
			["Mana every"] = {"MANA_REG",},
			["mana every"] = {"MANA_REG",},
			["Mana every 5 seconds"] = {"MANA_REG",}, -- [Royal Nightseye] ID: 24057
			["Mana every 5 Sec"] = {"MANA_REG",}, -- 
			["mana every 5 sec"] = {"MANA_REG",}, -- 
			["Mana per 5 Seconds"] = {"MANA_REG",}, -- [Royal Shadow Draenite] ID: 23109
			["Mana Per 5 sec"] = {"MANA_REG",}, -- [Royal Shadow Draenite] ID: 23109
			["Mana per 5 sec"] = {"MANA_REG",}, -- [Cyclone Shoulderpads] ID: 29031
			["mana per 5 sec"] = {"MANA_REG",}, -- [Royal Tanzanite] ID: 30603
			["Restoresmana per 5 sec"] = {"MANA_REG",}, -- [Resurgence Rod] ID:17743
	
			["Spell Penetration"] = {"SPELLPEN",},
			["Increases your spell penetration"] = {"SPELLPEN",},
			["Healing and Spell Damage"] = {"SPELL_DMG", "HEAL",},
			["Damage and Healing Spells"] = {"SPELL_DMG", "HEAL",},
			["Spell Damage and Healing"] = {"SPELL_DMG", "HEAL",},
			["Spell Damage"] = {"SPELL_DMG", "HEAL",},
			["Increases damage and healing done by magical spells and effects"] = {"SPELL_DMG", "HEAL"},
			["Increases damage and healing done by magical spells and effects of all party members within 30 yards"] = {"SPELL_DMG", "HEAL"}, -- Atiesh
			["Spell Damage and Healing"] = {"SPELL_DMG", "HEAL",},
			--StatLogic:GetSum("item:22630")
			["Damage"] = {"SPELL_DMG",},
			["Increases your spell damage"] = {"SPELL_DMG",}, -- Atiesh ID:22630, 22631, 22632, 22589
			["Spell Power"] = {"SPELL_DMG",},
			["Holy Damage"] = {"HOLY_SPELL_DMG",},
			["Arcane Damage"] = {"ARCANE_SPELL_DMG",},
			["Fire Damage"] = {"FIRE_SPELL_DMG",},
			["Nature Damage"] = {"NATURE_SPELL_DMG",},
			["Frost Damage"] = {"FROST_SPELL_DMG",},
			["Shadow Damage"] = {"SHADOW_SPELL_DMG",},
			["Holy Spell Damage"] = {"HOLY_SPELL_DMG",},
			["Arcane Spell Damage"] = {"ARCANE_SPELL_DMG",},
			["Fire Spell Damage"] = {"FIRE_SPELL_DMG",},
			["Nature Spell Damage"] = {"NATURE_SPELL_DMG",},
			["Frost Spell Damage"] = {"FROST_SPELL_DMG",}, -- Acrobatic Staff of Frozen Wrath ID:3185:0:0:0:0:0:1957
			["Shadow Spell Damage"] = {"SHADOW_SPELL_DMG",},
			["Increases damage done by Arcane spells and effects"] = {"ARCANE_SPELL_DMG",},
			["Increases damage done by Fire spells and effects"] = {"FIRE_SPELL_DMG",},
			["Increases damage done by Frost spells and effects"] = {"FROST_SPELL_DMG",}, -- Frozen Shadoweave Vest ID:21871
			["Increases damage done by Holy spells and effects"] = {"HOLY_SPELL_DMG",},
			["Increases damage done by Nature spells and effects"] = {"NATURE_SPELL_DMG",},
			["Increases damage done by Shadow spells and effects"] = {"SHADOW_SPELL_DMG",}, -- Frozen Shadoweave Vest ID:21871
			["Increases damage done to Undead by magical spells and effects"] = {"SPELL_DMG_UNDEAD"},
			["Increases damage done to Undead by magical spells and effects.  It also allows the acquisition of Scourgestones on behalf of the Argent Dawn"] = {"SPELL_DMG_UNDEAD"},
	
			["Healing Spells"] = {"HEAL",},
			["Increases Healing"] = {"HEAL",},
			["Healing"] = {"HEAL",},
			["healing Spells"] = {"HEAL",},
			["Healing Spells"] = {"HEAL",}, -- [Royal Nightseye] ID: 24057
			["Increases healing done by spells and effects"] = {"HEAL",},
			["Increases healing done by magical spells and effects of all party members within 30 yards"] = {"HEAL",}, -- Atiesh
			["your healing"] = {"HEAL",}, -- Atiesh
	
			["damage per second"] = {"DPS",},
			["Addsdamage per second"] = {"DPS",}, -- [Thorium Shells] ID: 15977
	
			--[[ Rating ID
			CR_WEAPON_SKILL = 1;
			CR_DEFENSE_SKILL = 2;
			CR_DODGE = 3;
			CR_PARRY = 4;
			CR_BLOCK = 5;
			CR_HIT_MELEE = 6;
			CR_HIT_RANGED = 7;
			CR_HIT_SPELL = 8;
			CR_CRIT_MELEE = 9;
			CR_CRIT_RANGED = 10;
			CR_CRIT_SPELL = 11;
			CR_HIT_TAKEN_MELEE = 12;
			CR_HIT_TAKEN_RANGED = 13;
			CR_HIT_TAKEN_SPELL = 14;
			CR_CRIT_TAKEN_MELEE = 15;
			CR_CRIT_TAKEN_RANGED = 16;
			CR_CRIT_TAKEN_SPELL = 17;
			CR_HASTE_MELEE = 18;
			CR_HASTE_RANGED = 19;
			CR_HASTE_SPELL = 20;
			CR_WEAPON_SKILL_MAINHAND = 21;
			CR_WEAPON_SKILL_OFFHAND = 22;
			CR_WEAPON_SKILL_RANGED = 23;
			--]]
			["Defense Rating"] = {"CR_DEFENSE",},
			["Increases defense rating"] = {"CR_DEFENSE",},
			["Dodge Rating"] = {"CR_DODGE",},
			["Increases your dodge rating"] = {"CR_DODGE",},
			["Parry Rating"] = {"CR_PARRY",},
			["Increases your parry rating"] = {"CR_PARRY",},
			["Block Rating"] = {"CR_BLOCK",},
			["Increases your block rating"] = {"CR_BLOCK",},
			["Increases your shield block rating"] = {"CR_BLOCK",},
	
			["Hit Rating"] = {"CR_HIT_MELEE",},
			["Improves hit rating"] = {"CR_HIT_MELEE",}, -- ITEM_MOD_HIT_RATING
			["Improves melee hit rating"] = {"CR_HIT_MELEE",}, -- ITEM_MOD_HIT_MELEE_RATING
			["Increases your hit rating"] = {"CR_HIT_MELEE",},
			["Spell Hit Rating"] = {"CR_HIT_SPELL",},
			["Improves spell hit rating"] = {"CR_HIT_SPELL",}, -- ITEM_MOD_HIT_SPELL_RATING
			["Increases your spell hit rating"] = {"CR_HIT_SPELL",},
			["Ranged Hit Rating"] = {"CR_HIT_RANGED",},
			["Improves ranged hit rating"] = {"CR_HIT_RANGED",}, -- ITEM_MOD_HIT_RANGED_RATING
			["Increases your ranged hit rating"] = {"CR_HIT_RANGED",},
	
			["Crit Rating"] = {"CR_CRIT_MELEE",},
			["Critical Rating"] = {"CR_CRIT_MELEE",},
			["Critical Strike Rating"] = {"CR_CRIT_MELEE",},
			["Increases your critical hit rating"] = {"CR_CRIT_MELEE",},
			["Increases your critical strike rating"] = {"CR_CRIT_MELEE",},
			["Improves critical strike rating"] = {"CR_CRIT_MELEE",},
			["Spell Critical Strike Rating"] = {"CR_CRIT_SPELL",},
			["Spell Critical strike rating"] = {"CR_CRIT_SPELL",},
			["Spell Critical Rating"] = {"CR_CRIT_SPELL",},
			["Spell Crit Rating"] = {"CR_CRIT_SPELL",},
			["Increases your spell critical strike rating"] = {"CR_CRIT_SPELL",},
			["Increases the spell critical strike rating of all party members within 30 yards"] = {"CR_CRIT_SPELL",},
			["Improves spell critical strike rating"] = {"CR_CRIT_SPELL",},
			["Increases your ranged critical strike rating"] = {"CR_CRIT_RANGED",}, -- Fletcher's Gloves ID:7348
	
			["Improves hit avoidance rating"] = {"CR_HIT_TAKEN_MELEE"}, -- ITEM_MOD_HIT_TAKEN_RATING
			["Improves melee hit avoidance rating"] = {"CR_HIT_TAKEN_MELEE"}, -- ITEM_MOD_HIT_TAKEN_MELEE_RATING
			["Improves ranged hit avoidance rating"] = {"CR_HIT_TAKEN_RANGED"}, -- ITEM_MOD_HIT_TAKEN_RANGED_RATING
			["Improves spell hit avoidance rating"] = {"CR_HIT_TAKEN_SPELL"}, -- ITEM_MOD_HIT_TAKEN_SPELL_RATING
			["Resilience"] = {"CR_RESILIENCE",},
			["Resilience Rating"] = {"CR_RESILIENCE",},
			["Improves your resilience rating"] = {"CR_RESILIENCE",},
			["Improves critical avoidance rating"] = {"CR_CRIT_TAKEN_MELEE",},
			["Improves melee critical avoidance rating"] = {"CR_CRIT_TAKEN_MELEE",},
			["Improves ranged critical avoidance rating"] = {"CR_CRIT_TAKEN_RANGED",},
			["Improves spell critical avoidance rating"] = {"CR_CRIT_TAKEN_SPELL",},
	
			["Haste Rating"] = {"CR_HASTE_MELEE"},
			["Spell Haste Rating"] = {"CR_HASTE_SPELL"},
			["Ranged Haste Rating"] = {"CR_HASTE_RANGED"},
			["Improves haste rating"] = {"CR_HASTE_MELEE"},
			["Improves melee haste rating"] = {"CR_HASTE_MELEE"},
			["Improves ranged haste rating"] = {"CR_HASTE_SPELL"},
			["Improves spell haste rating"] = {"CR_HASTE_RANGED"},
	
			["Increases dagger skill rating"] = {"CR_WEAPON_DAGGER"},
			["Increases sword skill rating"] = {"CR_WEAPON_SWORD"},
			["Increases Two-Handed Swords skill rating"] = {"CR_WEAPON_SWORD_2H"},
			["Increases axe skill rating"] = {"CR_WEAPON_AXE"},
			["Two-Handed Axe Skill Rating"] = {"CR_WEAPON_AXE_2H"},
			["Increases two-handed axes skill rating"] = {"CR_WEAPON_AXE_2H"},
			["Increases mace skill rating"] = {"CR_WEAPON_MACE"},
			["Increases two-handed maces skill rating"] = {"CR_WEAPON_MACE_2H"},
			["Increases gun skill rating"] = {"CR_WEAPON_GUN"},
			["Increases Crossbow skill rating"] = {"CR_WEAPON_CROSSBOW"},
			["Increases Bow skill rating"] = {"CR_WEAPON_BOW"},
			["Increases feral combat skill rating"] = {"CR_WEAPON_FERAL"},
			["Increases fist weapons skill rating"] = {"CR_WEAPON_FIST"}, -- Demonblood Eviscerator
			["Increases unarmed skill rating"] = {"CR_WEAPON_FIST"}, -- Demonblood Eviscerator ID:27533
			-- Exclude
			["sec"] = false,
			["to"] = false,
			["Slot Bag"] = false,
			["Slot Quiver"] = false,
			["Slot Ammo Pouch"] = false,
			["Increases ranged attack speed"] = false, -- AV quiver
		},
		----------------
		-- Stat Names --
		----------------
		["StatIDToName"] = {
			--[StatID] = {FullName, ShortName},
			["STEALTH_LEVEL"] = {"Stealth Level", "Stealth"},
			["MELEE_DMG"] = {"Melee Weapon Damage", "Weapon Damage"},
			["MOUNT_SPEED%"] = {"Mount Speed(%)", "Mount Speed(%)"},
			["RUN_SPEED%"] = {"Run Speed(%)", "Run Speed(%)"},
	
			["STR"] = {SPELL_STAT1_NAME, "Str"},
			["AGI"] = {SPELL_STAT2_NAME, "Agi"},
			["STA"] = {SPELL_STAT3_NAME, "Sta"},
			["INT"] = {SPELL_STAT4_NAME, "Int"},
			["SPI"] = {SPELL_STAT5_NAME, "Spi"},
			["ARMOR"] = {ARMOR.." from items", "ARMOR(Items)"},
			["ARMOR_BONUS"] = {ARMOR.." from bonus", "ARMOR(Bonus)"},
	
			["FIRE_RES"] = {RESISTANCE2_NAME, "FR"},
			["NATURE_RES"] = {RESISTANCE3_NAME, "NR"},
			["FROST_RES"] = {RESISTANCE4_NAME, "FrR"},
			["SHADOW_RES"] = {RESISTANCE5_NAME, "SR"},
			["ARCANE_RES"] = {RESISTANCE6_NAME, "AR"},
	
			["FISHING"] = {"Fishing", "Fishing"},
			["MINING"] = {"Mining", "Mining"},
			["HERBALISM"] = {"Herbalism", "Herbalism"},
			["SKINNING"] = {"Skinning", "Skinning"},
	
			["BLOCK_VALUE"] = {"Block Value", "Block Value"},

			["AP"] = {ATTACK_POWER_TOOLTIP, "AP"},
			["RANGED_AP"] = {RANGED_ATTACK_POWER, "RAP"},
			["FERAL_AP"] = {ATTACK_POWER_TOOLTIP.." (Feral)", "AP(Feral)"},
			["AP_UNDEAD"] = {ATTACK_POWER_TOOLTIP.." (Undead)", "AP(Undead)"},
			["AP_DEMON"] = {ATTACK_POWER_TOOLTIP.." (Demon)", "AP(Demon)"},
	
			["HEAL"] = {"Healing", "Heal"},
	
			["SPELL_DMG"] = {PLAYERSTAT_SPELL_COMBAT.." "..DAMAGE, PLAYERSTAT_SPELL_COMBAT.." Dmg"},
			["SPELL_DMG_UNDEAD"] = {PLAYERSTAT_SPELL_COMBAT.." "..DAMAGE.." (Undead)", PLAYERSTAT_SPELL_COMBAT.." Dmg".." (Undead)"},
			["SPELL_DMG_DEMON"] = {PLAYERSTAT_SPELL_COMBAT.." "..DAMAGE.." (Demon)", PLAYERSTAT_SPELL_COMBAT.." Dmg".." (Demon)"},
			["HOLY_SPELL_DMG"] = {SPELL_SCHOOL1_CAP.." "..DAMAGE, SPELL_SCHOOL1_CAP.." Dmg"},
			["FIRE_SPELL_DMG"] = {SPELL_SCHOOL2_CAP.." "..DAMAGE, SPELL_SCHOOL2_CAP.." Dmg"},
			["NATURE_SPELL_DMG"] = {SPELL_SCHOOL3_CAP.." "..DAMAGE, SPELL_SCHOOL3_CAP.." Dmg"},
			["FROST_SPELL_DMG"] = {SPELL_SCHOOL4_CAP.." "..DAMAGE, SPELL_SCHOOL4_CAP.." Dmg"},
			["SHADOW_SPELL_DMG"] = {SPELL_SCHOOL5_CAP.." "..DAMAGE, SPELL_SCHOOL5_CAP.." Dmg"},
			["ARCANE_SPELL_DMG"] = {SPELL_SCHOOL6_CAP.." "..DAMAGE, SPELL_SCHOOL6_CAP.." Dmg"},
	
			["SPELLPEN"] = {PLAYERSTAT_SPELL_COMBAT.." "..SPELL_PENETRATION, SPELL_PENETRATION},
	
			["HEALTH_REG"] = {HEALTH.." Regen", "HP5"},
			["MANA_REG"] = {MANA.." Regen", "MP5"},
			["HEALTH"] = {HEALTH, HP},
			["MANA"] = {MANA, MP},
	
			["MAX_DAMAGE"] = {"Max Damage", "Max Dmg"},
			["DPS"] = {"Damage Per Second", "DPS"},
	
			["CR_DEFENSE"] = {COMBAT_RATING_NAME2, COMBAT_RATING_NAME2}, -- COMBAT_RATING_NAME2 = "Defense Rating"
			["CR_DODGE"] = {COMBAT_RATING_NAME3, COMBAT_RATING_NAME3}, -- COMBAT_RATING_NAME3 = "Dodge Rating"
			["CR_PARRY"] = {COMBAT_RATING_NAME4, COMBAT_RATING_NAME4}, -- COMBAT_RATING_NAME4 = "Parry Rating"
			["CR_BLOCK"] = {COMBAT_RATING_NAME5, COMBAT_RATING_NAME5}, -- COMBAT_RATING_NAME5 = "Block Rating"
			["CR_HIT_MELEE"] = {COMBAT_RATING_NAME6, COMBAT_RATING_NAME6}, -- COMBAT_RATING_NAME6 = "Hit Rating"
			["CR_HIT_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." "..COMBAT_RATING_NAME6, PLAYERSTAT_RANGED_COMBAT.." "..COMBAT_RATING_NAME6}, -- PLAYERSTAT_RANGED_COMBAT = "Ranged"
			["CR_HIT_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." "..COMBAT_RATING_NAME6, PLAYERSTAT_SPELL_COMBAT.." "..COMBAT_RATING_NAME6}, -- PLAYERSTAT_SPELL_COMBAT = "Spell"
			["CR_HIT_TAKEN_MELEE"] = {"Hit Avoidance "..RATING, "Hit Avoidance "..RATING},
			["CR_HIT_TAKEN_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Hit Avoidance "..RATING, PLAYERSTAT_RANGED_COMBAT.." Hit Avoidance "..RATING},
			["CR_HIT_TAKEN_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Hit Avoidance "..RATING, PLAYERSTAT_SPELL_COMBAT.." Hit Avoidance "..RATING},
			["CR_CRIT_MELEE"] = {COMBAT_RATING_NAME9, COMBAT_RATING_NAME9}, -- COMBAT_RATING_NAME9 = "Crit Rating"
			["CR_CRIT_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." "..COMBAT_RATING_NAME9, PLAYERSTAT_RANGED_COMBAT.." "..COMBAT_RATING_NAME9},
			["CR_CRIT_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." "..COMBAT_RATING_NAME9, PLAYERSTAT_SPELL_COMBAT.." "..COMBAT_RATING_NAME9},
			["CR_CRIT_TAKEN_MELEE"] = {"Crit Avoidance "..RATING, "Crit Avoidance "..RATING},
			["CR_CRIT_TAKEN_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Crit Avoidance "..RATING, PLAYERSTAT_RANGED_COMBAT.." Crit Avoidance "..RATING},
			["CR_CRIT_TAKEN_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Crit Avoidance "..RATING, PLAYERSTAT_SPELL_COMBAT.." Crit Avoidance "..RATING},
			["CR_RESILIENCE"] = {COMBAT_RATING_NAME15, COMBAT_RATING_NAME15}, -- COMBAT_RATING_NAME15 = "Resilience"
			["CR_HASTE_MELEE"] = {"Haste "..RATING, "Haste "..RATING}, --
			["CR_HASTE_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Haste "..RATING, PLAYERSTAT_RANGED_COMBAT.." Haste "..RATING},
			["CR_HASTE_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Haste "..RATING, PLAYERSTAT_SPELL_COMBAT.." Haste "..RATING},
			["CR_WEAPON_DAGGER"] = {"Dagger "..SKILL.." "..RATING, "Dagger "..RATING}, -- SKILL = "Skill"
			["CR_WEAPON_SWORD"] = {"Sword "..SKILL.." "..RATING, "Sword "..RATING},
			["CR_WEAPON_SWORD_2H"] = {"Two-Handed Sword "..SKILL.." "..RATING, "2H Sword "..RATING},
			["CR_WEAPON_AXE"] = {"Axe "..SKILL.." "..RATING, "Axe "..RATING},
			["CR_WEAPON_AXE_2H"] = {"Two-Handed Axe "..SKILL.." "..RATING, "2H Axe "..RATING},
			["CR_WEAPON_MACE"] = {"Mace "..SKILL.." "..RATING, "Mace "..RATING},
			["CR_WEAPON_MACE_2H"] = {"Two-Handed Mace "..SKILL.." "..RATING, "2H Mace "..RATING},
			["CR_WEAPON_GUN"] = {"Gun "..SKILL.." "..RATING, "Gun "..RATING},
			["CR_WEAPON_CROSSBOW"] = {"Crossbow "..SKILL.." "..RATING, "Crossbow "..RATING},
			["CR_WEAPON_BOW"] = {"Bow "..SKILL.." "..RATING, "Bow "..RATING},
			["CR_WEAPON_FERAL"] = {"Feral "..SKILL.." "..RATING, "Feral "..RATING},
			["CR_WEAPON_FIST"] = {"Unarmed "..SKILL.." "..RATING, "Unarmed "..RATING},
	
			-- Tier2 Stats
			-- Str -> AP, Block Value
			-- Agi -> AP, Crit, Dodge
			-- Sta -> Health
			-- Int -> Mana, Spell Crit
			-- Spi -> mp5oc, hp5oc
			-- Ratings -> Effect
			["DEFENSE"] = {DEFENSE, "Def"},
			["DODGE"] = {DODGE_CHANCE, DODGE},
			["PARRY"] = {PARRY_CHANCE, PARRY},
			["BLOCK"] = {BLOCK_CHANCE, BLOCK},
			["HIT_MELEE"] = {"Hit Chance", "Hit"},
			["HIT_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." ".."Hit Chance", PLAYERSTAT_RANGED_COMBAT.." Hit"},
			["HIT_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." ".."Hit Chance", PLAYERSTAT_SPELL_COMBAT.." Hit"},
			["HIT_TAKEN_MELEE"] = {"Hit Avoidance", "Hit Avoidance"},
			["HIT_TAKEN_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Hit Avoidance", PLAYERSTAT_RANGED_COMBAT.." Hit Avoidance"},
			["HIT_TAKEN_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Hit Avoidance", PLAYERSTAT_SPELL_COMBAT.." Hit Avoidance"},
			["CRIT_MELEE"] = {MELEE_CRIT_CHANCE, "Crit"}, -- MELEE_CRIT_CHANCE = "Crit Chance"
			["CRIT_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." "..MELEE_CRIT_CHANCE, PLAYERSTAT_RANGED_COMBAT.." Crit"},
			["CRIT_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." "..MELEE_CRIT_CHANCE, PLAYERSTAT_SPELL_COMBAT.." Crit"},
			["CRIT_TAKEN_MELEE"] = {"Crit Avoidance", "Crit Avoidance"},
			["CRIT_TAKEN_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Crit Avoidance", PLAYERSTAT_RANGED_COMBAT.." Crit Avoidance"},
			["CRIT_TAKEN_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Crit Avoidance", PLAYERSTAT_SPELL_COMBAT.." Crit Avoidance"},
			["HASTE_MELEE"] = {"Haste", "Haste"}, --
			["HASTE_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Haste", PLAYERSTAT_RANGED_COMBAT.." Haste"},
			["HASTE_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Haste", PLAYERSTAT_SPELL_COMBAT.." Haste"},
			["WEAPON_DAGGER"] = {"Dagger "..SKILL, "Dagger"}, -- SKILL = "Skill"
			["WEAPON_SWORD"] = {"Sword "..SKILL, "Sword"},
			["WEAPON_SWORD_2H"] = {"Two-Handed Sword "..SKILL, "2H Sword"},
			["WEAPON_AXE"] = {"Axe "..SKILL, "Axe"},
			["WEAPON_AXE_2H"] = {"Two-Handed Axe "..SKILL, "2H Axe"},
			["WEAPON_MACE"] = {"Mace "..SKILL, "Mace"},
			["WEAPON_MACE_2H"] = {"Two-Handed Mace "..SKILL, "2H Mace"},
			["WEAPON_GUN"] = {"Gun "..SKILL, "Gun"},
			["WEAPON_CROSSBOW"] = {"Crossbow "..SKILL, "Crossbow"},
			["WEAPON_BOW"] = {"Bow "..SKILL, "Bow"},
			["WEAPON_FERAL"] = {"Feral "..SKILL, "Feral"},
			["WEAPON_FIST"] = {"Unarmed "..SKILL, "Unarmed"},
			["CRIT_DAMAGE_TAKEN_MELEE"] = {"Crit Damage Taken", "Crit Damage Taken"},
			["CRIT_DAMAGE_TAKEN_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Crit Damage Taken", PLAYERSTAT_RANGED_COMBAT.." Crit Damage Taken"},
			["CRIT_DAMAGE_TAKEN_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Crit Damage Taken", PLAYERSTAT_SPELL_COMBAT.." Crit Damage Taken"},
	
			-- Tier3 Stats
			-- Defense -> Crit Avoidance, Hit Avoidance, Dodge, Parry, Block
			-- Weapon Skill -> Crit, Hit, Mob Dodge, Mob Parry, Mob Block
			--
		},
	}
end)

-- koKR localization by fenlis
L:RegisterTranslations("koKR", function()
	return {
		["Locale"] = "koKR",
		["tonumber"] = tonumber,
		--["Dual Wield"] = "쌍수 무기",
		-------------------
		-- Exclude Table --
		-------------------
		-- By looking at the first ExcludeLen letters of a line we can exclude a lot of lines
		["ExcludeLen"] = 3,
		["Exclude"] = {
			[""] = true,
			[" \n"] = true,
			["착용 시 귀속"] = true,
			["획득 시 귀속"] = true,
			["사용 시 귀속"] = true,
			["붉은색 보석 홈"] = true,
			["노란색 보석 홈"] = true,
			["푸른색 보석 홈"] = true,
			["보라색 보석 홈"] = true,
			["퀘스트 아이템"] = true,
			["퀘스트 시작 아이템"] = true,
			["고유 "] = true, -- Unique (20)
			["최소 "] = true, -- Requires Level xx
			["\n최소"] = true, -- Requires Level xx
			["직업:"] = true, -- Classes: xx
			["종족:"] = true, -- Races: xx (vendor mounts)
			["사용 "] = true, -- Use:
			["발동 "] = true, -- Chance On Hit: 
			-- Set Bonuses
			["세트 "] = true,
			["(2)"] = true,
			["(3)"] = true,
			["(4)"] = true,
			["(5)"] = true,
			["(6)"] = true,
			["(7)"] = true,
			["(8)"] = true,
			-- Equip type
			["투사체"] = true, -- Ice Threaded Arrow ID:19316
			[INVTYPE_AMMO] = true,
			[INVTYPE_HEAD] = true,
			[INVTYPE_NECK] = true,
			[INVTYPE_SHOULDER] = true,
			[INVTYPE_BODY] = true,
			[INVTYPE_CHEST] = true,
			[INVTYPE_ROBE] = true,
			[INVTYPE_WAIST] = true,
			[INVTYPE_LEGS] = true,
			[INVTYPE_FEET] = true,
			[INVTYPE_WRIST] = true,
			[INVTYPE_HAND] = true,
			[INVTYPE_FINGER] = true,
			[INVTYPE_TRINKET] = true,
			[INVTYPE_CLOAK] = true,
			[INVTYPE_WEAPON] = true,
			[INVTYPE_SHIELD] = true,
			[INVTYPE_2HWEAPON] = true,
			[INVTYPE_WEAPONMAINHAND] = true,
			[INVTYPE_WEAPONOFFHAND] = true,
			[INVTYPE_HOLDABLE] = true,
			[INVTYPE_RANGED] = true,
			[INVTYPE_THROWN] = true,
			[INVTYPE_RANGEDRIGHT] = true,
			[INVTYPE_RELIC] = true,
			[INVTYPE_TABARD] = true,
			[INVTYPE_BAG] = true,
		},
		--[[
		textTable = {
			"Spell Damage +6 and Spell Hit Rating +5",
			"+3 Stamina, +4 Critical Strike Rating",
			"+26 Healing Spells & 2% Reduced Threat",
			"+3 Stamina/+4 Critical Strike Rating",
			"Socket Bonus: 2 mana per 5 sec.",
			"Equip: Increases damage and healing done by magical spells and effects by up to 150.",
			"Equip: Increases the spell critical strike rating of all party members within 30 yards by 28.",
			"Equip: Increases damage and healing done by magical spells and effects of all party members within 30 yards by up to 33.",
			"Equip: Increases healing done by magical spells and effects of all party members within 30 yards by up to 62.",
			"Equip: Increases your spell damage by up to 120 and your healing by up to 300.",
			"Equip: Restores 11 mana per 5 seconds to all party members within 30 yards.",
			"Equip: Increases healing done by spells and effects by up to 300.",
			"Equip: Increases attack power by 420 in Cat, Bear, Dire Bear, and Moonkin forms only.",
			-- "+10 Defense Rating/+10 Stamina/+15 Block Value": ZG Enchant
			-- "+26 Attack Power and +14 Critical Strike Rating": Swift Windfire Diamond ID:28556
			-- "+26 Healing Spells & 2% Reduced Threat": Bracing Earthstorm Diamond ID:25897
			-- "+6 Spell Damage, +5 Spell Crit Rating": Potent Ornate Topaz ID: 28123
			----
			-- "Critical Rating +6 and Dodge Rating +5": Assassin's Fire Opal ID:30565
			-- "Healing +11 and 2 mana per 5 sec.": Royal Tanzanite ID: 30603
		}
		--]]
		-----------------------
		-- Whole Text Lookup --
		-----------------------
		-- Mainly used for enchants that doesn't have numbers in the text
		["WholeTextLookup"] = {
			["Savagery"] = {["AP"] = 70}, -- Savagery
	
			["최하급 마술사 오일"] = {["SPELL_DMG"] = 8, ["HEAL"] = 8}, -- 
			["하급 마나 오일"] = {["SPELL_DMG"] = 16, ["HEAL"] = 16}, -- 
			["마술사 오일"] = {["SPELL_DMG"] = 24, ["HEAL"] = 24}, -- 
			["반짝이는 마술사 오일"] = {["SPELL_DMG"] = 36, ["HEAL"] = 36, ["CR_CRIT_SPELL"] = 14}, -- 
			
			["상급 마나 오일"] = {["MANA_REG"] = 14}, -- 
			["최하급 마나 오일"] = {["MANA_REG"] = 4}, -- 
			["하급 마나 오일"] = {["MANA_REG"] = 8}, -- 
			["반짝이는 마나 오일"] = {["MANA_REG"] = 12, ["HEAL"] = 25}, -- 
			["상급 마술사 오일"] = {["SPELL_DMG"] = 42, ["HEAL"] = 42}, -- 
			
			["에터니움 낚시줄"] = {["FISHING"] = 5}, -- 
			["Vitality"] = {["MANA_REG"] = 4, ["HEALTH_REG"] = 4}, -- 
			["Soulfrost"] = {["SHADOW_SPELL_DMG"] = 54, ["FROST_SPELL_DMG"] = 54}, -- 
			["Sunfire"] = {["ARCANE_SPELL_DMG"] = 50, ["FIRE_SPELL_DMG"] = 50}, -- 
			
			["미스릴 박차"] = {["MOUNT_SPEED%"] = 4}, -- Mithril Spurs
			["최하급 탈것 속도 증가"] = {["MOUNT_SPEED%"] = 2}, -- Enchant Gloves - Riding Skill
			["착용 효과: 이동 속도가 약간 증가합니다."] = {["RUN_SPEED%"] = 8}, -- [Highlander's Plate Greaves] ID: 20048
			["이동 속도가 약간 증가합니다."] = {["RUN_SPEED%"] = 8}, -- 
			["탈것 속도 증가"] = {["RUN_SPEED%"] = 8}, -- Enchant Boots - Minor Speed
			["Equip: Allows underwater breathing."] = false, -- [Band of Icy Depths] ID: 21526
			["Allows underwater breathing"] = false, --
			["착용 효과: 무장해제에 면역이 됩니다."] = false, -- [Stronghold Gauntlets] ID: 12639
			["무장해제에 면역이 됩니다"] = false, -- 
			["성전사"] = false, -- Enchant Crusader
			["흡혈"] = false, -- Enchant Crusader
		},
		----------------------------
		-- Single Plus Stat Check --
		----------------------------
		-- depending on locale, it may be
		-- +19 Stamina = "^%+(%d+) ([%a ]+%a)$"
		-- Stamina +19 = "^([%a ]+%a) %+(%d+)$"
		-- +19 ұ = "^%+(%d+) (.-)$"
		--["SinglePlusStatCheck"] = "^%+(%d+) ([%a ]+%a)$",
		["SinglePlusStatCheck"] = "^(.-) %+(%d+)$",
		-----------------------------
		-- Single Equip Stat Check --
		-----------------------------
		-- stat1, value, stat2 = strfind
		-- stat = stat1..stat2
		-- "^Equip: (.-) by u?p? ?t?o? ?(%d+) ?(.-)%.$"
		["SingleEquipStatCheck"] = "^착용 효과: (.-) (%d+)만큼(.-)$",
		-------------
		-- PreScan --
		-------------
		-- Special cases that need to be dealt with before base scan
		["PreScanPatterns"] = {
			--["^Equip: Increases attack power by (%d+) in Cat"] = "FERAL_AP",
			--["^Equip: Increases attack power by (%d+) when fighting Undead"] = "AP_UNDEAD", -- Seal of the Dawn ID:13029
			["^(%d+)의 피해 방어$"] = "BLOCK_VALUE",
			["^방어도 (%d+)$"] = "ARMOR",
			["방어도 보강 %(%+(%d+)%)"] = "ARMOR_BONUS",
			["매 5초마다 (%d+)의 마나가 회복됩니다.$"] = "MANA_REG",
			["^.-공격력 %+?%d+ %- (%d+)$"] = "MAX_DAMAGE",
			["^%(초당 공격력 ([%d%.]+)%)$"] = "DPS",
			-- Exclude
			["^(%d+)칸"] = false, -- Set Name (0/9)
			["^[%D ]+ %((%d+)/%d+%)$"] = false, -- Set Name (0/9)
			["|cff808080"] = false, -- Gray text "  |cff808080Requires at least 2 Yellow gems|r\n  |cff808080Requires at least 1 Red gem|r"
			-- Procs
			["발동"] = false, -- [Mark of Defiance] ID:27924 -- [Staff of the Qiraji Prophets] ID:21128
			["확률로"] = false, -- [Darkmoon Card: Heroism] ID:19287
			["[Ww]hen struck in combat"] = false, -- [Essence of the Pure Flame] ID: 18815
			["성공하면"] = false,
		},
		--------------
		-- DeepScan --
		--------------
		["Equip: "] = "착용 효과: ",
		["DeepScanSeparators"] = {
			" and ", -- "Critical Rating +6 and Dodge Rating +5": Assassin's Fire Opal ID:30565
			"/", -- "+10 Defense Rating/+10 Stamina/+15 Block Value": ZG Enchant
			" & ", -- "+26 Healing Spells & 2% Reduced Threat": Bracing Earthstorm Diamond ID:25897
			", ", -- "+6 Spell Damage, +5 Spell Crit Rating": Potent Ornate Topaz ID: 28123
			"%. ", -- "Equip: Increases attack power by 81 when fighting Undead. It also allows the acquisition of Scourgestones on behalf of the Argent Dawn.": Seal of the Dawn
		},
		["DeepScanPatterns"] = {
			"^(.-) (%d+)만큼(.-)$", -- "xxx by up to 22 xxx" (scan first)
			"^(.-) 최대 (%d+)만큼(.-)$", -- "xxx by up to 22 xxx" (scan first)
			"^(.-) ?%+(%d+) ?(.-)$", -- "xxx xxx +22" or "+22 xxx xxx" or "xxx +22 xxx" (scan 2ed)
			"^(.-) ?([%d%.]+) ?(.-)$", -- 22.22 xxx xxx (scan last)
		},
		-----------------------
		-- Stat Lookup Table --
		-----------------------
		["StatIDLookup"] = {
	
			["은신 효과가 증가합니다"] = {"STEALTH_LEVEL"}, -- [Nightscape Boots] ID: 8197
			["무기 공격력"] = {"MELEE_DMG"}, -- Enchant
			["탈것의 속도가%만큼 증가합니다"] = {"MOUNT_SPEED%"}, -- [Highlander's Plate Greaves] ID: 20048
	
			["모든 능력치"] = {"STR", "AGI", "STA", "INT", "SPI",},
			["힘"] = {"STR",},
			["민첩성"] = {"AGI",},
			["체력"] = {"STA",},
			["지능"] = {"INT",},
			["정신력"] = {"SPI",},
	
			["비전 저항력"] = {"ARCANE_RES",},
			["화염 저항력"] = {"FIRE_RES",},
			["자연 저항력"] = {"NATURE_RES",},
			["냉기 저항력"] = {"FROST_RES",},
			["암흑 저항력"] = {"SHADOW_RES",},
			["암흑 저항력"] = {"SHADOW_RES",}, -- Demons Blood ID: 10779
			["모든 저항력"] = {"ARCANE_RES", "FIRE_RES", "FROST_RES", "NATURE_RES", "SHADOW_RES",},
			["모든 저항"] = {"ARCANE_RES", "FIRE_RES", "FROST_RES", "NATURE_RES", "SHADOW_RES",},
	
			["낚시"] = {"FISHING",}, -- Fishing enchant ID:846
			["낚시 숙련도"] = {"FISHING",}, -- Fishing lure
			["낚시 숙련도가 증가합니다"] = {"FISHING",}, -- Equip: Increased Fishing +20.
			["채광"] = {"MINING",}, -- Mining enchant ID:844
			["약초 채집"] = {"HERBALISM",}, -- Heabalism enchant ID:845
			["무두질"] = {"SKINNING",}, -- Skinning enchant ID:865
	
			["방어도"] = {"ARMOR",},
			["방어 숙련"] = {"DEFENSE",},
			["방어 숙련 증가"] = {"DEFENSE",},
			["피해 방어"] = {"BLOCK_VALUE",}, -- +22 Block Value
			["피해 방어량"] = {"BLOCK_VALUE",}, -- +22 Block Value
			["방패의 피해 방어량이 증가합니다"] = {"BLOCK_VALUE",},
	
			["생명력"] = {"HEALTH",},
			["HP"] = {"HEALTH",},
			["마나"] = {"MANA",},
	
			["전투력"] = {"AP",},
			["전투력이 증가합니다"] = {"AP",},
			["언데드 공격 시 전투력"] = {"AP_UNDEAD",},
			["언데드 공격 시 전투력이 증가합니다"] = {"AP_UNDEAD",},
			["Increases attack powerwhen fighting Undead.  It also allows the acquisition of Scourgestones on behalf of the Argent Dawn"] = {"AP_UNDEAD",},
			["악마에 대한 전투력이 증가합니다"] = {"AP_DEMON",},
			["달빛야수 변신 상태일 때 전투력"] = {"FERAL_AP",},
			["달빛야수 변신 상태일 때 전투력이 증가합니다"] = {"FERAL_AP",},
			["원거리 전투력"] = {"RANGED_AP",},
			["원거리 전투력이 증가합니다"] = {"RANGED_AP",}, -- [High Warlord's Crossbow] ID: 18837
	
			["생명력 회복량"] = {"MANA_REG",},
			["매 초마다 (.+)의 생명력"] = {"HEALTH_REG",},
			["health per"] = {"HEALTH_REG",}, -- Frostwolf Insignia Rank 6 ID:17909
			["Health every"] = {"MANA_REG",},
			["health every"] = {"HEALTH_REG",}, -- [Resurgence Rod] ID:17743
			["your normal health regeneration"] = {"HEALTH_REG",}, -- Demons Blood ID: 10779
			["매 5초마다 (.+)의 생명력"] = {"HEALTH_REG",}, -- [Onyxia Blood Talisman] ID: 18406
			["Restoreshealth every 5 sec"] = {"HEALTH_REG",}, -- [Resurgence Rod] ID:17743
			["마나 회복량"] = {"MANA_REG",},
			["매 초마다 (.+)의 마나"] = {"MANA_REG",},
			["mana per"] = {"MANA_REG",}, -- Resurgence Rod ID:17743 Most common
			["Mana every"] = {"MANA_REG",},
			["mana every"] = {"MANA_REG",},
			["매 5초마다 (.+)의 마나"] = {"MANA_REG",}, -- [Royal Nightseye] ID: 24057
			["Mana every 5 Sec"] = {"MANA_REG",}, -- 
			["mana every 5 sec"] = {"MANA_REG",}, -- 
			["Mana per 5 Seconds"] = {"MANA_REG",}, -- [Royal Shadow Draenite] ID: 23109
			["Mana Per 5 sec"] = {"MANA_REG",}, -- [Royal Shadow Draenite] ID: 23109
			["Mana per 5 sec"] = {"MANA_REG",}, -- [Cyclone Shoulderpads] ID: 29031
			["mana per 5 sec"] = {"MANA_REG",}, -- [Royal Tanzanite] ID: 30603
			["Restoresmana per 5 sec"] = {"MANA_REG",}, -- [Resurgence Rod] ID:17743
	
			["주문 관통력"] = {"SPELLPEN",},
			["주문 관통력이 증가합니다"] = {"SPELLPEN",},
			["치유 및 주문 공격력"] = {"SPELL_DMG", "HEAL",},
			["Damage and Healing Spells"] = {"SPELL_DMG", "HEAL",},
			["주문 공격력 및 치유량"] = {"SPELL_DMG", "HEAL",},
			["주문 피해"] = {"SPELL_DMG", "HEAL",},
			["모든 주문 및 효과의 공격력과 치유량이 증가합니다"] = {"SPELL_DMG", "HEAL"},
			["주위 30미터 반경에 있는 모든 파티원의 모든 주문 및 효과의 공격력과 치유량이 증가합니다"] = {"SPELL_DMG", "HEAL"}, -- Atiesh
			["주문 공격력 및 치유량"] = {"SPELL_DMG", "HEAL",},
			--StatLogic:GetSum("item:22630")
			["공격력"] = {"SPELL_DMG",},
			["주문 공격력이 증가합니다"] = {"SPELL_DMG",}, -- Atiesh ID:22630, 22631, 22632, 22589
			["주문 공격력"] = {"SPELL_DMG",},
			["신성 피해"] = {"HOLY_SPELL_DMG",},
			["비전 피해"] = {"ARCANE_SPELL_DMG",},
			["화염 피해"] = {"FIRE_SPELL_DMG",},
			["자연 피해"] = {"NATURE_SPELL_DMG",},
			["냉기 피해"] = {"FROST_SPELL_DMG",},
			["암흑 피해"] = {"SHADOW_SPELL_DMG",},
			["신성 주문 공격력"] = {"HOLY_SPELL_DMG",},
			["비전 주문 공격력"] = {"ARCANE_SPELL_DMG",},
			["화염 주문 공격력"] = {"FIRE_SPELL_DMG",},
			["자연 주문 공격력"] = {"NATURE_SPELL_DMG",},
			["냉기 주문 공격력"] = {"FROST_SPELL_DMG",}, -- Acrobatic Staff of Frozen Wrath ID:3185:0:0:0:0:0:1957
			["암흑 주문 공격력"] = {"SHADOW_SPELL_DMG",},
			["비전 계열의 주문과 효과의 공격력이 증가합니다"] = {"ARCANE_SPELL_DMG",},
			["화염 계열의 주문과 효과의 공격력이 증가합니다"] = {"FIRE_SPELL_DMG",},
			["냉기 계열의 주문과 효과의 공격력이 증가합니다"] = {"FROST_SPELL_DMG",}, -- Frozen Shadoweave Vest ID:21871
			["신성 계열의 주문과 효과의 공격력이 증가합니다"] = {"HOLY_SPELL_DMG",},
			["자연 계열의 주문과 효과의 공격력이 증가합니다"] = {"NATURE_SPELL_DMG",},
			["암흑 계열의 주문과 효과의 공격력이 증가합니다"] = {"SHADOW_SPELL_DMG",}, -- Frozen Shadoweave Vest ID:21871
			["언데드에 대한 효과나 주문에 의한 피해가 증가합니다"] = {"SPELL_DMG_UNDEAD"},
			["Increases damage done to Undead by magical spells and effects.  It also allows the acquisition of Scourgestones on behalf of the Argent Dawn"] = {"SPELL_DMG_UNDEAD"},
	
			["주문 치유량"] = {"HEAL",},
			["치유량 증가"] = {"HEAL",},
			["치유량"] = {"HEAL",},
			["healing Spells"] = {"HEAL",},
			["Healing Spells"] = {"HEAL",}, -- [Royal Nightseye] ID: 24057
			["모든 주문 및 효과에 의한 치유량이 증가합니다"] = {"HEAL",},
			["주위 30미터 반경에 있는 모든 파티원의 모든 주문 및 효과에 의한 치유량이 증가합니다"] = {"HEAL",}, -- Atiesh
			["your healing"] = {"HEAL",}, -- Atiesh
	
			["초당 공격력"] = {"DPS",},
			["초당의 피해 추가"] = {"DPS",}, -- [Thorium Shells] ID: 15977
	
			--[[ Rating ID
			CR_WEAPON_SKILL = 1;
			CR_DEFENSE_SKILL = 2;
			CR_DODGE = 3;
			CR_PARRY = 4;
			CR_BLOCK = 5;
			CR_HIT_MELEE = 6;
			CR_HIT_RANGED = 7;
			CR_HIT_SPELL = 8;
			CR_CRIT_MELEE = 9;
			CR_CRIT_RANGED = 10;
			CR_CRIT_SPELL = 11;
			CR_HIT_TAKEN_MELEE = 12;
			CR_HIT_TAKEN_RANGED = 13;
			CR_HIT_TAKEN_SPELL = 14;
			CR_CRIT_TAKEN_MELEE = 15;
			CR_CRIT_TAKEN_RANGED = 16;
			CR_CRIT_TAKEN_SPELL = 17;
			CR_HASTE_MELEE = 18;
			CR_HASTE_RANGED = 19;
			CR_HASTE_SPELL = 20;
			CR_WEAPON_SKILL_MAINHAND = 21;
			CR_WEAPON_SKILL_OFFHAND = 22;
			CR_WEAPON_SKILL_RANGED = 23;
			--]]
			["방어 숙련도"] = {"CR_DEFENSE",},
			["방어 숙련도가 증가합니다"] = {"CR_DEFENSE",},
			["회피 숙련도"] = {"CR_DODGE",},
			["회피 숙련도가 증가합니다."] = {"CR_DODGE",},
			["무기 막기 숙련도"] = {"CR_PARRY",},
			["무기 막기 숙련도가 증가합니다"] = {"CR_PARRY",},
			["방패 막기 숙련도"] = {"CR_BLOCK",},
			["방패 막기 숙련도가 증가합니다"] = {"CR_BLOCK",},
			["Increases your shield block rating"] = {"CR_BLOCK",},
	
			["적중도"] = {"CR_HIT_MELEE",},
			["적중도가 증가합니다"] = {"CR_HIT_MELEE",}, -- ITEM_MOD_HIT_RATING
			["근접 적중도가 증가합니다"] = {"CR_HIT_MELEE",}, -- ITEM_MOD_HIT_MELEE_RATING
			["Increases your hit rating"] = {"CR_HIT_MELEE",},
			["주문 적중도"] = {"CR_HIT_SPELL",},
			["주문의 적중도"] = {"CR_HIT_SPELL",}, -- ITEM_MOD_HIT_SPELL_RATING
			["주문 적중도가 증가합니다"] = {"CR_HIT_SPELL",},
			["원거리 적중도"] = {"CR_HIT_RANGED",},
			["원거리 적중도가 증가합니다"] = {"CR_HIT_RANGED",}, -- ITEM_MOD_HIT_RANGED_RATING
			["Increases your ranged hit rating"] = {"CR_HIT_RANGED",},
	
			["치명타 적중도"] = {"CR_CRIT_MELEE",},
			["Critical Rating"] = {"CR_CRIT_MELEE",},
			["Critical Strike Rating"] = {"CR_CRIT_MELEE",},
			["치명타 적중도가 증가합니다"] = {"CR_CRIT_MELEE",},
			["근접 치명타 적중도가 증가합니다"] = {"CR_CRIT_MELEE",},
			["Improves critical strike rating"] = {"CR_CRIT_MELEE",},
			["주문 극대화 적중도"] = {"CR_CRIT_SPELL",},
			["주문의 극대화 적중도"] = {"CR_CRIT_SPELL",},
			["Spell Critical Rating"] = {"CR_CRIT_SPELL",},
			["Spell Crit Rating"] = {"CR_CRIT_SPELL",},
			["주문의 극대화 적중도가 증가합니다"] = {"CR_CRIT_SPELL",},
			["Increases the spell critical strike rating of all party members within 30 yards"] = {"CR_CRIT_SPELL",},
			["주문 극대화 적중도가 증가합니다"] = {"CR_CRIT_SPELL",},
			["Increases your ranged critical strike rating"] = {"CR_CRIT_RANGED",}, -- Fletcher's Gloves ID:7348
	
			["Improves hit avoidance rating"] = {"CR_HIT_TAKEN_MELEE"}, -- ITEM_MOD_HIT_TAKEN_RATING
			["Improves melee hit avoidance rating"] = {"CR_HIT_TAKEN_MELEE"}, -- ITEM_MOD_HIT_TAKEN_MELEE_RATING
			["Improves ranged hit avoidance rating"] = {"CR_HIT_TAKEN_RANGED"}, -- ITEM_MOD_HIT_TAKEN_RANGED_RATING
			["Improves spell hit avoidance rating"] = {"CR_HIT_TAKEN_SPELL"}, -- ITEM_MOD_HIT_TAKEN_SPELL_RATING
			["탄력도"] = {"CR_RESILIENCE",},
			["Resilience Rating"] = {"CR_RESILIENCE",},
			["탄력도가 증가합니다"] = {"CR_RESILIENCE",},
			["Improves critical avoidance rating"] = {"CR_CRIT_TAKEN_MELEE",},
			["Improves melee critical avoidance rating"] = {"CR_CRIT_TAKEN_MELEE",},
			["Improves ranged critical avoidance rating"] = {"CR_CRIT_TAKEN_RANGED",},
			["Improves spell critical avoidance rating"] = {"CR_CRIT_TAKEN_SPELL",},
	
			["공격 가속도"] = {"CR_HASTE_MELEE"},
			["주문 가속도"] = {"CR_HASTE_SPELL"},
			["원거리 공격 가속도"] = {"CR_HASTE_RANGED"},
			["공격 가속도가 증가합니다"] = {"CR_HASTE_MELEE"},
			["근전 공격 가속도가 증가합니다"] = {"CR_HASTE_MELEE"},
			["원거리 공격 가속도가 증가합니다"] = {"CR_HASTE_SPELL"},
			["주문 가속도가 증가합니다"] = {"CR_HASTE_RANGED"},
	
			["단검류 숙련도가 증가합니다"] = {"CR_WEAPON_DAGGER"},
			["한손 도검류 숙련도가 증가합니다"] = {"CR_WEAPON_SWORD"},
			["양손 도검류 숙련도가 증가합니다"] = {"CR_WEAPON_SWORD_2H"},
			["한손 도끼류 숙련도가 증가합니다"] = {"CR_WEAPON_AXE"},
			["양손 도끼류 숙련도가 증가합니다"] = {"CR_WEAPON_AXE_2H"},
			["Increases two-handed axes skill rating"] = {"CR_WEAPON_AXE_2H"},
			["한손 둔기류 숙련도가 증가합니다"] = {"CR_WEAPON_MACE"},
			["양손 둔기류 숙련도가 증가합니다"] = {"CR_WEAPON_MACE_2H"},
			["총기류 숙련도가 증가합니다"] = {"CR_WEAPON_GUN"},
			["석궁류 숙련도가 증가합니다"] = {"CR_WEAPON_CROSSBOW"},
			["활류 숙련도가 증가합니다"] = {"CR_WEAPON_BOW"},
			["야생 전투 숙련도가 증가합니다"] = {"CR_WEAPON_FERAL"},
			["장착 무기류 숙련도가 증가합니다"] = {"CR_WEAPON_FIST"}, -- Demonblood Eviscerator
			["맨손 전투 숙련도가 증가합니다"] = {"CR_WEAPON_FIST"}, -- Demonblood Eviscerator ID:27533
			-- Exclude
			["sec"] = false,
			["to"] = false,
			["칸 가방"] = false,
			["칸 화살통"] = false,
			["칸 탄환 주머니"] = false,
			["원거리 공격 속도가%만큼 증가합니다"] = false, -- AV quiver
			["원거리 무기 공격 속도가%만큼 증가합니다"] = false, -- AV quiver
		},
		----------------
		-- Stat Names --
		----------------
		-- StatID are compatible with ItemBonusLib and BonusScanner
		["StatIDToName"] = {
			["STEALTH_LEVEL"] = {"Stealth Level", "Stealth"},
			["MELEE_DMG"] = {"Melee Weapon Damage", "Weapon Damage"},
			["MOUNT_SPEED%"] = {"Mount Speed(%)", "Mount Speed(%)"},
			["RUN_SPEED%"] = {"Run Speed(%)", "Run Speed(%)"},
	
			["STR"] = {SPELL_STAT1_NAME, "Str"},
			["AGI"] = {SPELL_STAT2_NAME, "Agi"},
			["STA"] = {SPELL_STAT3_NAME, "Sta"},
			["INT"] = {SPELL_STAT4_NAME, "Int"},
			["SPI"] = {SPELL_STAT5_NAME, "Spi"},
			["ARMOR"] = {ARMOR.." from items", "ARMOR(Items)"},
			["ARMOR_BONUS"] = {ARMOR.." from bonus", "ARMOR(Bonus)"},
	
			["FIRE_RES"] = {RESISTANCE2_NAME, "FR"},
			["NATURE_RES"] = {RESISTANCE3_NAME, "NR"},
			["FROST_RES"] = {RESISTANCE4_NAME, "FrR"},
			["SHADOW_RES"] = {RESISTANCE5_NAME, "SR"},
			["ARCANE_RES"] = {RESISTANCE6_NAME, "AR"},
	
			["FISHING"] = {"Fishing", "Fishing"},
			["MINING"] = {"Mining", "Mining"},
			["HERBALISM"] = {"Herbalism", "Herbalism"},
			["SKINNING"] = {"Skinning", "Skinning"},
	
			["BLOCK_VALUE"] = {"Block Value", "Block Value"},
	
			["AP"] = {ATTACK_POWER_TOOLTIP, "AP"},
			["RANGED_AP"] = {RANGED_ATTACK_POWER, "RAP"},
			["FERAL_AP"] = {ATTACK_POWER_TOOLTIP.." (Feral)", "AP(Feral)"},
			["AP_UNDEAD"] = {ATTACK_POWER_TOOLTIP.." (Undead)", "AP(Undead)"},
			["AP_DEMON"] = {ATTACK_POWER_TOOLTIP.." (Demon)", "AP(Demon)"},
	
			["HEAL"] = {"Healing", "Heal"},
	
			["SPELL_DMG"] = {PLAYERSTAT_SPELL_COMBAT.." "..DAMAGE, PLAYERSTAT_SPELL_COMBAT.." Dmg"},
			["SPELL_DMG_UNDEAD"] = {PLAYERSTAT_SPELL_COMBAT.." "..DAMAGE.." (Undead)", PLAYERSTAT_SPELL_COMBAT.." Dmg".." (Undead)"},
			["HOLY_SPELL_DMG"] = {SPELL_SCHOOL1_CAP.." "..DAMAGE, SPELL_SCHOOL1_CAP.." Dmg"},
			["FIRE_SPELL_DMG"] = {SPELL_SCHOOL2_CAP.." "..DAMAGE, SPELL_SCHOOL2_CAP.." Dmg"},
			["NATURE_SPELL_DMG"] = {SPELL_SCHOOL3_CAP.." "..DAMAGE, SPELL_SCHOOL3_CAP.." Dmg"},
			["FROST_SPELL_DMG"] = {SPELL_SCHOOL4_CAP.." "..DAMAGE, SPELL_SCHOOL4_CAP.." Dmg"},
			["SHADOW_SPELL_DMG"] = {SPELL_SCHOOL5_CAP.." "..DAMAGE, SPELL_SCHOOL5_CAP.." Dmg"},
			["ARCANE_SPELL_DMG"] = {SPELL_SCHOOL6_CAP.." "..DAMAGE, SPELL_SCHOOL6_CAP.." Dmg"},
	
			["SPELLPEN"] = {PLAYERSTAT_SPELL_COMBAT.." "..SPELL_PENETRATION, SPELL_PENETRATION},
	
			["HEALTH_REG"] = {HEALTH.." Regen", "HP5"},
			["MANA_REG"] = {MANA.." Regen", "MP5"},
			["HEALTH"] = {HEALTH, HP},
			["MANA"] = {MANA, MP},
	
			["MAX_DAMAGE"] = {"Max Damage", "Max Dmg"},
			["DPS"] = {"Damage Per Second", "DPS"},
	
			["CR_DEFENSE"] = {COMBAT_RATING_NAME2, COMBAT_RATING_NAME2}, -- COMBAT_RATING_NAME2 = "Defense Rating"
			["CR_DODGE"] = {COMBAT_RATING_NAME3, COMBAT_RATING_NAME3}, -- COMBAT_RATING_NAME3 = "Dodge Rating"
			["CR_PARRY"] = {COMBAT_RATING_NAME4, COMBAT_RATING_NAME4}, -- COMBAT_RATING_NAME4 = "Parry Rating"
			["CR_BLOCK"] = {COMBAT_RATING_NAME5, COMBAT_RATING_NAME5}, -- COMBAT_RATING_NAME5 = "Block Rating"
			["CR_HIT_MELEE"] = {COMBAT_RATING_NAME6, COMBAT_RATING_NAME6}, -- COMBAT_RATING_NAME6 = "Hit Rating"
			["CR_HIT_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." "..COMBAT_RATING_NAME6, PLAYERSTAT_RANGED_COMBAT.." "..COMBAT_RATING_NAME6}, -- PLAYERSTAT_RANGED_COMBAT = "Ranged"
			["CR_HIT_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." "..COMBAT_RATING_NAME6, PLAYERSTAT_SPELL_COMBAT.." "..COMBAT_RATING_NAME6}, -- PLAYERSTAT_SPELL_COMBAT = "Spell"
			["CR_HIT_TAKEN_MELEE"] = {"Hit Avoidance "..RATING, "Hit Avoidance "..RATING},
			["CR_HIT_TAKEN_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Hit Avoidance "..RATING, PLAYERSTAT_RANGED_COMBAT.." Hit Avoidance "..RATING},
			["CR_HIT_TAKEN_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Hit Avoidance "..RATING, PLAYERSTAT_SPELL_COMBAT.." Hit Avoidance "..RATING},
			["CR_CRIT_MELEE"] = {COMBAT_RATING_NAME9, COMBAT_RATING_NAME9}, -- COMBAT_RATING_NAME9 = "Crit Rating"
			["CR_CRIT_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." "..COMBAT_RATING_NAME9, PLAYERSTAT_RANGED_COMBAT.." "..COMBAT_RATING_NAME9},
			["CR_CRIT_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." "..COMBAT_RATING_NAME9, PLAYERSTAT_SPELL_COMBAT.." "..COMBAT_RATING_NAME9},
			["CR_CRIT_TAKEN_MELEE"] = {"Crit Avoidance "..RATING, "Crit Avoidance "..RATING},
			["CR_CRIT_TAKEN_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Crit Avoidance "..RATING, PLAYERSTAT_RANGED_COMBAT.." Crit Avoidance "..RATING},
			["CR_CRIT_TAKEN_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Crit Avoidance "..RATING, PLAYERSTAT_SPELL_COMBAT.." Crit Avoidance "..RATING},
			["CR_RESILIENCE"] = {COMBAT_RATING_NAME15, COMBAT_RATING_NAME15}, -- COMBAT_RATING_NAME15 = "Resilience"
			["CR_HASTE_MELEE"] = {"Haste "..RATING, "Haste "..RATING}, --
			["CR_HASTE_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Haste "..RATING, PLAYERSTAT_RANGED_COMBAT.." Haste "..RATING},
			["CR_HASTE_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Haste "..RATING, PLAYERSTAT_SPELL_COMBAT.." Haste "..RATING},
			["CR_WEAPON_DAGGER"] = {"Dagger "..SKILL.." "..RATING, "Dagger "..RATING}, -- SKILL = "Skill"
			["CR_WEAPON_SWORD"] = {"Sword "..SKILL.." "..RATING, "Sword "..RATING},
			["CR_WEAPON_SWORD_2H"] = {"Sword "..SKILL.." "..RATING, "Sword "..RATING},
			["CR_WEAPON_AXE"] = {"Axe "..SKILL.." "..RATING, "Axe "..RATING},
			["CR_WEAPON_AXE_2H"] = {"Axe "..SKILL.." "..RATING, "Axe "..RATING},
			["CR_WEAPON_MACE"] = {"Mace "..SKILL.." "..RATING, "Mace "..RATING},
			["CR_WEAPON_MACE_2H"] = {"Mace "..SKILL.." "..RATING, "Mace "..RATING},
			["CR_WEAPON_GUN"] = {"Gun "..SKILL.." "..RATING, "Gun "..RATING},
			["CR_WEAPON_CROSSBOW"] = {"Crossbow "..SKILL.." "..RATING, "Crossbow "..RATING},
			["CR_WEAPON_BOW"] = {"Bow "..SKILL.." "..RATING, "Bow "..RATING},
			["CR_WEAPON_FERAL"] = {"Feral "..SKILL.." "..RATING, "Feral "..RATING},
			["CR_WEAPON_FIST"] = {"Unarmed "..SKILL.." "..RATING, "Unarmed "..RATING},
	
			-- Tier2 Stats
			-- Str -> AP, Block Value
			-- Agi -> AP, Crit, Dodge
			-- Sta -> Health
			-- Int -> Mana, Spell Crit
			-- Spi -> mp5oc, hp5oc
			-- Ratings -> Effect
			["DEFENSE"] = {DEFENSE, "Def"},
			["DODGE"] = {DODGE_CHANCE, DODGE},
			["PARRY"] = {PARRY_CHANCE, PARRY},
			["BLOCK"] = {BLOCK_CHANCE, BLOCK},
			["HIT_MELEE"] = {"Hit Chance", "Hit"},
			["HIT_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." ".."Hit Chance", PLAYERSTAT_RANGED_COMBAT.." Hit"},
			["HIT_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." ".."Hit Chance", PLAYERSTAT_SPELL_COMBAT.." Hit"},
			["HIT_TAKEN_MELEE"] = {"Hit Avoidance", "Hit Avoidance"},
			["HIT_TAKEN_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Hit Avoidance", PLAYERSTAT_RANGED_COMBAT.." Hit Avoidance"},
			["HIT_TAKEN_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Hit Avoidance", PLAYERSTAT_SPELL_COMBAT.." Hit Avoidance"},
			["CRIT_MELEE"] = {MELEE_CRIT_CHANCE, "Crit"}, -- MELEE_CRIT_CHANCE = "Crit Chance"
			["CRIT_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." "..MELEE_CRIT_CHANCE, PLAYERSTAT_RANGED_COMBAT.." Crit"},
			["CRIT_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." "..MELEE_CRIT_CHANCE, PLAYERSTAT_SPELL_COMBAT.." Crit"},
			["CRIT_TAKEN_MELEE"] = {"Crit Avoidance", "Crit Avoidance"},
			["CRIT_TAKEN_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Crit Avoidance", PLAYERSTAT_RANGED_COMBAT.." Crit Avoidance"},
			["CRIT_TAKEN_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Crit Avoidance", PLAYERSTAT_SPELL_COMBAT.." Crit Avoidance"},
			["HASTE_MELEE"] = {"Haste", "Haste"}, --
			["HASTE_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Haste", PLAYERSTAT_RANGED_COMBAT.." Haste"},
			["HASTE_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Haste", PLAYERSTAT_SPELL_COMBAT.." Haste"},
			["WEAPON_DAGGER"] = {"Dagger "..SKILL, "Dagger"}, -- SKILL = "Skill"
			["WEAPON_SWORD"] = {"Sword "..SKILL, "Sword"},
			["WEAPON_SWORD_2H"] = {"Sword "..SKILL, "Sword"},
			["WEAPON_AXE"] = {"Axe "..SKILL, "Axe"},
			["WEAPON_AXE_2H"] = {"Axe "..SKILL, "Axe"},
			["WEAPON_MACE"] = {"Mace "..SKILL, "Mace"},
			["WEAPON_MACE_2H"] = {"Mace "..SKILL, "Mace"},
			["WEAPON_GUN"] = {"Gun "..SKILL, "Gun"},
			["WEAPON_CROSSBOW"] = {"Crossbow "..SKILL, "Crossbow"},
			["WEAPON_BOW"] = {"Bow "..SKILL, "Bow"},
			["WEAPON_FERAL"] = {"Feral "..SKILL, "Feral"},
			["WEAPON_FIST"] = {"Unarmed "..SKILL, "Unarmed"},
			["CRIT_DAMAGE_TAKEN_MELEE"] = {"Crit Damage Taken", "Crit Damage Taken"},
			["CRIT_DAMAGE_TAKEN_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Crit Damage Taken", PLAYERSTAT_RANGED_COMBAT.." Crit Damage Taken"},
			["CRIT_DAMAGE_TAKEN_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Crit Damage Taken", PLAYERSTAT_SPELL_COMBAT.." Crit Damage Taken"},
	
			-- Tier3 Stats
			-- Defense -> Crit Avoidance, Hit Avoidance, Dodge, Parry, Block
			-- Weapon Skill -> Crit, Hit, Mob Dodge, Mob Parry, Mob Block
			--
		},
	}
end)

-- zhTW localization by CuteMiyu
L:RegisterTranslations("zhTW", function()
	return {
		["Locale"] = "zhTW",
		["tonumber"] = tonumber,
		--["Dual Wield"] = "雙武器",
		-------------------
		-- Exclude Table --
		-------------------
		-- By looking at the first ExcludeLen letters of a line we can exclude a lot of lines
		["ExcludeLen"] = 3, -- using string.utf8len
		["Exclude"] = {
			[""] = true,
			[" \n"] = true,
			["裝備後綁定"] = true,
			["拾取後綁定"] = true;
			["使用後綁定"] = true,
			["紅色插槽"] = true,
			["黃色插槽"] = true,
			["藍色插槽"] = true,
			["變換插槽"] = true,
			["任務物品"] = true,
			["該物品將觸發一個任務"] = true,
			["獨特裝備"] = true, -- Unique-Equipped
			["唯一"] = true, -- Unique (20)
			["需要等"] = true, -- Requires Level xx
			["\n需要"] = true, -- Requires Level xx
			["職業："] = true, -- Classes: xx
			["種族："] = true, -- Races: xx (vendor mounts)
			["使用："] = true, -- Use:
			["擊中時"] = true, -- Chance On Hit:
			["需要鑄"] = true,
			-- Set Bonuses
			["套裝："] = true,
			["(2)"] = true,
			["(3)"] = true,
			["(4)"] = true,
			["(5)"] = true,
			["(6)"] = true,
			["(7)"] = true,
			["(8)"] = true,
			-- Equip type
			["彈藥"] = true, -- Ice Threaded Arrow ID:19316
			[INVTYPE_AMMO] = true,
			[INVTYPE_HEAD] = true,
			[INVTYPE_NECK] = true,
			[INVTYPE_SHOULDER] = true,
			[INVTYPE_BODY] = true,
			[INVTYPE_CHEST] = true,
			[INVTYPE_ROBE] = true,
			[INVTYPE_WAIST] = true,
			[INVTYPE_LEGS] = true,
			[INVTYPE_FEET] = true,
			[INVTYPE_WRIST] = true,
			[INVTYPE_HAND] = true,
			[INVTYPE_FINGER] = true,
			[INVTYPE_TRINKET] = true,
			[INVTYPE_CLOAK] = true,
			[INVTYPE_WEAPON] = true,
			[INVTYPE_SHIELD] = true,
			[INVTYPE_2HWEAPON] = true,
			[INVTYPE_WEAPONMAINHAND] = true,
			[INVTYPE_WEAPONOFFHAND] = true,
			[INVTYPE_HOLDABLE] = true,
			[INVTYPE_RANGED] = true,
			[INVTYPE_THROWN] = true,
			[INVTYPE_RANGEDRIGHT] = true,
			[INVTYPE_RELIC] = true,
			[INVTYPE_TABARD] = true,
			[INVTYPE_BAG] = true,
		},
		--[[
		textTable = {
			"法術傷害 +6 及法術命中等級 +5",
			"+3  耐力, +4 致命一擊等級",
			"++26 治療法術 & 降低2% 威脅值",
			"+3 耐力/+4 致命一擊等級",
			"插槽加成:每5秒+2法力",
			"裝備： 使所有法術和魔法效果所造成的傷害和治療效果提高最多150點。",
			"裝備： 使半徑30碼範圍內所有小隊成員的法術致命一擊等級提高28點。",
			"裝備： 使30碼範圍內的所有隊友提高所有法術和魔法效果所造成的傷害和治療效果，最多33點。",
			"裝備： 使周圍半徑30碼範圍內隊友的所有法術和魔法效果所造成的治療效果提高最多62點。",
			"裝備： 使你的法術傷害提高最多120點，以及你的治療效果最多300點。",
			"裝備： 使周圍半徑30碼範圍內的隊友每5秒恢復11點法力。",
			"裝備： 使法術所造成的治療效果提高最多300點。",
			"裝備： 在獵豹、熊、巨熊和梟獸形態下的攻擊強度提高420點。",
			-- "+10 Defense Rating/+10 Stamina/+15 Block Value": ZG Enchant
			-- "+26 Attack Power and +14 Critical Strike Rating": Swift Windfire Diamond ID:28556
			-- "+26 Healing Spells & 2% Reduced Threat": Bracing Earthstorm Diamond ID:25897
			-- "+6 Spell Damage, +5 Spell Crit Rating": Potent Ornate Topaz ID: 28123
			----
			-- "Critical Rating +6 and Dodge Rating +5": Assassin's Fire Opal ID:30565
			-- "Healing +11 and 2 mana per 5 sec.": Royal Tanzanite ID: 30603
		}
		--]]
		-----------------------
		-- Whole Text Lookup --
		-----------------------
		-- Mainly used for enchants that doesn't have numbers in the text
		["WholeTextLookup"] = {
			["野性"] = {["AP"] = 70}, -- 
	
			["初級巫師之油"] = {["SPELL_DMG"] = 8, ["HEAL"] = 8}, -- 
			["次級巫師之油"] = {["SPELL_DMG"] = 16, ["HEAL"] = 16}, -- 
			["巫師之油"] = {["SPELL_DMG"] = 24, ["HEAL"] = 24}, -- 
			["卓越巫師之油"] = {["SPELL_DMG"] = 36, ["HEAL"] = 36, ["CR_CRIT_SPELL"] = 14}, -- 
			
			["超強法力之油"] = {["MANA_REG"] = 14}, -- 
			["初級法力之油"] = {["MANA_REG"] = 4}, -- 
			["次級法力之油"] = {["MANA_REG"] = 8}, -- 
			["卓越法力之油"] = {["MANA_REG"] = 12, ["HEAL"] = 25}, -- 
			["超強巫師之油"] = {["SPELL_DMG"] = 42, ["HEAL"] = 42}, -- 
			
			["恆金漁線釣魚"] = {["FISHING"] = 5}, -- 
			["活力"] = {["MANA_REG"] = 4, ["HEALTH_REG"] = 4}, -- 
			["靈魂冰霜"] = {["SHADOW_SPELL_DMG"] = 54, ["FROST_SPELL_DMG"] = 54}, -- 
			["烈日火焰"] = {["ARCANE_SPELL_DMG"] = 50, ["FIRE_SPELL_DMG"] = 50}, -- 
			
			["祕銀馬刺"] = {["MOUNT_SPEED%"] = 4}, -- Mithril Spurs
			["坐騎移動速度略微提升"] = {["MOUNT_SPEED%"] = 2}, -- Enchant Gloves - Riding Skill
			["裝備：略微提高移動速度。"] = {["RUN_SPEED%"] = 8}, -- [Highlander's Plate Greaves] ID: 20048
			["略微提高移動速度"] = {["RUN_SPEED%"] = 8}, --
			["移動速度略微提升"] = {["RUN_SPEED%"] = 8}, -- Enchant Boots - Minor Speed
			["裝備： 使你可以在水下呼吸。"] = false, -- [Band of Icy Depths] ID: 21526
			["使你可以在水下呼吸"] = false, --
			["裝備： 免疫繳械。"] = false, -- [Stronghold Gauntlets] ID: 12639
			["免疫繳械"] = false, --
			["十字軍"] = false, -- Enchant Crusader
			["生命偷取"] = false, -- Enchant Crusader
		},
		----------------------------
		-- Single Plus Stat Check --
		----------------------------
		-- depending on locale, it may be
		-- +19 Stamina = "^%+(%d+) ([%a ]+%a)$"
		-- Stamina +19 = "^([%a ]+%a) %+(%d+)$"
		-- +19 耐力 = "^%+(%d+) (.-)$"
		--["SinglePlusStatCheck"] = "^%+(%d+) ([%a ]+%a)$",
		["SinglePlusStatCheck"] = "^%+(%d+) (.-)$",
		-----------------------------
		-- Single Equip Stat Check --
		-----------------------------
		-- stat1, value, stat2 = strfind
		-- stat = stat1..stat2
		-- "^Equip: (.-) by u?p? ?t?o? ?(%d+) ?(.-)%.$"
		["SingleEquipStatCheck"] = "裝備： (.-)提高\230?\156?\128?\229?\164?\154?(%d+)\233?\187?\158?(.-)。$",
		--"裝備： (.-)提高(最多)?(%d+)(點)?(.-)。$",
		-------------
		-- PreScan --
		-------------
		-- Special cases that need to be dealt with before base scan
		["PreScanPatterns"] = {
			--["^Equip: Increases attack power by (%d+) in Cat"] = "FERAL_AP",
			--["^Equip: Increases attack power by (%d+) when fighting Undead"] = "AP_UNDEAD", -- Seal of the Dawn ID:13029
			["^(%d+)格擋$"] = "BLOCK_VALUE",
			["^(%d+)點護甲$"] = "ARMOR",
			["強化護甲 %+(%d+)"] = "ARMOR_BONUS",
			["每5秒恢復(%d+)點法力。$"] = "MANA_REG",
			["每5秒恢復(%d+)點生命力。$"] = "HEALTH_REG",
			["^%+?%d+ %- (%d+).-傷害$"] = "MAX_DAMAGE",
			["^（每秒傷害([%d%.]+)）$"] = "DPS",
			-- Exclude
			["^(%d+)格"] = false, -- # of slots and bag type
			["^.+（(%d+)/%d+）$"] = false, -- Set Name (0/9)
			["|cff808080"] = false, -- Gray text "  |cff808080Requires at least 2 Yellow gems|r\n  |cff808080Requires at least 1 Red gem|r"
			-- Procs
			["有機會"] = false, -- [Mark of Defiance] ID:27924 -- [Staff of the Qiraji Prophets] ID:21128
			["有可能"] = false, -- [Darkmoon Card: Heroism] ID:19287
			["被擊中之後"] = false, -- [Essence of the Pure Flame] ID: 18815
		},
		--------------
		-- DeepScan --
		--------------
		["Equip: "] = "裝備： ",
		["Socket Bonus: "] = "插槽加成:",
		["."] = "。",
		["DeepScanSeparators"] = {
			" 及", -- "Critical Rating +6 and Dodge Rating +5": Assassin's Fire Opal ID:30565
			"/", -- "+10 Defense Rating/+10 Stamina/+15 Block Value": ZG Enchant
			" & ", -- "+26 Healing Spells & 2% Reduced Threat": Bracing Earthstorm Diamond ID:25897
			", ", -- "+6 Spell Damage, +5 Spell Crit Rating": Potent Ornate Topaz ID: 28123
			"。同時", -- "裝備： 對不死生物的攻擊強度提高$s1點。同時也可為銀色黎明收集天譴石。": 黎明聖印
		},
		["DeepScanPatterns"] = {
			"^(.-)提高最多(%d+)(.-)$", -- "xxx by up to 22 xxx" (scan first)
			"^(.-)，最多(%d+)(.-)$", -- "xxx by up to 22 xxx" (scan first)
			"^(.-)提高(%d+)(.-)$", -- "xxx by up to 22 xxx" (scan first)
			"^(.-) ?%+(%d+) ?(.-)$", -- "xxx xxx +22" or "+22 xxx xxx" or "xxx +22 xxx" (scan 2ed)
			"^(.-) ?([%d%.]+) ?(.-)$", -- 22.22 xxx xxx (scan last)
		},
		-----------------------
		-- Stat Lookup Table --
		-----------------------
		["StatIDLookup"] = {
			["使你的有效潛行等級提高"] = {"STEALTH_LEVEL"}, -- [Nightscape Boots] ID: 8197
			["武器傷害"] = {"MELEE_DMG"}, -- Enchant
			["使坐騎速度提高%。"] = {"MOUNT_SPEED%"}, -- [Highlander's Plate Greaves] ID: 20048
	
			["所有屬性"] = {"STR", "AGI", "STA", "INT", "SPI",},
			["力量"] = {"STR",},
			["敏捷"] = {"AGI",},
			["耐力"] = {"STA",},
			["智力"] = {"INT",},
			["精神"] = {"SPI",},
	
			["祕法抗性"] = {"ARCANE_RES",},
			["火焰抗性"] = {"FIRE_RES",},
			["自然抗性"] = {"NATURE_RES",},
			["冰霜抗性"] = {"FROST_RES",},
			["暗影抗性"] = {"SHADOW_RES",},
			["陰影抗性"] = {"SHADOW_RES",}, -- Demons Blood ID: 10779
			["所有抗性"] = {"ARCANE_RES", "FIRE_RES", "FROST_RES", "NATURE_RES", "SHADOW_RES",},
			["全部抗性"] = {"ARCANE_RES", "FIRE_RES", "FROST_RES", "NATURE_RES", "SHADOW_RES",},
			["抵抗全部"] = {"ARCANE_RES", "FIRE_RES", "FROST_RES", "NATURE_RES", "SHADOW_RES",},
	
			["釣魚"] = {"FISHING",}, -- Fishing enchant ID:846
			["釣魚技能"] = {"FISHING",}, -- Fishing lure
			["使釣魚技能"] = {"FISHING",}, -- Equip: Increased Fishing +20.
			["採礦"] = {"MINING",}, -- Mining enchant ID:844
			["草藥學"] = {"HERBALISM",}, -- Heabalism enchant ID:845
			["剝皮"] = {"SKINNING",}, -- Skinning enchant ID:865
	
			["護甲"] = {"ARMOR",},
			["防禦"] = {"DEFENSE",},
			["增加防禦"] = {"DEFENSE",},
			["格擋"] = {"BLOCK_VALUE",}, -- +22 Block Value
			["格擋值"] = {"BLOCK_VALUE",}, -- +22 Block Value
			["使你盾牌的格擋值"] = {"BLOCK_VALUE",},
	
			["生命力"] = {"HEALTH",},
			--["HP"] = {"HEALTH",},
			["法力"] = {"MANA",},
	
			["攻擊強度"] = {"AP",},
			["使攻擊強度"] = {"AP",},
			["Attack Power when fighting Undead"] = {"AP_UNDEAD",},
			["Increases attack powerwhen fighting Undead"] = {"AP_UNDEAD",},
			["Increases attack powerwhen fighting Undead.  It also allows the acquisition of Scourgestones on behalf of the Argent Dawn"] = {"AP_UNDEAD",},
			["Increases attack powerwhen fighting Demons"] = {"AP_DEMON",},
			["Attack Power in Cat, Bear, and Dire Bear forms only"] = {"FERAL_AP",},
			["在獵豹、熊、巨熊和梟獸形態下的攻擊強度提高點。"] = {"FERAL_AP",}, -- Atiesh, ID: 22632
			
			["Ranged Attack Power"] = {"RANGED_AP",},
			["Increases ranged attack power"] = {"RANGED_AP",}, -- [High Warlord's Crossbow] ID: 18837
	
			["Health Regen"] = {"MANA_REG",},
			["Health per"] = {"HEALTH_REG",},
			["health per"] = {"HEALTH_REG",}, -- Frostwolf Insignia Rank 6 ID:17909
			["Health every"] = {"MANA_REG",},
			--["health every"] = {"HEALTH_REG",}, -- [Resurgence Rod] ID:17743
			--["your normal health regeneration"] = {"HEALTH_REG",}, -- Demons Blood ID: 10779
			--["Restoreshealth per 5 sec"] = {"HEALTH_REG",}, -- [Onyxia Blood Talisman] ID: 18406
			--["Restoreshealth every 5 sec"] = {"HEALTH_REG",}, -- [Resurgence Rod] ID:17743
			["Mana Regen"] = {"MANA_REG",},
			--["Mana per"] = {"MANA_REG",},
			--["mana per"] = {"MANA_REG",}, -- Resurgence Rod ID:17743 Most common
			["Mana every"] = {"MANA_REG",},
			["mana every"] = {"MANA_REG",},
			["Mana every 5 seconds"] = {"MANA_REG",}, -- [Royal Nightseye] ID: 24057
			["Mana every 5 Sec"] = {"MANA_REG",}, -- 
			["mana every 5 sec"] = {"MANA_REG",}, -- 
			["Mana per 5 Seconds"] = {"MANA_REG",}, -- [Royal Shadow Draenite] ID: 23109
			["Mana Per 5 sec"] = {"MANA_REG",}, -- [Royal Shadow Draenite] ID: 23109
			["Mana per 5 sec"] = {"MANA_REG",}, -- [Cyclone Shoulderpads] ID: 29031
			["mana per 5 sec"] = {"MANA_REG",}, -- [Royal Tanzanite] ID: 30603
			["Restoresmana per 5 sec"] = {"MANA_REG",}, -- [Resurgence Rod] ID:17743
	
			["法術穿透"] = {"SPELLPEN",},
			["使你的法術穿透"] = {"SPELLPEN",},
			["Healing and Spell Damage"] = {"SPELL_DMG", "HEAL",},
			["Damage and Healing Spells"] = {"SPELL_DMG", "HEAL",},
			["Spell Damage and Healing"] = {"SPELL_DMG", "HEAL",},
			["Spell Damage"] = {"SPELL_DMG", "HEAL",},
			["使所有法術和魔法效果所造成的傷害和治療效果"] = {"SPELL_DMG", "HEAL"},
			["使30碼範圍內的所有隊友提高所有法術和魔法效果所造成的傷害和治療效果點"] = {"SPELL_DMG", "HEAL"}, -- Atiesh, ID: 22630
			--
			["Spell Damage and Healing"] = {"SPELL_DMG", "HEAL",},
			--StatLogic:GetSum("item:22630")
			["Damage"] = {"SPELL_DMG",},
			["Increases your spell damage"] = {"SPELL_DMG",}, -- Atiesh ID:22630, 22631, 22632, 22589
	        --使半徑$a1碼範圍內所有小隊成員的法術致命一擊等級提高$s點。
	        --使$a1碼範圍內的所有隊友提高所有法術和魔法效果所造成的傷害和治療效果，最多$s1點。
	        --使周圍半徑$a1碼範圍內隊友的所有法術和魔法效果所造成的治療效果提高最多$s1點。
	        --使周圍半徑$a1碼範圍內的隊友每5秒恢復$s1點法力。
	        --Increases the spell critical strike rating of all party members within $a1 yards by $s.
	        --Increases damage and healing done by magical spells and effects of all party members within $a1 yards by up to $s1.
	        --Increases healing done by magical spells and effects of all party members within $a1 yards by up to $s1.
	        --Restores $s1 mana per 5 seconds to all party members within $a1 yards.
			["法術能量"] = {"SPELL_DMG",},
			["神聖傷害"] = {"HOLY_SPELL_DMG",},
			["祕法傷害"] = {"ARCANE_SPELL_DMG",},
			["火焰傷害"] = {"FIRE_SPELL_DMG",},
			["自然傷害"] = {"NATURE_SPELL_DMG",},
			["冰霜傷害"] = {"FROST_SPELL_DMG",},
			["暗影傷害"] = {"SHADOW_SPELL_DMG",},
			["神聖法術傷害"] = {"HOLY_SPELL_DMG",},
			["祕法法術傷害"] = {"ARCANE_SPELL_DMG",},
			["火焰法術傷害"] = {"FIRE_SPELL_DMG",},
			["自然法術傷害"] = {"NATURE_SPELL_DMG",},
			["冰霜法術傷害"] = {"FROST_SPELL_DMG",}, -- Acrobatic Staff of Frozen Wrath ID:3185:0:0:0:0:0:1957
			["暗影法術傷害"] = {"SHADOW_SPELL_DMG",},
			["使祕法法術和效果所造成的傷害"] = {"ARCANE_SPELL_DMG",},
			["使火焰法術和效果所造成的傷害"] = {"FIRE_SPELL_DMG",},
			["使冰霜法術和效果所造成的傷害"] = {"FROST_SPELL_DMG",}, -- Frozen Shadoweave Vest ID:21871
			["使神聖法術和效果所造成的傷害"] = {"HOLY_SPELL_DMG",},
			["使自然法術和效果所造成的傷害"] = {"NATURE_SPELL_DMG",},
			["使暗影法術和效果所造成的傷害"] = {"SHADOW_SPELL_DMG",}, -- Frozen Shadoweave Vest ID:21871
			["Increases damage done to Undead by magical spells and effects"] = {"SPELL_DMG_UNDEAD"},
			--提高所有法術和效果對不死生物所造成的傷害，最多$s1點。
			["Increases damage done to Undead by magical spells and effects.  It also allows the acquisition of Scourgestones on behalf of the Argent Dawn"] = {"SPELL_DMG_UNDEAD"},
			["治療法術"] = {"HEAL",},
			["Increases Healing"] = {"HEAL",},
			["Healing"] = {"HEAL",},
			["healing Spells"] = {"HEAL",},
			["Healing Spells"] = {"HEAL",}, -- [Royal Nightseye] ID: 24057
			["使法術所造成的治療效果"] = {"HEAL",},
			["使周圍半徑30碼範圍內隊友的所有法術和魔法效果所造成的治療效果點"] = {"HEAL",}, -- Atiesh, ID: 22631
			["your healing"] = {"HEAL",}, -- Atiesh
	
			["每秒傷害"] = {"DPS",},
			["每秒傷害提高"] = {"DPS",}, -- [Thorium Shells] ID: 15997
	
			--[[ Rating ID
			CR_WEAPON_SKILL = 1;
			CR_DEFENSE_SKILL = 2;
			CR_DODGE = 3;
			CR_PARRY = 4;
			CR_BLOCK = 5;
			CR_HIT_MELEE = 6;
			CR_HIT_RANGED = 7;
			CR_HIT_SPELL = 8;
			CR_CRIT_MELEE = 9;
			CR_CRIT_RANGED = 10;
			CR_CRIT_SPELL = 11;
			CR_HIT_TAKEN_MELEE = 12;
			CR_HIT_TAKEN_RANGED = 13;
			CR_HIT_TAKEN_SPELL = 14;
			CR_CRIT_TAKEN_MELEE = 15;
			CR_CRIT_TAKEN_RANGED = 16;
			CR_CRIT_TAKEN_SPELL = 17;
			CR_HASTE_MELEE = 18;
			CR_HASTE_RANGED = 19;
			CR_HASTE_SPELL = 20;
			CR_WEAPON_SKILL_MAINHAND = 21;
			CR_WEAPON_SKILL_OFFHAND = 22;
			CR_WEAPON_SKILL_RANGED = 23;
			--]]
			["防禦等級"] = {"CR_DEFENSE",},
			["使防禦等級"] = {"CR_DEFENSE",},
			["閃躲等級"] = {"CR_DODGE",},
			["使你的閃躲等級"] = {"CR_DODGE",},
			["招架等級"] = {"CR_PARRY",},
			["提高你的招架等級。"] = {"CR_PARRY",},
			["格擋機率等級"] = {"CR_BLOCK",},
			["使你的格擋等級"] = {"CR_BLOCK",},
			["提高你的盾牌格擋等級。"] = {"CR_BLOCK",},
	
			["命中等級"] = {"CR_HIT_MELEE",},
			["提高命中等級。"] = {"CR_HIT_MELEE",}, -- ITEM_MOD_HIT_RATING
			["提高近戰命中等級。"] = {"CR_HIT_MELEE",}, -- ITEM_MOD_HIT_MELEE_RATING
			["使你的命中等級"] = {"CR_HIT_MELEE",},
			["法術命中等級"] = {"CR_HIT_SPELL",},
			["提高法術命中等級。"] = {"CR_HIT_SPELL",}, -- ITEM_MOD_HIT_SPELL_RATING
			["使你的法術命中等級"] = {"CR_HIT_SPELL",},
			["遠程命中等級"] = {"CR_HIT_RANGED",},
			["提高遠距命中等級。"] = {"CR_HIT_RANGED",}, -- ITEM_MOD_HIT_RANGED_RATING
			["使你的遠程命中等級"] = {"CR_HIT_RANGED",},
	
			["致命一擊等級"] = {"CR_CRIT_MELEE",},
			--["Critical Rating"] = {"CR_CRIT_MELEE",},
			--["Critical Strike Rating"] = {"CR_CRIT_MELEE",},
			["Increases your critical hit rating"] = {"CR_CRIT_MELEE",},
			["使你的致命一擊等級"] = {"CR_CRIT_MELEE",},
			["Improves critical strike rating"] = {"CR_CRIT_MELEE",},
			["法術致命一擊等級"] = {"CR_CRIT_SPELL",},
			--["Spell Critical strike rating"] = {"CR_CRIT_SPELL",},
			--["Spell Critical Rating"] = {"CR_CRIT_SPELL",},
			--["Spell Crit Rating"] = {"CR_CRIT_SPELL",},
			["使你的法術致命一擊等級"] = {"CR_CRIT_SPELL",},
			["使半徑30碼範圍內所有小隊成員的法術致命一擊等級點"] = {"CR_CRIT_SPELL",}, -- Atiesh, ID: 22589
			["Improves spell critical strike rating"] = {"CR_CRIT_SPELL",},
			["Increases your ranged critical strike rating"] = {"CR_CRIT_RANGED",}, -- Fletcher's Gloves ID:7348
	
			["提高命中迴避率。"] = {"CR_HIT_TAKEN_MELEE"}, -- ITEM_MOD_HIT_TAKEN_RATING, Necklace of Trophies ID: 31275 (Patch 2.0.10 changed it to Hit Rating)
			["提高近戰命中迴避率。"] = {"CR_HIT_TAKEN_MELEE"}, -- ITEM_MOD_HIT_TAKEN_MELEE_RATING
			["提高遠距命中迴避率。"] = {"CR_HIT_TAKEN_RANGED"}, -- ITEM_MOD_HIT_TAKEN_RANGED_RATING
			["提高法術命中迴避率。"] = {"CR_HIT_TAKEN_SPELL"}, -- ITEM_MOD_HIT_TAKEN_SPELL_RATING
			["韌性"] = {"CR_RESILIENCE",},
			["韌性等級"] = {"CR_RESILIENCE",},
			["使你的韌性等級提高。"] = {"CR_RESILIENCE",},
			["提高致命一擊等級迴避率"] = {"CR_CRIT_TAKEN_MELEE",},
			["提高近戰致命一擊等級迴避率。"] = {"CR_CRIT_TAKEN_MELEE",},
			["提高遠距致命一擊等級迴避率。"] = {"CR_CRIT_TAKEN_RANGED",},
			["提高法術致命一擊等級迴避率。"] = {"CR_CRIT_TAKEN_SPELL",},
	
			["攻擊速度"] = {"CR_HASTE_MELEE"},
			["攻擊速度等級"] = {"CR_HASTE_SPELL"},
			["遠程攻擊加速等級"] = {"CR_HASTE_RANGED"},
			["提高加速等級。"] = {"CR_HASTE_MELEE"},
			["提高近戰加速等級。"] = {"CR_HASTE_MELEE"},
			["提高遠程攻擊加速等級。"] = {"CR_HASTE_SPELL"},
			["提高法術加速等級。"] = {"CR_HASTE_RANGED"},
	
			["使匕首技能"] = {"CR_WEAPON_DAGGER"},
			["劍類武器技能"] = {"CR_WEAPON_SWORD"},
			["雙手劍技能"] = {"CR_WEAPON_SWORD_2H"},
			["斧類武器技能"] = {"CR_WEAPON_AXE"},
			["雙手斧技能"] = {"CR_WEAPON_AXE_2H"},
			["錘類武器技能"] = {"CR_WEAPON_MACE"},
			["雙手錘技能"] = {"CR_WEAPON_MACE_2H"},
			["使槍械技能"] = {"CR_WEAPON_GUN"},
			["使弩技能"] = {"CR_WEAPON_CROSSBOW"},
			["弓箭技能"] = {"CR_WEAPON_BOW"},
			["使野性戰鬥技巧等級"] = {"CR_WEAPON_FERAL"},
			["拳套武器技能"] = {"CR_WEAPON_FIST"}, -- Demonblood Eviscerator ID:27533
			
			-- Exclude
			["秒"] = false,
			["to"] = false,
			["格容器"] = false,
			["格箭袋"] = false,
			["格彈藥袋"] = false,
			["遠程攻擊速度%"] = false, -- AV quiver
		},
		----------------
		-- Stat Names --
		----------------
		-- StatID are compatible with ItemBonusLib and BonusScanner
		["StatIDToName"] = {
			["STEALTH_LEVEL"] = {"Stealth Level", "Stealth"},
			["MELEE_DMG"] = {"Melee Weapon Damage", "Weapon Damage"},
			["MOUNT_SPEED%"] = {"Mount Speed(%)", "Mount Speed(%)"},
			["RUN_SPEED%"] = {"Run Speed(%)", "Run Speed(%)"},
	
			["STR"] = {SPELL_STAT1_NAME, "Str"},
			["AGI"] = {SPELL_STAT2_NAME, "Agi"},
			["STA"] = {SPELL_STAT3_NAME, "Sta"},
			["INT"] = {SPELL_STAT4_NAME, "Int"},
			["SPI"] = {SPELL_STAT5_NAME, "Spi"},
			["ARMOR"] = {ARMOR.." from items", "ARMOR(Items)"},
			["ARMOR_BONUS"] = {ARMOR.." from bonus", "ARMOR(Bonus)"},
	
			["FIRE_RES"] = {RESISTANCE2_NAME, "FR"},
			["NATURE_RES"] = {RESISTANCE3_NAME, "NR"},
			["FROST_RES"] = {RESISTANCE4_NAME, "FrR"},
			["SHADOW_RES"] = {RESISTANCE5_NAME, "SR"},
			["ARCANE_RES"] = {RESISTANCE6_NAME, "AR"},
	
			["FISHING"] = {"Fishing", "Fishing"},
			["MINING"] = {"Mining", "Mining"},
			["HERBALISM"] = {"Herbalism", "Herbalism"},
			["SKINNING"] = {"Skinning", "Skinning"},
	
			["BLOCK_VALUE"] = {"Block Value", "Block Value"},
	
			["AP"] = {ATTACK_POWER_TOOLTIP, "AP"},
			["RANGED_AP"] = {RANGED_ATTACK_POWER, "RAP"},
			["FERAL_AP"] = {ATTACK_POWER_TOOLTIP.." (Feral)", "AP(Feral)"},
			["AP_UNDEAD"] = {ATTACK_POWER_TOOLTIP.." (Undead)", "AP(Undead)"},
			["AP_DEMON"] = {ATTACK_POWER_TOOLTIP.." (Demon)", "AP(Demon)"},
	
			["HEAL"] = {"Healing", "Heal"},
	
			["SPELL_DMG"] = {PLAYERSTAT_SPELL_COMBAT.." "..DAMAGE, PLAYERSTAT_SPELL_COMBAT.." Dmg"},
			["SPELL_DMG_UNDEAD"] = {PLAYERSTAT_SPELL_COMBAT.." "..DAMAGE.." (Undead)", PLAYERSTAT_SPELL_COMBAT.." Dmg".." (Undead)"},
			["HOLY_SPELL_DMG"] = {SPELL_SCHOOL1_CAP.." "..DAMAGE, SPELL_SCHOOL1_CAP.." Dmg"},
			["FIRE_SPELL_DMG"] = {SPELL_SCHOOL2_CAP.." "..DAMAGE, SPELL_SCHOOL2_CAP.." Dmg"},
			["NATURE_SPELL_DMG"] = {SPELL_SCHOOL3_CAP.." "..DAMAGE, SPELL_SCHOOL3_CAP.." Dmg"},
			["FROST_SPELL_DMG"] = {SPELL_SCHOOL4_CAP.." "..DAMAGE, SPELL_SCHOOL4_CAP.." Dmg"},
			["SHADOW_SPELL_DMG"] = {SPELL_SCHOOL5_CAP.." "..DAMAGE, SPELL_SCHOOL5_CAP.." Dmg"},
			["ARCANE_SPELL_DMG"] = {SPELL_SCHOOL6_CAP.." "..DAMAGE, SPELL_SCHOOL6_CAP.." Dmg"},
	
			["SPELLPEN"] = {PLAYERSTAT_SPELL_COMBAT.." "..SPELL_PENETRATION, SPELL_PENETRATION},
	
			["HEALTH_REG"] = {HEALTH.." Regen", "HP5"},
			["MANA_REG"] = {MANA.." Regen", "MP5"},
			["HEALTH"] = {HEALTH, HP},
			["MANA"] = {MANA, MP},
	
			["MAX_DAMAGE"] = {"Max Damage", "Max Dmg"},
			["DPS"] = {"Damage Per Second", "DPS"},
	
			["CR_DEFENSE"] = {COMBAT_RATING_NAME2, COMBAT_RATING_NAME2}, -- COMBAT_RATING_NAME2 = "Defense Rating"
			["CR_DODGE"] = {COMBAT_RATING_NAME3, COMBAT_RATING_NAME3}, -- COMBAT_RATING_NAME3 = "Dodge Rating"
			["CR_PARRY"] = {COMBAT_RATING_NAME4, COMBAT_RATING_NAME4}, -- COMBAT_RATING_NAME4 = "Parry Rating"
			["CR_BLOCK"] = {COMBAT_RATING_NAME5, COMBAT_RATING_NAME5}, -- COMBAT_RATING_NAME5 = "Block Rating"
			["CR_HIT_MELEE"] = {COMBAT_RATING_NAME6, COMBAT_RATING_NAME6}, -- COMBAT_RATING_NAME6 = "Hit Rating"
			["CR_HIT_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." "..COMBAT_RATING_NAME6, PLAYERSTAT_RANGED_COMBAT.." "..COMBAT_RATING_NAME6}, -- PLAYERSTAT_RANGED_COMBAT = "Ranged"
			["CR_HIT_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." "..COMBAT_RATING_NAME6, PLAYERSTAT_SPELL_COMBAT.." "..COMBAT_RATING_NAME6}, -- PLAYERSTAT_SPELL_COMBAT = "Spell"
			["CR_HIT_TAKEN_MELEE"] = {"Hit Avoidance "..RATING, "Hit Avoidance "..RATING},
			["CR_HIT_TAKEN_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Hit Avoidance "..RATING, PLAYERSTAT_RANGED_COMBAT.." Hit Avoidance "..RATING},
			["CR_HIT_TAKEN_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Hit Avoidance "..RATING, PLAYERSTAT_SPELL_COMBAT.." Hit Avoidance "..RATING},
			["CR_CRIT_MELEE"] = {COMBAT_RATING_NAME9, COMBAT_RATING_NAME9}, -- COMBAT_RATING_NAME9 = "Crit Rating"
			["CR_CRIT_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." "..COMBAT_RATING_NAME9, PLAYERSTAT_RANGED_COMBAT.." "..COMBAT_RATING_NAME9},
			["CR_CRIT_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." "..COMBAT_RATING_NAME9, PLAYERSTAT_SPELL_COMBAT.." "..COMBAT_RATING_NAME9},
			["CR_CRIT_TAKEN_MELEE"] = {"Crit Avoidance "..RATING, "Crit Avoidance "..RATING},
			["CR_CRIT_TAKEN_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Crit Avoidance "..RATING, PLAYERSTAT_RANGED_COMBAT.." Crit Avoidance "..RATING},
			["CR_CRIT_TAKEN_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Crit Avoidance "..RATING, PLAYERSTAT_SPELL_COMBAT.." Crit Avoidance "..RATING},
			["CR_RESILIENCE"] = {COMBAT_RATING_NAME15, COMBAT_RATING_NAME15}, -- COMBAT_RATING_NAME15 = "Resilience"
			["CR_HASTE_MELEE"] = {"Haste "..RATING, "Haste "..RATING}, --
			["CR_HASTE_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Haste "..RATING, PLAYERSTAT_RANGED_COMBAT.." Haste "..RATING},
			["CR_HASTE_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Haste "..RATING, PLAYERSTAT_SPELL_COMBAT.." Haste "..RATING},
			["CR_WEAPON_DAGGER"] = {"Dagger "..SKILL.." "..RATING, "Dagger "..RATING}, -- SKILL = "Skill"
			["CR_WEAPON_SWORD"] = {"Sword "..SKILL.." "..RATING, "Sword "..RATING},
			["CR_WEAPON_SWORD_2H"] = {"Sword "..SKILL.." "..RATING, "Sword "..RATING},
			["CR_WEAPON_AXE"] = {"Axe "..SKILL.." "..RATING, "Axe "..RATING},
			["CR_WEAPON_AXE_2H"] = {"Axe "..SKILL.." "..RATING, "Axe "..RATING},
			["CR_WEAPON_MACE"] = {"Mace "..SKILL.." "..RATING, "Mace "..RATING},
			["CR_WEAPON_MACE_2H"] = {"Mace "..SKILL.." "..RATING, "Mace "..RATING},
			["CR_WEAPON_GUN"] = {"Gun "..SKILL.." "..RATING, "Gun "..RATING},
			["CR_WEAPON_CROSSBOW"] = {"Crossbow "..SKILL.." "..RATING, "Crossbow "..RATING},
			["CR_WEAPON_BOW"] = {"Bow "..SKILL.." "..RATING, "Bow "..RATING},
			["CR_WEAPON_FERAL"] = {"Feral "..SKILL.." "..RATING, "Feral "..RATING},
			["CR_WEAPON_FIST"] = {"Unarmed "..SKILL.." "..RATING, "Unarmed "..RATING},
	
			-- Tier2 Stats
			-- Str -> AP, Block Value
			-- Agi -> AP, Crit, Dodge
			-- Sta -> Health
			-- Int -> Mana, Spell Crit
			-- Spi -> mp5oc, hp5oc
			-- Ratings -> Effect
			["DEFENSE"] = {DEFENSE, "Def"},
			["DODGE"] = {DODGE_CHANCE, DODGE},
			["PARRY"] = {PARRY_CHANCE, PARRY},
			["BLOCK"] = {BLOCK_CHANCE, BLOCK},
			["HIT_MELEE"] = {"Hit Chance", "Hit"},
			["HIT_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." ".."Hit Chance", PLAYERSTAT_RANGED_COMBAT.." Hit"},
			["HIT_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." ".."Hit Chance", PLAYERSTAT_SPELL_COMBAT.." Hit"},
			["HIT_TAKEN_MELEE"] = {"Hit Avoidance", "Hit Avoidance"},
			["HIT_TAKEN_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Hit Avoidance", PLAYERSTAT_RANGED_COMBAT.." Hit Avoidance"},
			["HIT_TAKEN_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Hit Avoidance", PLAYERSTAT_SPELL_COMBAT.." Hit Avoidance"},
			["CRIT_MELEE"] = {MELEE_CRIT_CHANCE, "Crit"}, -- MELEE_CRIT_CHANCE = "Crit Chance"
			["CRIT_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." "..MELEE_CRIT_CHANCE, PLAYERSTAT_RANGED_COMBAT.." Crit"},
			["CRIT_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." "..MELEE_CRIT_CHANCE, PLAYERSTAT_SPELL_COMBAT.." Crit"},
			["CRIT_TAKEN_MELEE"] = {"Crit Avoidance", "Crit Avoidance"},
			["CRIT_TAKEN_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Crit Avoidance", PLAYERSTAT_RANGED_COMBAT.." Crit Avoidance"},
			["CRIT_TAKEN_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Crit Avoidance", PLAYERSTAT_SPELL_COMBAT.." Crit Avoidance"},
			["HASTE_MELEE"] = {"Haste", "Haste"}, --
			["HASTE_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Haste", PLAYERSTAT_RANGED_COMBAT.." Haste"},
			["HASTE_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Haste", PLAYERSTAT_SPELL_COMBAT.." Haste"},
			["WEAPON_DAGGER"] = {"Dagger "..SKILL, "Dagger"}, -- SKILL = "Skill"
			["WEAPON_SWORD"] = {"Sword "..SKILL, "Sword"},
			["WEAPON_SWORD_2H"] = {"Sword "..SKILL, "Sword"},
			["WEAPON_AXE"] = {"Axe "..SKILL, "Axe"},
			["WEAPON_AXE_2H"] = {"Axe "..SKILL, "Axe"},
			["WEAPON_MACE"] = {"Mace "..SKILL, "Mace"},
			["WEAPON_MACE_2H"] = {"Mace "..SKILL, "Mace"},
			["WEAPON_GUN"] = {"Gun "..SKILL, "Gun"},
			["WEAPON_CROSSBOW"] = {"Crossbow "..SKILL, "Crossbow"},
			["WEAPON_BOW"] = {"Bow "..SKILL, "Bow"},
			["WEAPON_FERAL"] = {"Feral "..SKILL, "Feral"},
			["WEAPON_FIST"] = {"Unarmed "..SKILL, "Unarmed"},
			["CRIT_DAMAGE_TAKEN_MELEE"] = {"Crit Damage Taken", "Crit Damage Taken"},
			["CRIT_DAMAGE_TAKEN_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Crit Damage Taken", PLAYERSTAT_RANGED_COMBAT.." Crit Damage Taken"},
			["CRIT_DAMAGE_TAKEN_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Crit Damage Taken", PLAYERSTAT_SPELL_COMBAT.." Crit Damage Taken"},
	
			-- Tier3 Stats
			-- Defense -> Crit Avoidance, Hit Avoidance, Dodge, Parry, Block
			-- Weapon Skill -> Crit, Hit, Mob Dodge, Mob Parry, Mob Block
			--
		},
	}
end)

-- deDE localization by Gailly
L:RegisterTranslations("deDE", function()
	return {
		["Locale"] = "deDE",
		["tonumber"] = function(s)
			local n = tonumber(s)
			if n then
				return n
			else
				return tonumber((gsub(s, ",", "%.")))
			end
		end,
		-------------------
		-- Exclude Table --
		-------------------
		-- By looking at the first ExcludeLen letters of a line we can exclude a lot of lines
		["ExcludeLen"] = 5,
		["Exclude"] = {
			[""] = true,
			[" \n"] = true,
			["Wird beim Anlegen gebunden"] = true,
			["Wird beim Aufheben gebunden"] = true,
			["Wird beim Benutzen gebunden"] = true,
			["Roter Sockel"] = true,
			["Gelber Sockel"] = true,
			["Blauer Sockel"] = true,
			["Metasockel"] = true,
			["Questgegenstand"] = true,
			["Dieser Gegenstand startet ein Quest"] = true,
			["Einzi"] = true, -- Unique (20)
			["Benöt"] = true, -- Requires Level xx
			["\nBen"] = true, -- Requires Level xx
			["Klass"] = true, -- Classes: xx
			["Rasse"] = true, -- Races: xx (vendor mounts)
			["Benut"] = true, -- Use:
			["Treff"] = true, -- Chance On Hit:
			-- Set Bonuses
			["Set: "] = true,
			["(2) S"] = true,
			["(2) S"] = true,
			["(3) S"] = true,
			["(4) S"] = true,
			["(5) S"] = true,
			["(6) S"] = true,
			["(7) S"] = true,
			["(8) S"] = true,
			-- Equip type
			["Projektil"] = true, -- Ice Threaded Arrow ID:19316
			[INVTYPE_AMMO] = true,
			[INVTYPE_HEAD] = true,
			[INVTYPE_NECK] = true,
			[INVTYPE_SHOULDER] = true,
			[INVTYPE_BODY] = true,
			[INVTYPE_CHEST] = true,
			[INVTYPE_ROBE] = true,
			[INVTYPE_WAIST] = true,
			[INVTYPE_LEGS] = true,
			[INVTYPE_FEET] = true,
			[INVTYPE_WRIST] = true,
			[INVTYPE_HAND] = true,
			[INVTYPE_FINGER] = true,
			[INVTYPE_TRINKET] = true,
			[INVTYPE_CLOAK] = true,
			[INVTYPE_WEAPON] = true,
			[INVTYPE_SHIELD] = true,
			[INVTYPE_2HWEAPON] = true,
			[INVTYPE_WEAPONMAINHAND] = true,
			[INVTYPE_WEAPONOFFHAND] = true,
			[INVTYPE_HOLDABLE] = true,
			[INVTYPE_RANGED] = true,
			[INVTYPE_THROWN] = true,
			[INVTYPE_RANGEDRIGHT] = true,
			[INVTYPE_RELIC] = true,
			[INVTYPE_TABARD] = true,
			[INVTYPE_BAG] = true,
		},
		--[[
		textTable = {
			"Spell Damage +6 and Spell Hit Rating +5",
			"+3 Stamina, +4 Critical Strike Rating",
			"+26 Healing Spells & 2% Reduced Threat",
			"+3 Stamina/+4 Critical Strike Rating",
			"Socket Bonus: 2 mana per 5 sec.",
			"Equip: Increases damage and healing done by magical spells and effects by up to 150.",
			"Equip: Increases the spell critical strike rating of all party members within 30 yards by 28.",
			"Equip: Increases damage and healing done by magical spells and effects of all party members within 30 yards by up to 33.",
			"Equip: Increases healing done by magical spells and effects of all party members within 30 yards by up to 62.",
			"Equip: Increases your spell damage by up to 120 and your healing by up to 300.",
			"Equip: Restores 11 mana per 5 seconds to all party members within 30 yards.",
			"Equip: Increases healing done by spells and effects by up to 300.",
			"Equip: Increases attack power by 420 in Cat, Bear, Dire Bear, and Moonkin forms only.",
			-- "+10 Defense Rating/+10 Stamina/+15 Block Value": ZG Enchant
			-- "+26 Attack Power and +14 Critical Strike Rating": Swift Windfire Diamond ID:28556
			-- "+26 Healing Spells & 2% Reduced Threat": Bracing Earthstorm Diamond ID:25897
			-- "+6 Spell Damage, +5 Spell Crit Rating": Potent Ornate Topaz ID: 28123
			----
			-- "Critical Rating +6 and Dodge Rating +5": Assassin's Fire Opal ID:30565
			-- "Healing +11 and 2 mana per 5 sec.": Royal Tanzanite ID: 30603
		}
		--]]
		-----------------------
		-- Whole Text Lookup --
		-----------------------
		-- Mainly used for enchants that doesn't have numbers in the text
		["WholeTextLookup"] = {
			["Wildheit"] = {["AP"] = 70}, -- 
	
			["Schwaches Zauberöl"] = {["SPELL_DMG"] = 8, ["HEAL"] = 8}, -- 
			["Geringes Zauberöl"] = {["SPELL_DMG"] = 16, ["HEAL"] = 16}, -- 
			["Zauberöl"] = {["SPELL_DMG"] = 24, ["HEAL"] = 24}, -- 
			["Überragendes Zauberöl"] = {["SPELL_DMG"] = 42, ["HEAL"] = 42}, -- 
			["Hervorragendes Zauberöl"] = {["SPELL_DMG"] = 36, ["HEAL"] = 36, ["CR_CRIT_SPELL"] = 14}, -- 
			
			["Schwaches Manaöl"] = {["MANA_REG"] = 4}, -- 
			["Geringes Manaöl"] = {["MANA_REG"] = 8}, -- 
			["Überragendes Manaöl"] = {["MANA_REG"] = 14}, -- 
			["Hervorragendes Manaöl"] = {["MANA_REG"] = 12, ["HEAL"] = 25}, -- 
								
			
			["Eterniumangelschnur"] = {["FISHING"] = 5}, -- 
			["Vitalität"] = {["MANA_REG"] = 4, ["HEALTH_REG"] = 4}, -- 
			["Seelenfrost"] = {["SHADOW_SPELL_DMG"] = 54, ["FROST_SPELL_DMG"] = 54}, -- 
			["Sonnenfeuer"] = {["ARCANE_SPELL_DMG"] = 50, ["FIRE_SPELL_DMG"] = 50}, -- 
			
			["Mithrilsporen"] = {["MOUNT_SPEED%"] = 4}, -- Mithril Spurs
								["Minor Mount Speed Increase"] = {["MOUNT_SPEED%"] = 2}, -- Enchant Gloves - Riding Skill
			["Anlegen: Lauftempo ein wenig erhöht."] = {["RUN_SPEED%"] = 8}, -- [Highlander's Plate Greaves] ID: 20048
			["Lauftempo ein wenig erhöht"] = {["RUN_SPEED%"] = 8}, --
								["Minor Speed Increase"] = {["RUN_SPEED%"] = 8}, -- Enchant Boots - Minor Speed
			["Anlegen: Ermöglicht Unterwasseratmung."] = false, -- [Band of Icy Depths] ID: 21526
			["Ermöglicht Unterwasseratmung"] = false, --
			["Anlegen: Immun gegen Entwaffnen."] = false, -- [Stronghold Gauntlets] ID: 12639
			["Immun gegen Entwaffnen"] = false, --
			["Kreuzfahrer"] = false, -- Enchant Crusader
			["Lebensdiebstahl"] = false, -- Enchant Crusader
		},
		----------------------------
		-- Single Plus Stat Check --
		----------------------------
		-- depending on locale, it may be
		-- +19 Stamina = "^%+(%d+) ([%a ]+%a)$"
		-- Stamina +19 = "^([%a ]+%a) %+(%d+)$"
		-- +19 ?? = "^%+(%d+) (.-)$"
		--["SinglePlusStatCheck"] = "^%+(%d+) ([%a ]+%a)$",
		["SinglePlusStatCheck"] = "^%+(%d+) (.-)$",
		-----------------------------
		-- Single Equip Stat Check --
		-----------------------------
		-- stat1, value, stat2 = strfind
		-- stat = stat1..stat2
		-- "^Equip: (.-) by u?p? ?t?o? ?(%d+) ?(.-)%.$"
		["SingleEquipStatCheck"] = "^Anlegen: (.-) um b?i?s? ?z?u? ?(%d+) ?(.-)%.$",
		-------------
		-- PreScan --
		-------------
		-- Special cases that need to be dealt with before base scan
		["PreScanPatterns"] = {
			--["^Equip: Increases attack power by (%d+) in Cat"] = "FERAL_AP",
			--["^Equip: Increases attack power by (%d+) when fighting Undead"] = "AP_UNDEAD", -- Seal of the Dawn ID:13029
			["^(%d+) Block$"] = "BLOCK_VALUE",
			["^(%d+) Rüstung$"] = "ARMOR",
			["Verstärkte %(%+(%d+) Rüstung%)"] = "ARMOR_BUFF",
			["Mana Regeneration (%d+) alle 5 Sek%.$"] = "MANA_REG",
			["^%+?%d+ %- (%d+) .-Schaden$"] = "MAX_DAMAGE",
			["^%(([%d%,]+) Schaden pro Sekunde%)$"] = "DPS",
			-- Exclude
			["^(%d+) Slot"] = false, -- Set Name (0/9)
			["^[%a '%-]+%((%d+)/%d+%)$"] = false, -- Set Name (0/9)
			["|cff808080"] = false, -- Gray text "  |cff808080Requires at least 2 Yellow gems|r\n  |cff808080Requires at least 1 Red gem|r"
			-- Procs
			["[Cc]hance"] = false, -- [Mark of Defiance] ID:27924 -- [Staff of the Qiraji Prophets] ID:21128
			["[Ee]s besteht eine Chance"] = false, -- [Darkmoon Card: Heroism] ID:19287
			["[Ff]ügt dem Angreifer"] = false, -- [Essence of the Pure Flame] ID: 18815
		},
		--------------
		-- DeepScan --
		--------------
		["Equip: "] = "Anlegen: ",
		["DeepScanSeparators"] = {
			" und ", -- "Critical Rating +6 and Dodge Rating +5": Assassin's Fire Opal ID:30565
			"/", -- "+10 Defense Rating/+10 Stamina/+15 Block Value": ZG Enchant
			" & ", -- "+26 Healing Spells & 2% Reduced Threat": Bracing Earthstorm Diamond ID:25897
			", ", -- "+6 Spell Damage, +5 Spell Crit Rating": Potent Ornate Topaz ID: 28123
			"%. ", -- "Equip: Increases attack power by 81 when fighting Undead. It also allows the acquisition of Scourgestones on behalf of the Argent Dawn.": Seal of the Dawn
		},
		["DeepScanPatterns"] = {
			"^(.-) um b?i?s? ?z?u? ?(%d+) ?(.-)$", -- "xxx by up to 22 xxx" (scan first)
			"^(.-) ?%+(%d+) ?(.-)$", -- "xxx xxx +22" or "+22 xxx xxx" or "xxx +22 xxx" (scan 2ed)
			"^(.-) ?([%d%,]+) ?(.-)$", -- 22.22 xxx xxx (scan last)
		},
		-----------------------
		-- Stat Lookup Table --
		-----------------------
		["StatIDLookup"] = {
			["Increases your effective stealth level"] = {"STEALTH_LEVEL"}, -- [Nightscape Boots] ID: 8197
			["Waffenschaden"] = {"MELEE_DMG"}, -- Enchant
			["Erhöht das Reittiertempo%"] = {"MOUNT_SPEED%"}, -- [Highlander's Plate Greaves] ID: 20048
	
			["Alle Werte"] = {"STR", "AGI", "STA", "INT", "SPI",},
			["St\195\164rke"] = {"STR",},
			["Beweglichkeit"] = {"AGI",},
			["Ausdauer"] = {"STA",},
			["Intelligenz"] = {"INT",},
			["Willenskraft"] = {"SPI",},
	
			["Arkanwiderstand"] = {"ARCANE_RES",},
			["Feuerwiderstand"] = {"FIRE_RES",},
			["Naturwiderstand"] = {"NATURE_RES",},
			["Frostwiderstand"] = {"FROST_RES",},
			["Schattenwiderstand"] = {"SHADOW_RES",}, -- Demons Blood ID: 10779
			["Alle Widerstände"] = {"ARCANE_RES", "FIRE_RES", "FROST_RES", "NATURE_RES", "SHADOW_RES",},
								["Resist All"] = {"ARCANE_RES", "FIRE_RES", "FROST_RES", "NATURE_RES", "SHADOW_RES",},
	
			["Angeln"] = {"FISHING",}, -- Fishing enchant ID:846
			["Angelfertigkeit"] = {"FISHING",}, -- Fishing lure
			["Bergbau"] = {"MINING",}, -- Mining enchant ID:844
			["Kräuterkunde"] = {"HERBALISM",}, -- Heabalism enchant ID:845
			["Kürschnerei"] = {"SKINNING",}, -- Skinning enchant ID:865
	
			["Rüstung"] = {"ARMOR",},
			["Verteidigung"] = {"DEFENSE",},
			["Erhöhte Verteidugung"] = {"DEFENSE",},
			["Blocken"] = {"BLOCK_VALUE",}, -- +22 Block Value
			["Blockwert"] = {"BLOCK_VALUE",}, -- +22 Block Value
			["Erhöht den Blockwert Eures Schildes"] = {"BLOCK_VALUE",},
	
			["Gesundheit"] = {"HEALTH",},
			["HP"] = {"HEALTH",},
			["Mana"] = {"MANA",},
	
			["Angriffskraft"] = {"AP",},
			["Erhöht Angriffskraft"] = {"AP",},
			["Erhöht die Angriffskraft"] = {"AP",},
			["Erhöht die Angriffskraft im Kampf gegen Untote"] = {"AP_UNDEAD",},
			["Increases attack powerwhen fighting Undead"] = {"AP_UNDEAD",},
			["Erhöht die Angriffskraft im Kampf gegen Untote. Ermöglicht das Einsammel von Geißelsteinen im Namen der Argentumdämmerung"] = {"AP_UNDEAD",},
			["Increases attack powerwhen fighting Demons"] = {"AP_DEMON",},
			["Attack Power in Cat, Bear, and Dire Bear forms only"] = {"FERAL_AP",},
			["Increases attack powerin Cat, Bear, Dire Bear, and Moonkin forms only"] = {"FERAL_AP",},
			["Ranged Attack Power"] = {"RANGED_AP",},
			["Increases ranged attack power"] = {"RANGED_AP",}, -- [High Warlord's Crossbow] ID: 18837
	
			["Gesundheit wieder her"] = {"HEALTH_REG",}, -- Frostwolf Insignia Rank 6 ID:17909
			["Gesundheitsregeneration"] = {"HEALTH_REG",}, -- Demons Blood ID: 10779
								
			["Mana wieder her"] = {"MANA_REG",},
			["Mana alle 5 Sek"] = {"MANA_REG",}, -- [Royal Nightseye] ID: 24057
			["Mana alle 5 Sekunden"] = {"MANA_REG",},
			["alle 5 Sek.Mana"] = {"MANA_REG",}, -- [Royal Shadow Draenite] ID: 23109
			["Mana bei allen Gruppenmitgliedern, die sich im Umkreis von 30 befinden, wieder her"] = {"MANA_REG",}, -- Atiesh
	
			["Spell Penetration"] = {"SPELLPEN",},
			["Increases your spell penetration"] = {"SPELLPEN",},
			["Schaden und Heilung"] = {"SPELL_DMG", "HEAL",},
			["Damage and Healing Spells"] = {"SPELL_DMG", "HEAL",},
			["Spell Damage and Healing"] = {"SPELL_DMG", "HEAL",},
			["Zauberschaden"] = {"SPELL_DMG", "HEAL",},
			["Erhöht durch Zauber und magische Effekte verursachten Schaden und Heilung"] = {"SPELL_DMG", "HEAL"},
			["Erhöht durch Zauber und magische Effekte zugefügten Schaden und Heilung aller Gruppenmitglieder, die sich im Umkreis von 30 befinden,"] = {"SPELL_DMG", "HEAL"}, -- Atiesh
			["Zauberschaden und Heilung"] = {"SPELL_DMG", "HEAL",},
			--StatLogic:GetSum("item:22630")
			["Schaden"] = {"SPELL_DMG",},
			["Erhöht Euren Zauberschaden"] = {"SPELL_DMG",}, -- Atiesh ID:22630, 22631, 22632, 22589
			["Zauberschaden"] = {"SPELL_DMG",},
			["Heiligschaden"] = {"HOLY_SPELL_DMG",},
			["Arkanschaden"] = {"ARCANE_SPELL_DMG",},
			["Feuerschaden"] = {"FIRE_SPELL_DMG",},
			["Naturschaden"] = {"NATURE_SPELL_DMG",},
			["Frostschaden"] = {"FROST_SPELL_DMG",},
			["Schattenschaden"] = {"SHADOW_SPELL_DMG",},
			["Heiligzauberschaden"] = {"HOLY_SPELL_DMG",},
			["Arkanzauberschaden"] = {"ARCANE_SPELL_DMG",},
			["Feuerzauberschaden"] = {"FIRE_SPELL_DMG",},
			["Naturzauberschaden"] = {"NATURE_SPELL_DMG",},
			["Frostzauberschaden"] = {"FROST_SPELL_DMG",}, -- Acrobatic Staff of Frozen Wrath ID:3185:0:0:0:0:0:1957
			["Schattenzauberschaden"] = {"SHADOW_SPELL_DMG",},
			["Erhöht durch Arlkanzauber und Arkaneffekte zugefügten Schaden"] = {"ARCANE_SPELL_DMG",},
			["Erhöht durch Feuerzauber und Feuereffekte zugefügten Schaden"] = {"FIRE_SPELL_DMG",},
			["Erhöht durch Frostzauber und Frosteffekte zugefügten Schaden"] = {"FROST_SPELL_DMG",}, -- Frozen Shadoweave Vest ID:21871
			["Erhöht durch Heiligzauber und Heiligeffekte zugefügten Schaden"] = {"HOLY_SPELL_DMG",},
			["Erhöht durch Naturzauber und Natureffekte zugefügten Schaden"] = {"NATURE_SPELL_DMG",},
			["Erhöht durch Schattenzauber und Schatteneffekte zugefügten Schaden"] = {"SHADOW_SPELL_DMG",}, -- Frozen Shadoweave Vest ID:21871
			["Increases damage done to Undead by magical spells and effects"] = {"DMG_UNDEAD"},
			["Increases damage done to Undead by magical spells and effects.  It also allows the acquisition of Scourgestones on behalf of the Argent Dawn"] = {"DMG_UNDEAD"},
	
			["Erhöht Heilung"] = {"HEAL",},
			["Heilung"] = {"HEAL",},
			["Heilzauber"] = {"HEAL",}, -- [Royal Nightseye] ID: 24057
								
			["Erhöht durch Zauber und Effekte verursachte Heilung"] = {"HEAL",},
			["Erhöht durch Zauber und magische Effekte zugefügte Heilung aller Gruppenmitglieder, die sich im Umkreis von 30 befinden,"] = {"HEAL",}, -- Atiesh
			--					["your healing"] = {"HEAL",}, -- Atiesh
	
			["Schaden pro Sekunde"] = {"DPS",},
			["zusätzlichen Schaden pro Sekunde"] = {"DPS",}, -- [Thorium Shells] ID: 15977
			["Verursachtzusätzlichen Schaden pro Sekunde"] = {"DPS",}, -- [Thorium Shells] ID: 15977
	
			--[[ Rating ID
			CR_WEAPON_SKILL = 1;
			CR_DEFENSE_SKILL = 2;
			CR_DODGE = 3;
			CR_PARRY = 4;
			CR_BLOCK = 5;
			CR_HIT_MELEE = 6;
			CR_HIT_RANGED = 7;
			CR_HIT_SPELL = 8;
			CR_CRIT_MELEE = 9;
			CR_CRIT_RANGED = 10;
			CR_CRIT_SPELL = 11;
			CR_HIT_TAKEN_MELEE = 12;
			CR_HIT_TAKEN_RANGED = 13;
			CR_HIT_TAKEN_SPELL = 14;
			CR_CRIT_TAKEN_MELEE = 15;
			CR_CRIT_TAKEN_RANGED = 16;
			CR_CRIT_TAKEN_SPELL = 17;
			CR_HASTE_MELEE = 18;
			CR_HASTE_RANGED = 19;
			CR_HASTE_SPELL = 20;
			CR_WEAPON_SKILL_MAINHAND = 21;
			CR_WEAPON_SKILL_OFFHAND = 22;
			CR_WEAPON_SKILL_RANGED = 23;
			--]]
			["Verteidigungswertung"] = {"CR_DEFENSE",},
			["Erhöht Verteidigungswertung"] = {"CR_DEFENSE",},
			["Erhöht die Verteidigungswertung"] = {"CR_DEFENSE",},
			["Ausweichwertung"] = {"CR_DODGE",},
			["Erhöht Eure Ausweichwertung"] = {"CR_DODGE",},
			["Parierwertung"] = {"CR_PARRY",},
			["Increases your parry rating"] = {"CR_PARRY",},
			["Blockwertung"] = {"CR_BLOCK",},
			["Erhöht Eure Blockwertung"] = {"CR_BLOCK",},
			["Increases your shield block rating"] = {"CR_BLOCK",},
	
			["Trefferwertung"] = {"CR_HIT_MELEE",},
			["Erhöht Trefferwertung"] = {"CR_HIT_MELEE",}, -- ITEM_MOD_HIT_RATING
			["Erhöht Eure Trefferwertung"] = {"CR_HIT_MELEE",}, -- ITEM_MOD_HIT_MELEE_RATING
			["Zaubertrefferwertung"] = {"CR_HIT_SPELL",},
			["Erhöht Zaubertrefferwertung"] = {"CR_HIT_SPELL",}, -- ITEM_MOD_HIT_SPELL_RATING
			["Erhöht Eure Zaubertrefferwertung"] = {"CR_HIT_SPELL",},
			["Distanztrefferwertung"] = {"CR_HIT_RANGED",},
			["Erhöht Distanztrefferwertung"] = {"CR_HIT_RANGED",}, -- ITEM_MOD_HIT_RANGED_RATING
			["Erhöht Eure Distanztrefferwertung"] = {"CR_HIT_RANGED",},
	
			["kritische Trefferwertung"] = {"CR_CRIT_MELEE",},
			["Erhöht kritische Trefferwertung"] = {"CR_CRIT_MELEE",},
			["Erhöht Eure kritische Trefferwertung"] = {"CR_CRIT_MELEE",},
			["kritische Zaubertrefferwertung"] = {"CR_CRIT_SPELL",},
			["Erhöht kritische Zaubertrefferwertung"] = {"CR_CRIT_SPELL",},
			["Erhöht Eure kritische Zaubertrefferwertung"] = {"CR_CRIT_SPELL",},
			["Erhöht die kritische Zaubertrefferwertung aller Gruppenmitglieder innerhalb von 30 Metern"] = {"CR_CRIT_SPELL",},
			["Erhöht Eure kritische Distanztrefferwertung"] = {"CR_CRIT_RANGED",}, -- Fletcher's Gloves ID:7348
					
							--	["Improves hit avoidance rating"] = {"CR_HIT_TAKEN_MELEE"}, -- ITEM_MOD_HIT_TAKEN_RATING
							--	["Improves melee hit avoidance rating"] = {"CR_HIT_TAKEN_MELEE"}, -- ITEM_MOD_HIT_TAKEN_MELEE_RATING
							--	["Improves ranged hit avoidance rating"] = {"CR_HIT_TAKEN_RANGED"}, -- ITEM_MOD_HIT_TAKEN_RANGED_RATING
							--	["Improves spell hit avoidance rating"] = {"CR_HIT_TAKEN_SPELL"}, -- ITEM_MOD_HIT_TAKEN_SPELL_RATING
			["Abhärtung"] = {"CR_RESILIENCE",},
			["Abhärtungswertung"] = {"CR_RESILIENCE",},
			["Erhöht Eure Abhärtungswertung"] = {"CR_RESILIENCE",},
							--	["Improves critical avoidance rating"] = {"CR_CRIT_TAKEN_MELEE",},
							--	["Improves melee critical avoidance rating"] = {"CR_CRIT_TAKEN_MELEE",},
							--	["Improves ranged critical avoidance rating"] = {"CR_CRIT_TAKEN_RANGED",},
							--	["Improves spell critical avoidance rating"] = {"CR_CRIT_TAKEN_SPELL",},
	
			["Angriffstempowertung"] = {"CR_HASTE_MELEE"},
								["Spell Haste Rating"] = {"CR_HASTE_SPELL"},
			["Distanzangriffstempowertung"] = {"CR_HASTE_RANGED"},
			["Erhöht Angriffstempowertung"] = {"CR_HASTE_MELEE"},
			["Erhöht Eure Angriffstempowertung"] = {"CR_HASTE_MELEE"},
			["Erhöht Eure Distanzangriffstempowertung"] = {"CR_HASTE_SPELL"},
			["Erhöht Zaubertempowertung"] = {"CR_HASTE_RANGED"},
	
			["Erhöht die Fertigkeitswertung für Dolche"] = {"CR_WEAPON_DAGGER"},
			["Erhöht die Fertigkeitswertung für Schwerter"] = {"CR_WEAPON_SWORD"},
			["Erhöht die Fertigkeitswertung für Zweihandschwerter"] = {"CR_WEAPON_SWORD_2H"},
			["Erhöht die Fertigkeitswertung für Äxte"] = {"CR_WEAPON_AXE"},
			["Erhöht die Fertigkeitswertung für Zweihandäxte"] = {"CR_WEAPON_AXE_2H"},
			["Erhöht die Fertigkeitswertung für Kolben"] = {"CR_WEAPON_MACE"},
			["Erhöht die Fertigkeitswertung für Zweihandkolben"] = {"CR_WEAPON_MACE_2H"},
			["Erhöht die Fertigkeitswertung für Schusswaffen"] = {"CR_WEAPON_GUN"},
			["Erhöht die Fertigkeitswertung für Armbrüste"] = {"CR_WEAPON_CROSSBOW"},
			["Erhöht die Fertigkeitswertung für Bögen"] = {"CR_WEAPON_BOW"},
			["Erhöht die Fertigkeitswertung für 'Wilder Kampf'"] = {"CR_WEAPON_FERAL"},
			["Erhöht die Fertigkeitswertung für Faustwaffen"] = {"CR_WEAPON_FIST"}, -- Demonblood Eviscerator
			["Erhöht die Fertigkeitswertung für unbewaffneten Kampf"] = {"CR_WEAPON_FIST"}, -- Demonblood Eviscerator ID:27533
			-- Exclude
			["Sek"] = false,
			["bis"] = false,
			["Platz Tasche"] = false,
			["Platz Köcher"] = false,
			["Platz Munitionsbeutel"] = false,
			["Erhöht das Distanzangriffstempo"] = false, -- AV quiver
		},
		----------------
		-- Stat Names --
		----------------
		-- StatID are compatible with ItemBonusLib and BonusScanner
		["StatIDToName"] = {
			["STEALTH_LEVEL"] = {"Stealth Level", "Stealth"},
			["MELEE_DMG"] = {"Melee Weapon Damage", "Weapon Damage"},
			["MOUNT_SPEED%"] = {"Mount Speed(%)", "Mount Speed(%)"},
			["RUN_SPEED%"] = {"Run Speed(%)", "Run Speed(%)"},
	
			["STR"] = {SPELL_STAT1_NAME, "Str"},
			["AGI"] = {SPELL_STAT2_NAME, "Agi"},
			["STA"] = {SPELL_STAT3_NAME, "Sta"},
			["INT"] = {SPELL_STAT4_NAME, "Int"},
			["SPI"] = {SPELL_STAT5_NAME, "Spi"},
			["ARMOR"] = {ARMOR.." from items", "ARMOR(Items)"},
			["ARMOR_BUFF"] = {ARMOR.." from bonus", "ARMOR(Bonus)"},
	
			["FIRE_RES"] = {RESISTANCE2_NAME, "FR"},
			["NATURE_RES"] = {RESISTANCE3_NAME, "NR"},
			["FROST_RES"] = {RESISTANCE4_NAME, "FrR"},
			["SHADOW_RES"] = {RESISTANCE5_NAME, "SR"},
			["ARCANE_RES"] = {RESISTANCE6_NAME, "AR"},
	
			["FISHING"] = {"Fishing", "Fishing"},
			["MINING"] = {"Mining", "Mining"},
			["HERBALISM"] = {"Herbalism", "Herbalism"},
			["SKINNING"] = {"Skinning", "Skinning"},
	
			["BLOCK_VALUE"] = {"Block Value", "Block Value"},

			["AP"] = {ATTACK_POWER_TOOLTIP, "AP"},
			["RANGED_AP"] = {RANGED_ATTACK_POWER, "RAP"},
			["FERAL_AP"] = {ATTACK_POWER_TOOLTIP.." (Feral)", "AP(Feral)"},
			["AP_UNDEAD"] = {ATTACK_POWER_TOOLTIP.." (Undead)", "AP(Undead)"},
			["AP_DEMON"] = {ATTACK_POWER_TOOLTIP.." (Demon)", "AP(Demon)"},
	
			["HEAL"] = {"Healing", "Heal"},
	
			["SPELL_DMG"] = {PLAYERSTAT_SPELL_COMBAT.." "..DAMAGE, PLAYERSTAT_SPELL_COMBAT.." Dmg"},
			["SPELL_DMG_UNDEAD"] = {PLAYERSTAT_SPELL_COMBAT.." "..DAMAGE.." (Undead)", PLAYERSTAT_SPELL_COMBAT.." Dmg".." (Undead)"},
			["HOLY_SPELL_DMG"] = {SPELL_SCHOOL1_CAP.." "..DAMAGE, SPELL_SCHOOL1_CAP.." Dmg"},
			["FIRE_SPELL_DMG"] = {SPELL_SCHOOL2_CAP.." "..DAMAGE, SPELL_SCHOOL2_CAP.." Dmg"},
			["NATURE_SPELL_DMG"] = {SPELL_SCHOOL3_CAP.." "..DAMAGE, SPELL_SCHOOL3_CAP.." Dmg"},
			["FROST_SPELL_DMG"] = {SPELL_SCHOOL4_CAP.." "..DAMAGE, SPELL_SCHOOL4_CAP.." Dmg"},
			["SHADOW_SPELL_DMG"] = {SPELL_SCHOOL5_CAP.." "..DAMAGE, SPELL_SCHOOL5_CAP.." Dmg"},
			["ARCANE_SPELL_DMG"] = {SPELL_SCHOOL6_CAP.." "..DAMAGE, SPELL_SCHOOL6_CAP.." Dmg"},
	
			["SPELLPEN"] = {PLAYERSTAT_SPELL_COMBAT.." "..SPELL_PENETRATION, SPELL_PENETRATION},
	
			["HEALTH_REG"] = {HEALTH.." Regen", "HP5"},
			["MANA_REG"] = {MANA.." Regen", "MP5"},
			["HEALTH"] = {HEALTH, HP},
			["MANA"] = {MANA, MP},
	
			["MAX_DAMAGE"] = {"Max Damage", "Max Dmg"},
			["DPS"] = {"Damage Per Second", "DPS"},
	
			["CR_DEFENSE"] = {COMBAT_RATING_NAME2, COMBAT_RATING_NAME2}, -- COMBAT_RATING_NAME2 = "Defense Rating"
			["CR_DODGE"] = {COMBAT_RATING_NAME3, COMBAT_RATING_NAME3}, -- COMBAT_RATING_NAME3 = "Dodge Rating"
			["CR_PARRY"] = {COMBAT_RATING_NAME4, COMBAT_RATING_NAME4}, -- COMBAT_RATING_NAME4 = "Parry Rating"
			["CR_BLOCK"] = {COMBAT_RATING_NAME5, COMBAT_RATING_NAME5}, -- COMBAT_RATING_NAME5 = "Block Rating"
			["CR_HIT_MELEE"] = {COMBAT_RATING_NAME6, COMBAT_RATING_NAME6}, -- COMBAT_RATING_NAME6 = "Hit Rating"
			["CR_HIT_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." "..COMBAT_RATING_NAME6, PLAYERSTAT_RANGED_COMBAT.." "..COMBAT_RATING_NAME6}, -- PLAYERSTAT_RANGED_COMBAT = "Ranged"
			["CR_HIT_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." "..COMBAT_RATING_NAME6, PLAYERSTAT_SPELL_COMBAT.." "..COMBAT_RATING_NAME6}, -- PLAYERSTAT_SPELL_COMBAT = "Spell"
			["CR_HIT_TAKEN_MELEE"] = {"Hit Avoidance "..RATING, "Hit Avoidance "..RATING},
			["CR_HIT_TAKEN_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Hit Avoidance "..RATING, PLAYERSTAT_RANGED_COMBAT.." Hit Avoidance "..RATING},
			["CR_HIT_TAKEN_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Hit Avoidance "..RATING, PLAYERSTAT_SPELL_COMBAT.." Hit Avoidance "..RATING},
			["CR_CRIT_MELEE"] = {COMBAT_RATING_NAME9, COMBAT_RATING_NAME9}, -- COMBAT_RATING_NAME9 = "Crit Rating"
			["CR_CRIT_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." "..COMBAT_RATING_NAME9, PLAYERSTAT_RANGED_COMBAT.." "..COMBAT_RATING_NAME9},
			["CR_CRIT_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." "..COMBAT_RATING_NAME9, PLAYERSTAT_SPELL_COMBAT.." "..COMBAT_RATING_NAME9},
			["CR_CRIT_TAKEN_MELEE"] = {"Crit Avoidance "..RATING, "Crit Avoidance "..RATING},
			["CR_CRIT_TAKEN_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Crit Avoidance "..RATING, PLAYERSTAT_RANGED_COMBAT.." Crit Avoidance "..RATING},
			["CR_CRIT_TAKEN_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Crit Avoidance "..RATING, PLAYERSTAT_SPELL_COMBAT.." Crit Avoidance "..RATING},
			["CR_RESILIENCE"] = {COMBAT_RATING_NAME15, COMBAT_RATING_NAME15}, -- COMBAT_RATING_NAME15 = "Resilience"
			["CR_HASTE_MELEE"] = {"Haste "..RATING, "Haste "..RATING}, --
			["CR_HASTE_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Haste "..RATING, PLAYERSTAT_RANGED_COMBAT.." Haste "..RATING},
			["CR_HASTE_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Haste "..RATING, PLAYERSTAT_SPELL_COMBAT.." Haste "..RATING},
			["CR_WEAPON_DAGGER"] = {"Dagger "..SKILL.." "..RATING, "Dagger "..SKILL.." "..RATING}, -- SKILL = "Skill"
			["CR_WEAPON_SWORD"] = {"Sword "..SKILL.." "..RATING, "Sword "..SKILL.." "..RATING},
			["CR_WEAPON_SWORD_2H"] = {"Sword "..SKILL.." "..RATING, "Sword "..SKILL.." "..RATING},
			["CR_WEAPON_AXE"] = {"Axe "..SKILL.." "..RATING, "Axe "..SKILL.." "..RATING},
			["CR_WEAPON_AXE_2H"] = {"Axe "..SKILL.." "..RATING, "Axe "..SKILL.." "..RATING},
			["CR_WEAPON_MACE"] = {"Mace "..SKILL.." "..RATING, "Mace "..SKILL.." "..RATING},
			["CR_WEAPON_MACE_2H"] = {"Mace "..SKILL.." "..RATING, "Mace "..SKILL.." "..RATING},
			["CR_WEAPON_GUN"] = {"Gun "..SKILL.." "..RATING, "Gun "..SKILL.." "..RATING},
			["CR_WEAPON_CROSSBOW"] = {"Crossbow "..SKILL.." "..RATING, "Crossbow "..SKILL.." "..RATING},
			["CR_WEAPON_BOW"] = {"Bow "..SKILL.." "..RATING, "Bow "..SKILL.." "..RATING},
			["CR_WEAPON_FERAL"] = {"Feral "..SKILL.." "..RATING, "Feral "..SKILL.." "..RATING},
			["CR_WEAPON_FIST"] = {"Unarmed "..SKILL.." "..RATING, "Unarmed "..SKILL.." "..RATING},
	
			-- Tier2 Stats
			-- Str -> AP, Block Value
			-- Agi -> AP, Crit, Dodge
			-- Sta -> Health
			-- Int -> Mana, Spell Crit
			-- Spi -> mp5oc, hp5oc
			-- Ratings -> Effect
			["DEFENSE"] = {DEFENSE, "Def"},
			["DODGE"] = {DODGE_CHANCE, DODGE},
			["PARRY"] = {PARRY_CHANCE, PARRY},
			["BLOCK"] = {BLOCK_CHANCE, BLOCK},
			["HIT_MELEE"] = {"Hit Chance", "Hit"},
			["HIT_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." ".."Hit Chance", PLAYERSTAT_RANGED_COMBAT.." Hit"},
			["HIT_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." ".."Hit Chance", PLAYERSTAT_SPELL_COMBAT.." Hit"},
			["HIT_TAKEN_MELEE"] = {"Hit Avoidance", "Hit Avoidance"},
			["HIT_TAKEN_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Hit Avoidance", PLAYERSTAT_RANGED_COMBAT.." Hit Avoidance"},
			["HIT_TAKEN_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Hit Avoidance", PLAYERSTAT_SPELL_COMBAT.." Hit Avoidance"},
			["CRIT_MELEE"] = {MELEE_CRIT_CHANCE, "Crit"}, -- MELEE_CRIT_CHANCE = "Crit Chance"
			["CRIT_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." "..MELEE_CRIT_CHANCE, PLAYERSTAT_RANGED_COMBAT.." Crit"},
			["CRIT_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." "..MELEE_CRIT_CHANCE, PLAYERSTAT_SPELL_COMBAT.." Crit"},
			["CRIT_TAKEN_MELEE"] = {"Crit Avoidance", "Crit Avoidance"},
			["CRIT_TAKEN_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Crit Avoidance", PLAYERSTAT_RANGED_COMBAT.." Crit Avoidance"},
			["CRIT_TAKEN_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Crit Avoidance", PLAYERSTAT_SPELL_COMBAT.." Crit Avoidance"},
			["HASTE_MELEE"] = {"Haste", "Haste"}, --
			["HASTE_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Haste", PLAYERSTAT_RANGED_COMBAT.." Haste"},
			["HASTE_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Haste", PLAYERSTAT_SPELL_COMBAT.." Haste"},
			["WEAPON_DAGGER"] = {"Dagger "..SKILL, "Dagger"}, -- SKILL = "Skill"
			["WEAPON_SWORD"] = {"Sword "..SKILL, "Sword"},
			["WEAPON_SWORD_2H"] = {"Sword "..SKILL, "Sword"},
			["WEAPON_AXE"] = {"Axe "..SKILL, "Axe"},
			["WEAPON_AXE_2H"] = {"Axe "..SKILL, "Axe"},
			["WEAPON_MACE"] = {"Mace "..SKILL, "Mace"},
			["WEAPON_MACE_2H"] = {"Mace "..SKILL, "Mace"},
			["WEAPON_GUN"] = {"Gun "..SKILL, "Gun"},
			["WEAPON_CROSSBOW"] = {"Crossbow "..SKILL, "Crossbow"},
			["WEAPON_BOW"] = {"Bow "..SKILL, "Bow"},
			["WEAPON_FERAL"] = {"Feral "..SKILL, "Feral"},
			["WEAPON_FIST"] = {"Unarmed "..SKILL, "Unarmed"},
			["CRIT_DAMAGE_TAKEN_MELEE"] = {"Crit Damage Taken", "Crit Damage Taken"},
			["CRIT_DAMAGE_TAKEN_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Crit Damage Taken", PLAYERSTAT_RANGED_COMBAT.." Crit Damage Taken"},
			["CRIT_DAMAGE_TAKEN_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Crit Damage Taken", PLAYERSTAT_SPELL_COMBAT.." Crit Damage Taken"},
	
			-- Tier3 Stats
			-- Defense -> Crit Avoidance, Hit Avoidance, Dodge, Parry, Block
			-- Weapon Skill -> Crit, Hit, Mob Dodge, Mob Parry, Mob Block
			--
		},
	}
end)

--[[
L:RegisterTranslations("frFR", function()
	return {
	}
end)

L:RegisterTranslations("esES", function()
	return {
	}
end)
--]]

L:Debug()

--------------------
-- Initialization --
--------------------
local StatLogic = {}
local cache = {}
setmetatable(cache, {__mode = "kv"}) -- weak table to enable garbage collection
local tip

-- Called when a newer version is registered
local function activate(self, oldLib)
	if not oldLib then
		-- Create a custom tooltip for scanning
		tip = CreateFrame("GameTooltip", "StatLogicTooltip", nil, "GameTooltipTemplate")
		self.tip = tip
		tip:SetOwner(UIParent, "ANCHOR_NONE")
		for i = 1, 30 do
			tip[i] = _G["StatLogicTooltipTextLeft"..i]
			if not tip[i] then
				tip[i] = tip:CreateFontString()
				tip:AddFontStrings(tip[i], tip:CreateFontString())
			end
		end
	else
		tip = oldLib.tip
		self.tip = tip
	end
end


-----------
-- Tools --
-----------
local DEBUG = false

-- Localize globals
local _G = getfenv(0)
local strfind = strfind
local strsub = strsub
local strupper = strupper
local strmatch = strmatch
local gmatch = gmatch
local gsub = gsub
local strutf8sub = string.utf8sub
local pairs = pairs
local ipairs = ipairs
local type = type
local tonumber = L.tonumber
local GetInventoryItemLink = GetInventoryItemLink
local GetItemInfo = GetItemInfo
local GetLocale = GetLocale
local IsUsableSpell = IsUsableSpell
local UnitLevel = UnitLevel
local UnitClass = UnitClass
local UnitRace = UnitRace
local GetShapeshiftForm = GetShapeshiftForm
local GetShapeshiftFormInfo = GetShapeshiftFormInfo
local GetPlayerBuffName = GetPlayerBuffName
local GetTalentInfo = GetTalentInfo

-- metatable for stat tables
local statTableMetatable = {
	__add = function(op1, op2)
		for k, v in pairs(op2) do
			if type(v) == "number" then
				op1[k] = (op1[k] or 0) + v
				if op1[k] == 0 then
					op1[k] = nil
				end
			end
		end
		return op1
	end,
	__sub = function(op1, op2)
		for k, v in pairs(op2) do
			if type(v) == "number" then
				op1[k] = (op1[k] or 0) - v
				if op1[k] == 0 then
					op1[k] = nil
				end
			elseif k == "itemType" then
				local i = 1
				while op1["diffItemType"..i] do
					i = i + 1
				end
				op1["diffItemType"..i] = op2.itemType
			end
		end
		return op1
	end,
}

-- Table pool borrowed from Tablet-2.0 (ckknight) --
local pool = {}

-- Delete table and return to pool
local function del(t)
	if t then
		for k in pairs(t) do
			t[k] = nil
		end
		setmetatable(t, nil)
		pool[t] = true
	end
end

local function delMulti(t)
	if t then
		for k in pairs(t) do
			if type(t[k]) == "table" then
				del(t[k])
			else
				t[k] = nil
			end
		end
		setmetatable(t, nil)
		pool[t] = true
	end
end

-- Copy table
local function copy(parent)
	local t = next(pool) or {}
	pool[t] = nil
	if parent then
		for k,v in pairs(parent) do
			t[k] = v
		end
		setmetatable(t, getmetatable(parent))
	end
	return t
end

-- New table
local function new(...)
	local t = next(pool) or {}
	pool[t] = nil

	for i = 1, select('#', ...), 2 do
		local k = select(i, ...)
		if k then
			t[k] = select(i+1, ...)
		else
			break
		end
	end
	return t
end

-- New stat table
local function newStatTable(...)
	local t = next(pool) or {}
	pool[t] = nil
	setmetatable(t, statTableMetatable)

	for i = 1, select('#', ...), 2 do
		local k = select(i, ...)
		if k then
			t[k] = select(i+1, ...)
		else
			break
		end
	end
	return t
end

StatLogic.StatTable = {}
StatLogic.StatTable.new = newStatTable
StatLogic.StatTable.del = del
StatLogic.StatTable.copy = copy

-- End of Table pool --

-- copyTable
local function copyTable(to, from)
	if to then
		for k in pairs(to) do
			to[k] = nil
		end
		setmetatable(to, nil)
	else
		to = {}
	end
	for k,v in pairs(from) do
		if type(k) == "table" then
			k = copyTable({}, k)
		end
		if type(v) == "table" then
			v = copyTable({}, v)
		end
		to[k] = v
	end
	setmetatable(to, getmetatable(from))
	return to
end

local function print(text)
	if DEBUG == true then
		DEFAULT_CHAT_FRAME:AddMessage(text)
	end
end

-- SetTip("item:3185:0:0:0:0:0:1957")
function SetTip(item)
	local name, link, _, _, reqLv, _, _, _, itemType = GetItemInfo(item)
	ItemRefTooltip:ClearLines()
	ItemRefTooltip:SetHyperlink(link)
	ItemRefTooltip:Show()
end
-- End of Tools
--------------------

--============--
-- Base Stats --
--============--
--[[
local RaceClassStatBase = {
	-- The Human Spirit - Increase Spirit by 5%
	Human = { --{20, 20, 20, 20, 22(21)}
		WARRIOR = { --{3, 0, 2, 0, 0}
			{23, 20, 22, 20, 22}
		},
		PALADIN = { --{2, 0, 2, 0, 1}
			{22, 20, 22, 20, 23}
		},
		ROGUE = { --{1, 3, 1, 0, 0}
			{21, 23, 21, 20, 22}
		},
		PRIEST = { --{0, 0, 0, 2, 3}
			{20, 20, 20, 22, 25}
		},
		MAGE = { --{0, 0, 0, 3, 2}
			{20, 20, 20, 23, 24}
		},
		WARLOCK = { --{0, 0, 1, 2, 2}
			{20, 20, 21, 22, 24}
		},
	},
	Dwarf = { --{22, 16, 23, 19, 19}
		WARRIOR = {
			{25, 16, 25, 19, 19}
		},
		PALADIN = {
			{24, 16, 25, 19, 20}
		},
		HUNTER = { --{0, 3, 1, 0, 1}
			{22, 19, 24, 19, 20}
		},
		ROGUE = {
			{23, 19, 24, 19, 19}
		},
		PRIEST = {
			{22, 16, 23, 21, 22}
		},
	},
	NightElf = { --{17, 25, 19, 20, 20}
		WARRIOR = {--{3, 0, 2, 0, 0}
			{20, 25, 21, 20, 20}
		},
		HUNTER = {
			{17, 28, 20, 20, 21}
		},
		ROGUE = {
			{18, 28, 20, 20, 20}
		},
		PRIEST = {
			{17, 25, 19, 22, 23}
		},
		DRUID = { --{1, 0, 0, 2, 2}
			{18, 25, 19, 22, 22}
		},
	},
	-- Expansive Mind - Increase Intelligence by 5%
	Gnome = { --{15, 23, 19, 24, 20}
		WARRIOR = {--{3, 0, 2, 0, 0}
			{18, 23, 21, 24, 20}
		},
		ROGUE = {
			{, , , , }
		},
		MAGE = {
			{, , , , }
		},
		WARLOCK = {
			{, , , , }
		},
	},
	Draenei = { --{21, 22, 21, 16, 21}
		WARRIOR = {--{3, 0, 2, 0, 0}
			{24, 22, 23, 16, 21}
		},
		PALADIN = { --{2, -4, 0, 5, 2}
			{23, 17, 21, 21, 23}
		},
		HUNTER = { --{0, -2, -1, 5, 2}
			{21, 20, 20, 21, 23}
		},
		PRIEST = { --{0, -5, -2, 7, 4}
			{21, 17, 19, 23, 25}
		},
		SHAMAN = { --{5, -7, 2, 0, 3}
			{26, 15, 23, 16, 24}
		},
		MAGE = { --{0, -5, -2, 8, 3}
			{21, 17, 19, 24, 24}
		},
	},
	Orc = { --{23, 17, 22, 17, 23}
		WARRIOR = {--{3, 0, 2, 0, 0}
			{26, 17, 24, 17, 23}
		},
		HUNTER = { --{0, 3, 1, 0, 1}
			{23, 20, 23, 17, 24}
		},
		ROGUE = { --{1, 3, 1, 0, 0}
			{, , , , }
		},
		SHAMAN = { --{1, 0, 1, 1, 2}
			{24, 17, 23, 18, 25}
		},
		WARLOCK = { --{0, 0, 1, 2, 2}
			{, , , , }
		},
	},
	Scourge = { --{19, 18, 21, 18, 25}
		WARRIOR = {--{3, 0, 2, 0, 0}
			{22, 18, 23, 18, 25}
		},
		ROGUE = {
			{, , , , }
		},
		PRIEST = {
			{, , , , }
		},
		MAGE = {
			{, , , , }
		},
		WARLOCK = {
			{, , , , }
		},
	},
	Tauren = { --{25, 15, 22, 15, 22}
		WARRIOR = {--{3, 0, 2, 0, 0}
			{28, 15, 24, 15, 22}
		},
		HUNTER = { --{0, 3, 1, 0, 1}
			{, , , , }
		},
		SHAMAN = {
			{, , , , }
		},
		DRUID = { --{1, 0, 0, 2, 2}
			{26, 15, 22, 17, 24}
		},
	},
	Troll = { --{21, 22, 21, 16, 21}
		WARRIOR = {--{3, 0, 2, 0, 0}
			{24, 22, 23, 16, 21}
		},
		HUNTER = { --{0, 3, 1, 0, 1}
			{, , , , }
		},
		ROGUE = {
			{, , , , }
		},
		PRIEST = {
			{, , , , }
		},
		SHAMAN = {
			{, , , , }
		},
		MAGE = {
			{, , , , }
		},
	},
	BloodElf = { --{22, 16, 23, 19, 19}
		PALADIN = {--{2, 0, 2, 0, 1}
			{24, 16, 25, 19, 20}
		},
		HUNTER = { --{-1, 7, -1, -3, 3}
			{21, 25, 22, 16, 22}
		},
		ROGUE = {
			{, , , , }
		},
		PRIEST = {
			{, , , , }
		},
		MAGE = {
			{, , , , }
		},
		WARLOCK = {
			{, , , , }
		},
	},
}
--]]
--============--
-- Stat Tools --
--============--
local _, class = UnitClass("player")
local _, race = UnitRace("player")

local function GetStanceIcon()
	local currentStance = GetShapeshiftForm()
	if currentStance ~= 0 then
		return GetShapeshiftFormInfo(currentStance)
	end
end

local function GetPlayerBuffRank(buff)
	local hasBuff, rank = GetPlayerBuffName(buff)
	if hasBuff then
		return strmatch(rank, "(%d+)") or 1
	end
end

local function GetTotalDefense(unit)
	local base, modifier = UnitDefense(unit);
	return base + modifier
end

--==================================--
-- Stat Mods from Talants and Buffs --
--==================================--
--[[ Aura mods from Thottbot
Apply Aura: Mod Total Stat % (All stats)
Apply Aura: Mod Total Stat % (Strength)
Apply Aura: Mod Total Stat % (Agility)
Apply Aura: Mod Total Stat % (Stamina)
Apply Aura: Mod Total Stat % (Intellect)
Apply Aura: Mod Total Stat % (Spirit)
Apply Aura: Mod Max Health %
Apply Aura: Reduces Attacker Chance to Hit with Melee
Apply Aura: Reduces Attacker Chance to Hit with Ranged
Apply Aura: Reduces Attacker Chance to Hit with Spells (Spells)
Apply Aura: Reduces Attacker Chance to Crit with Melee
Apply Aura: Reduces Attacker Chance to Crit with Ranged
Apply Aura: Reduces Attacker Critical Hit Damage with Melee by %
Apply Aura: Reduces Attacker Critical Hit Damage with Ranged by %
Apply Aura: Mod Dmg % (Spells)
Apply Aura: Mod Dmg % Taken (Fire, Frost)
Apply Aura: Mod Dmg % Taken (Spells)
Apply Aura: Mod Dmg % Taken (All)
Apply Aura: Mod Dmg % Taken (Physical)
Apply Aura: Mod Base Resistance % (Physical)
Apply Aura: Mod Block Percent
Apply Aura: Mod Parry Percent
Apply Aura: Mod Dodge Percent
Apply Aura: Mod Shield Block %
Apply Aura: Mod Detect
Apply Aura: Mod Skill Talent (Defense)
--]]
--[[ StatModAuras, mods not in use are commented out for now
"MOD_STR",
"MOD_AGI",
"MOD_STA",
"MOD_INT",
"MOD_SPI",
"MOD_HEALTH",
"MOD_MANA",
"MOD_ARMOR",
"MOD_BLOCK_VALUE",
--"MOD_DMG", school,
"MOD_DMG_TAKEN", school,
--"MOD_CRIT_DAMAGE", school,
"MOD_CRIT_DAMAGE_TAKEN", school,
--"MOD_THREAT", school,

"ADD_DODGE",
--"ADD_PARRY",
--"ADD_BLOCK",
--"ADD_STEALTH_DETECT",
--"ADD_STEALTH",
--"ADD_DEFENSE",
--"ADD_THREAT", school,
"ADD_HIT_TAKEN", school,
"ADD_CRIT_TAKEN", school,

--Talents
ADD_SPELL_DMG_MOD_INT
ADD_HEALING_MOD_INT
ADD_MANA_REG_MOD_INT
ADD_RANGED_AP_MOD_INT
ADD_ARMOR_MOD_INT
ADD_SPELL_DMG_MOD_STA
ADD_SPELL_DMG_MOD_SPI
ADD_HEALING_MOD_SPI
ADD_HEALING_MOD_STR
ADD_MANA_REG_MOD_NORMAL_MANA_REG
MOD_AP
MOD_RANGED_AP
MOD_SPELL_DMG
MOD_HEALING
--]]

local StatModInfo = {
	------------------------------------------------------------------------------
	-- initialValue: sets the initial value for the stat mod
	-- if initialValue == 0, inter-mod operations are done with addition,
	-- if initialValue == 1, inter-mod operations are done with multiplication,
	------------------------------------------------------------------------------
	-- finalAdjust: added to the final result before returning,
	-- so we can adjust the return value to be used in addition or multiplication
	-- for addition: initialValue + finalAdjust = 0
	-- for multiplication: initialValue + finalAdjust = 1
	------------------------------------------------------------------------------
	-- school: school arg is required for these mods
	------------------------------------------------------------------------------
	["ADD_CRIT_TAKEN"] = {
		initialValue = 0,
		finalAdjust = 0,
		school = true,
	},
	["ADD_HIT_TAKEN"] = {
		initialValue = 0,
		finalAdjust = 0,
		school = true,
	},
	["ADD_DODGE"] = {
		initialValue = 0,
		finalAdjust = 0,
	},
	["ADD_SPELL_DMG_MOD_INT"] = {
		initialValue = 0,
		finalAdjust = 0,
	},
	["ADD_HEALING_MOD_INT"] = {
		initialValue = 0,
		finalAdjust = 0,
	},
	["ADD_MANA_REG_MOD_INT"] = {
		initialValue = 0,
		finalAdjust = 0,
	},
	["ADD_RANGED_AP_MOD_INT"] = {
		initialValue = 0,
		finalAdjust = 0,
	},
	["ADD_ARMOR_MOD_INT"] = {
		initialValue = 0,
		finalAdjust = 0,
	},
	["ADD_SPELL_DMG_MOD_STA"] = {
		initialValue = 0,
		finalAdjust = 0,
	},
	["ADD_SPELL_DMG_MOD_SPI"] = {
		initialValue = 0,
		finalAdjust = 0,
	},
	["ADD_HEALING_MOD_SPI"] = {
		initialValue = 0,
		finalAdjust = 0,
	},
	["ADD_HEALING_MOD_STR"] = {
		initialValue = 0,
		finalAdjust = 0,
	},
	["ADD_MANA_REG_MOD_NORMAL_MANA_REG"] = {
		initialValue = 0,
		finalAdjust = 0,
	},
	["MOD_CRIT_DAMAGE_TAKEN"] = {
		initialValue = 0,
		finalAdjust = 1,
		school = true,
	},
	["MOD_DMG_TAKEN"] = {
		initialValue = 0,
		finalAdjust = 1,
		school = true,
	},
	["MOD_CRIT_DAMAGE"] = {
		initialValue = 0,
		finalAdjust = 1,
		school = true,
	},
	["MOD_DMG"] = {
		initialValue = 0,
		finalAdjust = 1,
		school = true,
	},
	["MOD_ARMOR"] = {
		initialValue = 1,
		finalAdjust = 0,
	},
	["MOD_HEALTH"] = {
		initialValue = 1,
		finalAdjust = 0,
	},
	["MOD_MANA"] = {
		initialValue = 1,
		finalAdjust = 0,
	},
	["MOD_STR"] = {
		initialValue = 0,
		finalAdjust = 1,
	},
	["MOD_AGI"] = {
		initialValue = 0,
		finalAdjust = 1,
	},
	["MOD_STA"] = {
		initialValue = 0,
		finalAdjust = 1,
	},
	["MOD_INT"] = {
		initialValue = 0,
		finalAdjust = 1,
	},
	["MOD_SPI"] = {
		initialValue = 0,
		finalAdjust = 1,
	},
	["MOD_BLOCK_VALUE"] = {
		initialValue = 0,
		finalAdjust = 1,
	},
	["MOD_AP"] = {
		initialValue = 0,
		finalAdjust = 1,
	},
	["MOD_RANGED_AP"] = {
		initialValue = 0,
		finalAdjust = 1,
	},
	["MOD_SPELL_DMG"] = {
		initialValue = 0,
		finalAdjust = 1,
	},
	["MOD_HEALING"] = {
		initialValue = 0,
		finalAdjust = 1,
	},
}

------------------
-- StatModTable --
------------------
local StatModTable = {
	["DRUID"] = {
		-- Druid: Lunar Guidance (Rank 3) - 1,12
		--        Increases your spell damage and healing by 8%/16%/25% of your total Intellect.
		["ADD_SPELL_DMG_MOD_INT"] = {
			[1] = {
				["tab"] = 1,
				["num"] = 12,
				["rank"] = {
					0.08, 0.16, 0.25,
				},
			},
		},
		-- Druid: Lunar Guidance (Rank 3) - 1,12
		--        Increases your spell damage and healing by 8%/16%/25% of your total Intellect.
		["ADD_HEALING_MOD_INT"] = {
			[1] = {
				["tab"] = 1,
				["num"] = 12,
				["rank"] = {
					0.08, 0.16, 0.25,
				},
			},
		},
		-- Druid: Nurturing Instinct (Rank 2) - 2,14
		--        Increases your healing spells by up to 25%/50% of your Strength.
		["ADD_HEALING_MOD_STR"] = {
			[1] = {
				["tab"] = 2,
				["num"] = 14,
				["rank"] = {
					0.25, 0.50,
				},
			},
		},
		-- Druid: Intensity (Rank 3) - 3,6
		--        Allows 5%/10%/15% of your Mana regeneration to continue while casting and causes your Enrage ability to instantly generate 10 rage. 
		["ADD_MANA_REG_MOD_NORMAL_MANA_REG"] = {
			[1] = {
				["tab"] = 3,
				["num"] = 6,
				["rank"] = {
					0.05, 0.10, 0.15,
				},
			},
		},
		-- Druid: Dreamstate (Rank 3) - 1,17
		--        Regenerate mana equal to 4%/7%/10% of your Intellect every 5 sec, even while casting.
		["ADD_MANA_REG_MOD_INT"] = {
			[1] = {
				["tab"] = 1,
				["num"] = 17,
				["rank"] = {
					0.04, 0.07, 0.10,
				},
			},
		},
		-- Druid: Feral Swiftness (Rank 2) - 2,6
		--        Increases your movement speed by 15%/30% while outdoors in Cat Form and increases your chance to dodge while in Cat Form, Bear Form and Dire Bear Form by 2%/4%.
		["ADD_DODGE"] = {
			[1] = {
				["tab"] = 2,
				["num"] = 6,
				["rank"] = {
					2, 4,
				},
				["buff"] = BS["Bear Form"],
			},
			[2] = {
				["tab"] = 2,
				["num"] = 6,
				["rank"] = {
					2, 4,
				},
				["buff"] = BS["Dire Bear Form"],
			},
			[3] = {
				["tab"] = 2,
				["num"] = 6,
				["rank"] = {
					2, 4,
				},
				["buff"] = BS["Cat Form"],
			},
		},
		-- Druid: Survival of the Fittest (Rank 3) - 2,16
		--        Increases all attributes by 1%/2%/3% and reduces the chance you'll be critically hit by melee attacks by 1%/2%/3%.
		["ADD_CRIT_TAKEN"] = {
			[1] = {
				["MELEE"] = true,
				["tab"] = 2,
				["num"] = 16,
				["rank"] = {
					-0.01, -0.02, -0.03,
				},
			},
		},
		-- Druid: Natural Perfection (Rank 3) - 3,18
		--        Your critical strike chance with all spells is increased by 3% and melee and ranged critical strikes against you cause 4%/7%/10% less damage.
		["MOD_CRIT_DAMAGE_TAKEN"] = {
			[1] = {
				["MELEE"] = true,
				["RANGED"] = true,
				["tab"] = 3,
				["num"] = 18,
				["rank"] = {
					-0.04, -0.07, -0.1,
				},
			},
		},
		-- Druid: Balance of Power (Rank 2) - 1,16
		--        Increases your chance to hit with all spells and reduces the chance you'll be hit by spells by 2%/4%.
		["ADD_HIT_TAKEN"] = {
			[1] = {
				["HOLY"] = true,
				["FIRE"] = true,
				["NATURE"] = true,
				["FROST"] = true,
				["SHADOW"] = true,
				["ARCANE"] = true,
				["tab"] = 1,
				["num"] = 16,
				["rank"] = {
					-0.02, -0.04,
				},
			},
		},
		-- Druid: Thick Hide (Rank 3) - 2,5
		--        Increases your Armor contribution from items by 4%/7%/10%.
		-- Druid: Bear Form - buff (didn't use stance because Bear Form and Dire Bear Form has the same icon)
		--        Shapeshift into a bear, increasing melee attack power by 30, armor contribution from items by 180%, and stamina by 25%.
		-- Druid: Dire Bear Form - Buff
		--        Shapeshift into a dire bear, increasing melee attack power by 120, armor contribution from items by 400%, and stamina by 25%.
		-- Druid: Moonkin Form - Buff
		--        While in this form the armor contribution from items is increased by 400%, attack power is increased by 150% of your level and all party members within 30 yards have their spell critical chance increased by 5%.
		["MOD_ARMOR"] = {
			[1] = {
				["tab"] = 2,
				["num"] = 5,
				["rank"] = {
					1.04, 1.07, 1.1,
				},
			},
			[2] = {
				["rank"] = {
					2.8,
				},
				["buff"] = BS["Bear Form"],
			},
			[3] = {
				["rank"] = {
					5,
				},
				["buff"] = BS["Dire Bear Form"],
			},
			[4] = {
				["rank"] = {
					5,
				},
				["buff"] = BS["Moonkin Form"],
			},
		},
		-- Druid: Heart of the Wild (Rank 5) - 2,15
		--        Increases your Intellect by 4%/8%/12%/16%/20%. In addition, while in Bear or Dire Bear Form your Stamina is increased by 4%/8%/12%/16%/20% and while in Cat Form your Strength is increased by 4%/8%/12%/16%/20%.
		-- Druid: Bear Form - Stance (use stance because bear and dire bear increases are the same)
		--        Shapeshift into a bear, increasing melee attack power by 30, armor contribution from items by 180%, and stamina by 25%.
		-- Druid: Dire Bear Form - Stance (use stance because bear and dire bear increases are the same)
		--        Shapeshift into a dire bear, increasing melee attack power by 120, armor contribution from items by 400%, and stamina by 25%.
		-- Druid: Survival of the Fittest (Rank 3) - 2,16
		--        Increases all attributes by 1%/2%/3% and reduces the chance you'll be critically hit by melee attacks by 1%/2%/3%.
		["MOD_STA"] = { -- Heart of the Wild: +4%/8%/12%/16%/20% stamina in bear / dire bear
			[1] = {
				["tab"] = 2,
				["num"] = 15,
				["rank"] = {
					0.04, 0.08, 0.12, 0.16, 0.2,
				},
				["buff"] = BS["Bear Form"],
			},
			[2] = {
				["tab"] = 2,
				["num"] = 15,
				["rank"] = {
					0.04, 0.08, 0.12, 0.16, 0.2,
				},
				["buff"] = BS["Dire Bear Form"],
			},
			[3] = { -- Survival of the Fittest: +1%/2%/3% all stats
				["tab"] = 2,
				["num"] = 16,
				["rank"] = {
					0.01, 0.02, 0.03,
				},
			},
			[4] = { -- Bear Form / Dire Bear Form: +25% stamina
				["rank"] = {
					0.25,
				},
				["buff"] = BS["Bear Form"],
			},
			[5] = { -- Bear Form / Dire Bear Form: +25% stamina
				["rank"] = {
					0.25,
				},
				["buff"] = BS["Dire Bear Form"],
			},
		},
		-- Druid: Heart of the Wild (Rank 5) - 2,15
		--        Increases your Intellect by 4%/8%/12%/16%/20%. In addition, while in Bear or Dire Bear Form your Stamina is increased by 4%/8%/12%/16%/20% and while in Cat Form your Strength is increased by 4%/8%/12%/16%/20%.
		-- Druid: Survival of the Fittest (Rank 3) - 2,16
		--        Increases all attributes by 1%/2%/3% and reduces the chance you'll be critically hit by melee attacks by 1%/2%/3%.
		["MOD_STR"] = {
			[1] = {
				["tab"] = 2,
				["num"] = 15,
				["rank"] = {
					0.04, 0.08, 0.12, 0.16, 0.2,
				},
				["buff"] = BS["Cat Form"],
			},
			[2] = {
				["tab"] = 2,
				["num"] = 16,
				["rank"] = {
					0.01, 0.02, 0.03,
				},
			},
		},
		-- Druid: Survival of the Fittest (Rank 3) - 2,16
		--        Increases all attributes by 1%/2%/3% and reduces the chance you'll be critically hit by melee attacks by 1%/2%/3%.
		["MOD_AGI"] = {
			[1] = {
				["tab"] = 2,
				["num"] = 16,
				["rank"] = {
					0.01, 0.02, 0.03,
				},
			},
		},
		-- Druid: Heart of the Wild (Rank 5) - 2,15
		--        Increases your Intellect by 4%/8%/12%/16%/20%. In addition, while in Bear or Dire Bear Form your Stamina is increased by 4%/8%/12%/16%/20% and while in Cat Form your Strength is increased by 4%/8%/12%/16%/20%.
		-- Druid: Survival of the Fittest (Rank 3) - 2,16
		--        Increases all attributes by 1%/2%/3% and reduces the chance you'll be critically hit by melee attacks by 1%/2%/3%.
		["MOD_INT"] = {
			[1] = {
				["tab"] = 2,
				["num"] = 15,
				["rank"] = {
					0.04, 0.08, 0.12, 0.16, 0.2,
				},
			},
			[2] = {
				["tab"] = 2,
				["num"] = 16,
				["rank"] = {
					0.01, 0.02, 0.03,
				},
			},
		},
		-- Druid: Living Spirit (Rank 3) - 3,16
		--        Increases your total Spirit by 5%/10%/15%.
		-- Druid: Survival of the Fittest (Rank 3) - 2,16
		--        Increases all attributes by 1%/2%/3% and reduces the chance you'll be critically hit by melee attacks by 1%/2%/3%.
		["MOD_SPI"] = {
			[1] = {
				["tab"] = 3,
				["num"] = 16,
				["rank"] = {
					0.05, 0.1, 0.15,
				},
			},
			[2] = {
				["tab"] = 2,
				["num"] = 16,
				["rank"] = {
					0.01, 0.02, 0.03,
				},
			},
		},
	},
	["HUNTER"] = {
		-- Hunter: Careful Aim (Rank 3) - 2,16
		--         Increases your ranged attack power by an amount equal to 15%/30%/45% of your total Intellect.
		["ADD_RANGED_AP_MOD_INT"] = {
			[1] = {
				["tab"] = 2,
				["num"] = 16,
				["rank"] = {
					0.15, 0.30, 0.45,
				},
			},
		},
		-- Hunter: Master Marksman (Rank 5) - 2,19
		--         Increases your ranged attack power by 2%/4%/6%/8%/10%.
		["MOD_RANGED_AP"] = {
			[1] = {
				["tab"] = 2,
				["num"] = 19,
				["rank"] = {
					0.02, 0.04, 0.06, 0.08, 0.1,
				},
			},
		},
		-- Hunter: Catlike Reflexes (Rank 3) - 1,19
		--         Increases your chance to dodge by 1%/2%/3% and your pet's chance to dodge by an additional 3%/6%/9%.
		-- Hunter: Aspect of the Monkey - Buff
		--         The hunter takes on the aspects of a monkey, increasing chance to dodge by 8%. Only one Aspect can be active at a time.
		-- Hunter: Improved Aspect of the Monkey (Rank 3) - 1,4
		--         Increases the Dodge bonus of your Aspect of the Monkey by 2%/4%/6%.
		-- Hunter: Deterrence - Buff
		--         Dodge and Parry chance increased by 25%.
		["ADD_DODGE"] = {
			[1] = {
				["tab"] = 1,
				["num"] = 19,
				["rank"] = {
					1, 2, 3,
				},
			},
			[2] = {
				["rank"] = {
					8,
				},
				["buff"] = BS["Aspect of the Monkey"],
			},
			[3] = {
				["tab"] = 1,
				["num"] = 4,
				["rank"] = {
					2, 4, 6,
				},
				["buff"] = BS["Aspect of the Monkey"],
			},
			[4] = {
				["rank"] = {
					25,
				},
				["buff"] = BS["Deterrence"],
			},
		},
		-- Hunter: Survival Instincts (Rank 2) - 1,14
		--         Reduces all damage taken by 2%/4%.
		["MOD_DMG_TAKEN"] = {
			[1] = {
				["MELEE"] = true,
				["RANGED"] = true,
				["HOLY"] = true,
				["FIRE"] = true,
				["NATURE"] = true,
				["FROST"] = true,
				["SHADOW"] = true,
				["ARCANE"] = true,
				["tab"] = 1,
				["num"] = 14,
				["rank"] = {
					-0.02, -0.04,
				},
			},
		},
		-- Hunter: Thick Hide (Rank 3) - 1,5
		--         Increases the armor rating of your pets by 20% and your armor contribution from items by 4%/7%/10%.
		["MOD_ARMOR"] = {
			[1] = {
				["tab"] = 1,
				["num"] = 5,
				["rank"] = {
					1.04, 1.07, 1.1,
				},
			},
		},
		-- Hunter: Survivalist (Rank 5) - 3,9
		--         Increases total health by 2%/4%/6%/8%/10%.
		-- Hunter: Endurance Training (Rank 5) - 1,2
		--         Increases the Health of your pet by 2%/4%/6%/8%/10% and your total health by 1%/2%/3%/4%/5%.
		["MOD_HEALTH"] = {
			[1] = {
				["tab"] = 3,
				["num"] = 9,
				["rank"] = {
					1.02, 1.04, 1.06, 1.08, 1.1,
				},
			},
			[2] = {
				["tab"] = 1,
				["num"] = 2,
				["rank"] = {
					1.01, 1.02, 1.03, 1.04, 1.05,
				},
			},
		},
		-- Hunter: Combat Experience (Rank 2) - 2,14
		--         Increases your total Agility by 1%/2% and your total Intellect by 3%/6%.
		-- Hunter: Lightning Reflexes (Rank 5) - 3,18
		--         Increases your Agility by 3%/6%/9%/12%/15%.
		["MOD_AGI"] = {
			[1] = {
				["tab"] = 2,
				["num"] = 14,
				["rank"] = {
					0.01, 0.02,
				},
			},
			[2] = {
				["tab"] = 3,
				["num"] = 18,
				["rank"] = {
					0.03, 0.06, 0.09, 0.12, 0.15,
				},
			},
		},
		-- Hunter: Combat Experience (Rank 2) - 2,14
		--         Increases your total Agility by 1%/2% and your total Intellect by 3%/6%.
		["MOD_INT"] = {
			[1] = {
				["tab"] = 2,
				["num"] = 14,
				["rank"] = {
					0.03, 0.06,
				},
			},
		},
	},
	["MAGE"] = {
		-- Mage: Arcane Fortitude - 1,9
		--       Increases your armor by an amount equal to 50% of your Intellect.
		["ADD_ARMOR_MOD_INT"] = {
			[1] = {
				["tab"] = 1,
				["num"] = 9,
				["rank"] = {
					0.5,
				},
			},
		},
		-- Mage: Arcane Meditation (Rank 3) - 1,12
		--       Allows 5%/10%/15% of your Mana regeneration to continue while casting.
		["ADD_MANA_REG_MOD_NORMAL_MANA_REG"] = {
			[1] = {
				["tab"] = 1,
				["num"] = 12,
				["rank"] = {
					0.05, 0.1, 0.15,
				},
			},
		},
		-- Mage: Mind Mastery (Rank 5) - 1,22
		--       Increases spell damage by up to 5%/10%/15%/20%/25% of your total Intellect.
		["ADD_SPELL_DMG_MOD_INT"] = {
			[1] = {
				["tab"] = 1,
				["num"] = 22,
				["rank"] = {
					0.05, 0.1, 0.15, 0.2, 0.25,
				},
			},
		},
		-- Mage: Arctic Winds (Rank 5) - 3,20
		--       Reduces the chance melee and ranged attacks will hit you by 1%/2%/3%/4%/5%.
		["ADD_HIT_TAKEN"] = {
			[1] = {
				["MELEE"] = true,
				["RANGED"] = true,
				["tab"] = 3,
				["num"] = 20,
				["rank"] = {
					-0.01, -0.02, -0.03, -0.04, -0.05,
				},
			},
		},
		-- Mage: Prismatic Cloak (Rank 2) - 1,16
		--       Reduces all damage taken by 2%/4%.
		-- Mage: Playing with Fire (Rank 3) - 2,13
		--       Increases all spell damage caused by 1%/2%/3% and all spell damage taken by 1%/2%/3%.
		-- Mage: Frozen Core (Rank 3) - 3,14
		--       Reduces the damage taken by Frost and Fire effects by 2%/4%/6%.
		["MOD_DMG_TAKEN"] = {
			[1] = {
				["MELEE"] = true,
				["RANGED"] = true,
				["HOLY"] = true,
				["FIRE"] = true,
				["NATURE"] = true,
				["FROST"] = true,
				["SHADOW"] = true,
				["ARCANE"] = true,
				["tab"] = 1,
				["num"] = 16,
				["rank"] = {
					-0.02, -0.04,
				},
			},
			[2] = {
				["HOLY"] = true,
				["FIRE"] = true,
				["NATURE"] = true,
				["FROST"] = true,
				["SHADOW"] = true,
				["ARCANE"] = true,
				["tab"] = 2,
				["num"] = 13,
				["rank"] = {
					-0.01, -0.02, -0.03,
				},
			},
			[3] = {
				["FIRE"] = true,
				["FROST"] = true,
				["tab"] = 3,
				["num"] = 14,
				["rank"] = {
					-0.02, -0.04, -0.06,
				},
			},
		},
		-- Mage: Arcane Mind (Rank 5) - 1,15
		--       Increases your total Intellect by 3%/6%/9%/12%/15%.
		["MOD_INT"] = {
			[1] = {
				["tab"] = 1,
				["num"] = 15,
				["rank"] = {
					0.03, 0.06, 0.09, 0.12, 0.15,
				},
			},
		},
	},
	["PALADIN"] = {
		-- Paladin: Holy Guidance (Rank 5) - 1,19
		--          Increases your spell damage and healing by 7%/14%/21%/28%/35% of your total Intellect.
		["ADD_SPELL_DMG_MOD_INT"] = {
			[1] = {
				["tab"] = 1,
				["num"] = 19,
				["rank"] = {
					0.07, 0.14, 0.21, 0.28, 0.35,
				},
			},
		},
		-- Paladin: Holy Guidance (Rank 5) - 1,19
		--          Increases your spell damage and healing by 7%/14%/21%/28%/35% of your total Intellect.
		["ADD_HEALING_MOD_INT"] = {
			[1] = {
				["tab"] = 1,
				["num"] = 19,
				["rank"] = {
					0.07, 0.14, 0.21, 0.28, 0.35,
				},
			},
		},
		-- Paladin: Divine Purpose (Rank 3) - 3,20
		--          Melee and ranged critical strikes against you cause 4%/7%/10% less damage.
		["MOD_CRIT_DAMAGE_TAKEN"] = {
			[1] = {
				["MELEE"] = true,
				["RANGED"] = true,
				["tab"] = 3,
				["num"] = 20,
				["rank"] = {
					-0.04, -0.07, -0.1,
				},
			},
		},
		-- Paladin: Blessed Life (Rank 3) - 1,18
		--          All attacks against you have a 4%/7%/10% chance to cause half damage.
		-- Paladin: Ardent Defender (Rank 5) - 2,19
		--          When you have less than 20% health, all damage taken is reduced by 10%/20%/30%/40%/50%.
		-- Paladin: Spell Warding (Rank 2) - 2,13
		--          All spell damage taken is reduced by 2%/4%.
		-- Paladin: Improved Righteous Fury (Rank 3) - 2,7
		--          While Righteous Fury is active, all damage taken is reduced by 2%/4%/6% and increases the amount of threat generated by your Righteous Fury spell by 16%/33%/50%.
		["MOD_DMG_TAKEN"] = {
			[1] = {
				["MELEE"] = true,
				["RANGED"] = true,
				["HOLY"] = true,
				["FIRE"] = true,
				["NATURE"] = true,
				["FROST"] = true,
				["SHADOW"] = true,
				["ARCANE"] = true,
				["tab"] = 1,
				["num"] = 18,
				["rank"] = {
					-0.02, -0.035, -0.05,
				},
			},
			[2] = {
				["MELEE"] = true,
				["RANGED"] = true,
				["HOLY"] = true,
				["FIRE"] = true,
				["NATURE"] = true,
				["FROST"] = true,
				["SHADOW"] = true,
				["ARCANE"] = true,
				["tab"] = 2,
				["num"] = 19,
				["rank"] = {
					-0.1, -0.2, -0.3, -0.4, -0.5,
				},
				["condition"] = "(UnitHealth('player') / UnitHealthMax('player')) < 0.2",
			},
			[3] = {
				["HOLY"] = true,
				["FIRE"] = true,
				["NATURE"] = true,
				["FROST"] = true,
				["SHADOW"] = true,
				["ARCANE"] = true,
				["tab"] = 2,
				["num"] = 13,
				["rank"] = {
					-0.02, -0.04,
				},
			},
			[4] = {
				["MELEE"] = true,
				["RANGED"] = true,
				["HOLY"] = true,
				["FIRE"] = true,
				["NATURE"] = true,
				["FROST"] = true,
				["SHADOW"] = true,
				["ARCANE"] = true,
				["tab"] = 2,
				["num"] = 7,
				["rank"] = {
					-0.02, -0.04, -0.06,
				},
				["buff"] = BS["Righteous Fury"],
			},
		},
		-- Paladin: Toughness (Rank 5) - 2,5
		--          Increases your armor value from items by 2%/4%/6%/8%/10%.
		["MOD_ARMOR"] = {
			[1] = {
				["tab"] = 2,
				["num"] = 5,
				["rank"] = {
					1.02, 1.04, 1.06, 1.08, 1.1,
				},
			},
		},
		-- Paladin: Divine Strength (Rank 5) - 1,1
		--          Increases your total Strength by 2%/4%/6%/8%/10%.
		["MOD_STR"] = {
			[1] = {
				["tab"] = 1,
				["num"] = 1,
				["rank"] = {
					0.02, 0.04, 0.06, 0.08, 0.1,
				},
			},
		},
		-- Paladin: Sacred Duty (Rank 2) - 2,16
		--          Increases your total Stamina by 3%/6%, reduces the cooldown of your Divine Shield spell by 60 sec and reduces the attack speed penalty by 100%.
		["MOD_STA"] = {
			[1] = {
				["tab"] = 2,
				["num"] = 16,
				["rank"] = {
					0.03, 0.06
				},
			},
		},
		-- Paladin: Divine Intellect (Rank 5) - 1,2
		--          Increases your total Intellect by 2%/4%/6%/8%/10%.
		["MOD_INT"] = {
			[1] = {
				["tab"] = 1,
				["num"] = 2,
				["rank"] = {
					0.02, 0.04, 0.06, 0.08, 0.1,
				},
			},
		},
		-- Paladin: Shield Specialization (Rank 3) - 2,8
		--          Increases the amount of damage absorbed by your shield by 10%/20%/30%.
		["MOD_BLOCK_VALUE"] = {
			[1] = {
				["tab"] = 2,
				["num"] = 8,
				["rank"] = {
					0.1, 0.2, 0.3,
				},
			},
		},
	},
	["PRIEST"] = {
		-- Priest: Meditation (Rank 3) - 1,9
		--         Allows 5%/10%/15% of your Mana regeneration to continue while casting.
		["ADD_MANA_REG_MOD_NORMAL_MANA_REG"] = {
			[1] = {
				["tab"] = 1,
				["num"] = 9,
				["rank"] = {
					0.05, 0.1, 0.15,
				},
			},
		},
		-- Priest: Spiritual Guidance (Rank 5) - 2,14
		--         Increases spell damage and healing by up to 5%/10%/15%/20%/25% of your total Spirit.
		-- Priest: Improved Divine Spirit (Rank 2) - 1,15 - Buff
		--         Your Divine Spirit and Prayer of Spirit spells also increase the target's spell damage and healing by an amount equal to 5%/10% of their total Spirit.
		["ADD_SPELL_DMG_MOD_SPI"] = {
			[1] = {
				["tab"] = 2,
				["num"] = 14,
				["rank"] = {
					0.05, 0.1, 0.15, 0.2, 0.25,
				},
			},
			[2] = {
				["tab"] = 1,
				["num"] = 15,
				["rank"] = {
					0.05, 0.1,
				},
				["buff"] = BS["Divine Spirit"],
			},
			[3] = {
				["tab"] = 1,
				["num"] = 15,
				["rank"] = {
					0.05, 0.1,
				},
				["buff"] = BS["Prayer of Spirit"],
			},
		},
		-- Priest: Spiritual Guidance (Rank 5) - 2,14
		--         Increases spell damage and healing by up to 5%/10%/15%/20%/25% of your total Spirit.
		-- Priest: Improved Divine Spirit (Rank 2) - 1,15 - Buff
		--         Your Divine Spirit and Prayer of Spirit spells also increase the target's spell damage and healing by an amount equal to 5%/10% of their total Spirit.
		["ADD_HEALING_MOD_SPI"] = {
			[1] = {
				["tab"] = 2,
				["num"] = 14,
				["rank"] = {
					0.05, 0.1, 0.15, 0.2, 0.25,
				},
			},
			[2] = {
				["tab"] = 1,
				["num"] = 15,
				["rank"] = {
					0.05, 0.1,
				},
				["buff"] = BS["Divine Spirit"],
			},
			[3] = {
				["tab"] = 1,
				["num"] = 15,
				["rank"] = {
					0.05, 0.1,
				},
				["buff"] = BS["Prayer of Spirit"],
			},
		},
		-- Priest: Elune's Grace (Rank 6) - Buff, NE priest only
		--         Ranged damage taken reduced by 167 and chance to dodge increased by 10%.
		["ADD_DODGE"] = {
			[1] = {
				["rank"] = {
					10, 10, 10, 10, 10, 10,
				},
				["buff"] = BS["Elune's Grace"],
			},
		},
		-- Priest: Shadow Resilience (Rank 2) - 3,16
		--         Reduces the chance you'll be critically hit by all spells by 2%/4%.
		["MOD_CRIT_DAMAGE_TAKEN"] = {
			[1] = {
				["HOLY"] = true,
				["FIRE"] = true,
				["NATURE"] = true,
				["FROST"] = true,
				["SHADOW"] = true,
				["ARCANE"] = true,
				["tab"] = 3,
				["num"] = 16,
				["rank"] = {
					-0.02, -0.04,
				},
			},
		},
		-- Priest: Spell Warding (Rank 5) - 2,4
		--         Reduces all spell damage taken by 2%/4%/6%/8%/10%.
		-- Priest: Pain Suppression - Buff
		--         Reduces all damage taken by 60% for 8 sec.
		["MOD_DMG_TAKEN"] = {
			[1] = {
				["HOLY"] = true,
				["FIRE"] = true,
				["NATURE"] = true,
				["FROST"] = true,
				["SHADOW"] = true,
				["ARCANE"] = true,
				["tab"] = 2,
				["num"] = 4,
				["rank"] = {
					-0.02, -0.04, -0.06, -0.08, -0.1,
				},
			},
			[2] = {
				["MELEE"] = true,
				["RANGED"] = true,
				["HOLY"] = true,
				["FIRE"] = true,
				["NATURE"] = true,
				["FROST"] = true,
				["SHADOW"] = true,
				["ARCANE"] = true,
				["rank"] = {
					-0.6,
				},
				["buff"] = BS["Pain Suppression"],
			},
		},
		-- Priest: Mental Strength (Rank 5) - 1,13
		--         IIncreases your maximum Mana by 2%/4%/6%/8%/10%.
		["MOD_MANA"] = {
			[1] = {
				["tab"] = 1,
				["num"] = 13,
				["rank"] = {
					1.02, 1.04, 1.06, 1.08, 1.1,
				},
			},
		},
		-- Priest: Enlightenment (Rank 5) - 1,20
		--         Increases your total Stamina, Intellect and Spirit by 1%/2%/3%/4%/5%.
		["MOD_STA"] = {
			[1] = {
				["tab"] = 1,
				["num"] = 20,
				["rank"] = {
					0.01, 0.02, 0.03, 0.04, 0.05,
				},
			},
		},
		-- Priest: Enlightenment (Rank 5) - 1,20
		--         Increases your total Stamina, Intellect and Spirit by 1%/2%/3%/4%/5%.
		["MOD_INT"] = {
			[1] = {
				["tab"] = 1,
				["num"] = 20,
				["rank"] = {
					0.01, 0.02, 0.03, 0.04, 0.05,
				},
			},
		},
		-- Priest: Enlightenment (Rank 5) - 1,20
		--         Increases your total Stamina, Intellect and Spirit by 1%/2%/3%/4%/5%.
		-- Priest: Spirit of Redemption - 2,13
		--         Increases total Spirit by 5% and upon death, the priest becomes the Spirit of Redemption for 15 sec.
		["MOD_SPI"] = {
			[1] = {
				["tab"] = 1,
				["num"] = 20,
				["rank"] = {
					0.01, 0.02, 0.03, 0.04, 0.05,
				},
			},
			[2] = {
				["tab"] = 2,
				["num"] = 13,
				["rank"] = {
					0.05,
				},
			},
		},
	},
	["ROGUE"] = {
		-- Rogue: Deadliness (Rank 5) - 3,17
		--        Increases your attack power by 2%/4%/6%/8%/10%.
		["MOD_AP"] = {
			[1] = {
				["tab"] = 3,
				["num"] = 17,
				["rank"] = {
					0.02, 0.04, 0.06, 0.08, 0.1,
				},
			},
		},
		-- Rogue: Lightning Reflexes (Rank 5) - 2,3
		--        Increases your Dodge chance by 1%/2%/3%/4%/5%.
		-- Rogue: Evasion (Rank 1/2) - Buff
		--        Dodge chance increased by 50%/50% and chance ranged attacks hit you reduced by 0%/25%.
		-- Rogue: Ghostly Strike - Buff
		--        Dodge chance increased by 15%.
		["ADD_DODGE"] = {
			[1] = {
				["tab"] = 2,
				["num"] = 3,
				["rank"] = {
					1, 2, 3, 4, 5,
				},
			},
			[2] = {
				["rank"] = {
					50, 50,
				},
				["buff"] = BS["Evasion"],
			},
			[3] = {
				["rank"] = {
					15,
				},
				["buff"] = BS["Ghostly Strike"],
			},
		},
		-- Rogue: Sleight of Hand (Rank 2) - 3,3
		--        Reduces the chance you are critically hit by melee and ranged attacks by 2% and increases the threat reduction of your Feint ability by 20%.
		["ADD_CRIT_TAKEN"] = {
			[1] = {
				["MELEE"] = true,
				["RANGED"] = true,
				["tab"] = 3,
				["num"] = 3,
				["rank"] = {
					-0.02, -0.04,
				},
			},
		},
		-- Rogue: Heightened Senses (Rank 2) - 3,12
		--        Increases your Stealth detection and reduces the chance you are hit by spells and ranged attacks by 2%/4%.
		-- Rogue: Cloak of Shadows - buff
		--        Instantly removes all existing harmful spell effects and increases your chance to resist all spells by 90% for 5 sec. Does not remove effects that prevent you from using Cloak of Shadows.
		-- Rogue: Evasion (Rank 1/2) - Buff
		--        Dodge chance increased by 50%/50% and chance ranged attacks hit you reduced by 0%/25%.
		["ADD_HIT_TAKEN"] = {
			[1] = {
				["RANGED"] = true,
				["HOLY"] = true,
				["FIRE"] = true,
				["NATURE"] = true,
				["FROST"] = true,
				["SHADOW"] = true,
				["ARCANE"] = true,
				["tab"] = 3,
				["num"] = 12,
				["rank"] = {
					-0.02, -0.04,
				},
			},
			[2] = {
				["HOLY"] = true,
				["FIRE"] = true,
				["NATURE"] = true,
				["FROST"] = true,
				["SHADOW"] = true,
				["ARCANE"] = true,
				["rank"] = {
					-0.9,
				},
				["buff"] = BS["Cloak of Shadows"],
			},
			[3] = {
				["RANGED"] = true,
				["rank"] = {
					0, -0.25,
				},
				["buff"] = BS["Evasion"],
			},
		},
		-- Rogue: Deadened Nerves (Rank 5) - 1,19
		--        Decreases all physical damage taken by 1%/2%/3%/4%/5%
		["MOD_DMG_TAKEN"] = {
			[1] = {
				["MELEE"] = true,
				["RANGED"] = true,
				["tab"] = 1,
				["num"] = 19,
				["rank"] = {
					-0.01, -0.02, -0.03, -0.04, -0.05,
				},
			},
		},
		-- Rogue: Vitality (Rank 2) - 2,20
		--        Increases your total Stamina by 2%/4% and your total Agility by 1%/2%.
		-- Rogue: Sinister Calling (Rank 5) - 3,21
		--        Increases your total Agility by 3%/6%/9%/12%/15%.
		["MOD_AGI"] = {
			[1] = {
				["tab"] = 2,
				["num"] = 20,
				["rank"] = {
					0.01, 0.02,
				},
			},
			[2] = {
				["tab"] = 3,
				["num"] = 21,
				["rank"] = {
					0.03, 0.06, 0.09, 0.12, 0.15,
				},
			},
		},
		-- Rogue: Vitality (Rank 2) - 2,20
		--        Increases your total Stamina by 2%/4% and your total Agility by 1%/2%.
		["MOD_STA"] = {
			[1] = {
				["tab"] = 2,
				["num"] = 20,
				["rank"] = {
					0.02, 0.04,
				},
			},
		},
	},
	["SHAMAN"] = {
		-- Shaman: Unrelenting Storm (Rank 5) - 1,14
		--         Regenerate mana equal to 2%/4%/6%/8%/10% of your Intellect every 5 sec, even while casting.
		["ADD_MANA_REG_MOD_INT"] = {
			[1] = {
				["tab"] = 1,
				["num"] = 14,
				["rank"] = {
					0.02, 0.04, 0.06, 0.08, 0.1,
				},
			},
		},
		-- Shaman: Nature's Blessing (Rank 3) - 3,18
		--         Increases your spell damage and healing by an amount equal to 10%/20%/30% of your Intellect.
		["ADD_SPELL_DMG_MOD_INT"] = {
			[1] = {
				["tab"] = 3,
				["num"] = 18,
				["rank"] = {
					0.1, 0.2, 0.3,
				},
			},
		},
		-- Shaman: Nature's Blessing (Rank 3) - 3,18
		--         Increases your spell damage and healing by an amount equal to 10%/20%/30% of your Intellect.
		["ADD_HEALING_MOD_INT"] = {
			[1] = {
				["tab"] = 3,
				["num"] = 18,
				["rank"] = {
					0.1, 0.2, 0.3,
				},
			},
		},
		-- Shaman: Anticipation (Rank 5) - 2,9
		--         Increases your chance to dodge by an additional 1%/2%/3%/4%/5%.
		["ADD_DODGE"] = {
			[1] = {
				["tab"] = 2,
				["num"] = 9,
				["rank"] = {
					1, 2, 3, 4, 5,
				},
			},
		},
		-- Shaman: Elemental Shields (Rank 3) - 1,18
		--         Reduces the chance you will be critically hit by melee and ranged attacks by 2%/4%/6%.
		["ADD_CRIT_TAKEN"] = {
			[1] = {
				["MELEE"] = true,
				["RANGED"] = true,
				["tab"] = 1,
				["num"] = 18,
				["rank"] = {
					-0.02, -0.04, -0.06,
				},
			},
		},
		-- Shaman: Elemental Warding (Rank 3) - 1,4
		--         Reduces damage taken from Fire, Frost and Nature effects by 4%/7%/10%.
		["MOD_DMG_TAKEN"] = {
			[1] = {
				["FIRE"] = true,
				["NATURE"] = true,
				["FROST"] = true,
				["tab"] = 1,
				["num"] = 14,
				["rank"] = {
					-0.04, -0.07, -0.1,
				},
			},
		},
		-- Shaman: Toughness (Rank 5) - 2,11
		--         Increases your armor value from items by 2%/4%/6%/8%/10%.
		["MOD_ARMOR"] = {
			[1] = {
				["tab"] = 2,
				["num"] = 11,
				["rank"] = {
					1.02, 1.04, 1.06, 1.08, 1.1,
				},
			},
		},
		-- Shaman: Ancestral Knowledge (Rank 5) - 2,1
		--         Increases your maximum Mana by 1%/2%/3%/4%/5%.
		["MOD_MANA"] = {
			[1] = {
				["tab"] = 2,
				["num"] = 1,
				["rank"] = {
					1.01, 1.02, 1.03, 1.04, 1.05,
				},
			},
		},
		-- Shaman: Shield Specialization 5/5 - 2,2
		--         Increases your chance to block attacks with a shield by 5% and increases the amount blocked by 5%/10%/15%/20%/25%.
		["MOD_BLOCK_VALUE"] = {
			[1] = {
				["tab"] = 2,
				["num"] = 2,
				["rank"] = {
					0.05, 0.1, 0.15, 0.2, 0.25,
				},
			},
		},
	},
	["WARLOCK"] = {
		-- Warlock: Demonic Knowledge (Rank 3) - 2,20 - UnitExists("pet")
		--          Increases your spell damage by an amount equal to 5%/10%/15% of the total of your active demon's Stamina plus Intellect.
		["ADD_SPELL_DMG_MOD_INT"] = {
			[1] = {
				["tab"] = 2,
				["num"] = 20,
				["rank"] = {
					0.015, 0.03, 0.045,
				},
				["condition"] = "UnitExists('pet')",
			},
		},
		-- Warlock: Demonic Knowledge (Rank 3) - 2,20 - UnitExists("pet")
		--          Increases your spell damage by an amount equal to 5%/10%/15% of the total of your active demon's Stamina plus Intellect.
		["ADD_SPELL_DMG_MOD_STA"] = {
			[1] = {
				["tab"] = 2,
				["num"] = 20,
				["rank"] = {
					0.015, 0.03, 0.045,
				},
				["condition"] = "UnitExists('pet')",
			},
		},
		-- Warlock: Demonic Resilience (Rank 3) - 2,18
		--          Reduces the chance you'll be critically hit by melee and spells by 1%/2%/3% and reduces all damage your summoned demon takes by 15%.
		["ADD_CRIT_TAKEN"] = {
			[1] = {
				["MELEE"] = true,
				["HOLY"] = true,
				["FIRE"] = true,
				["NATURE"] = true,
				["FROST"] = true,
				["SHADOW"] = true,
				["ARCANE"] = true,
				["tab"] = 2,
				["num"] = 18,
				["rank"] = {
					-0.01, -0.02, -0.03,
				},
			},
		},
		-- Warlock: Master Demonologist (Rank 5) - 2,17
		--          Voidwalker - Reduces physical damage taken by 2%/4%/6%/8%/10%.
		-- Warlock: Soul Link (Rank 1) - 2,19
		--          When active, 20% of all damage taken by the caster is taken by your Imp, Voidwalker, Succubus, Felhunter or Felguard demon instead. In addition, both the demon and master will inflict 5% more damage. Lasts as long as the demon is active.
		["MOD_DMG_TAKEN"] = {
			[1] = {
				["MELEE"] = true,
				["RANGED"] = true,
				["tab"] = 2,
				["num"] = 17,
				["rank"] = {
					-0.02, -0.04, -0.06, -0.08, -0.1,
				},
				["condition"] = "IsUsableSpell('"..BS["Torment"].."')" --"UnitCreatureFamily('pet') == '"..L["Voidwalker"].."'",
			},
			[2] = {
				["MELEE"] = true,
				["RANGED"] = true,
				["HOLY"] = true,
				["FIRE"] = true,
				["NATURE"] = true,
				["FROST"] = true,
				["SHADOW"] = true,
				["ARCANE"] = true,
				["rank"] = {
					-0.2,
				},
				["buff"] = BS["Soul Link"],
			},
		},
		-- Warlock: Fel Stamina (Rank 3) - 2,9
		--          Increases the Stamina of your Imp, Voidwalker, Succubus, Felhunter and Felguard by 5%/10%/15% and increases your maximum health by 1%/2%/3%.
		["MOD_HEALTH"] = {
			[1] = {
				["tab"] = 2,
				["num"] = 9,
				["rank"] = {
					1.01, 1.02, 1.03,
				},
			},
		},
		-- Warlock: Fel Intellect (Rank 3) - 2,6
		--          Increases the Intellect of your Imp, Voidwalker, Succubus, Felhunter and Felguard by 5%/10%/15% and increases your maximum mana by 1%/2%/3%.
		["MOD_MANA"] = {
			[1] = {
				["tab"] = 2,
				["num"] = 6,
				["rank"] = {
					1.01, 1.02, 1.03,
				},
			},
		},
		-- Warlock: Demonic Embrace (Rank 5) - 2,3
		--          Increases your total Stamina by 3%/6%/9%/12%/15% but reduces your total Spirit by 1%/2%/3%/4%/5%.
		["MOD_STA"] = {
			[1] = {
				["tab"] = 2,
				["num"] = 3,
				["rank"] = {
					0.03, 0.06, 0.09, 0.12, 0.15,
				},
			},
		},
		-- Warlock: Demonic Embrace (Rank 5) - 2,3
		--          Increases your total Stamina by 3%/6%/9%/12%/15% but reduces your total Spirit by 1%/2%/3%/4%/5%.
		["MOD_SPI"] = {
			[1] = {
				["tab"] = 2,
				["num"] = 3,
				["rank"] = {
					-0.01, -0.02, -0.03, -0.04, -0.05,
				},
			},
		},
	},
	["WARRIOR"] = {
		-- Warrior: Improved Berserker Stance (Rank 5) - 2,20 - Stance
		--          Increases attack power by 2%/4%/6%/8%/10% while in Berserker Stance.
		["MOD_AP"] = {
			[1] = {
				["tab"] = 2,
				["num"] = 20,
				["rank"] = {
					0.02, 0.04, 0.06, 0.08, 0.1,
				},
				["stance"] = "Interface\\Icons\\Ability_Racial_Avatar",
			},
		},
		-- Warrior: Shield Wall - Buff
		--          Reduces the Physical and magical damage taken by the caster by 75% for 10 sec.
		-- Warrior: Defensive Stance - stance
		--          A defensive combat stance. Decreases damage taken by 10% and damage caused by 10%. Increases threat generated.
		-- Warrior: Berserker Stance - stance
		--          An aggressive stance. Critical hit chance is increased by 3% and all damage taken is increased by 10%.
		-- Warrior: Death Wish - Buff
		--          When activated, increases your physical damage by 20% and makes you immune to Fear effects, but increases all damage taken by 5%. Lasts 30 sec.
		-- Warrior: Recklessness - Buff
		--          The warrior will cause critical hits with most attacks and will be immune to Fear effects for the next 15 sec, but all damage taken is increased by 20%.
		-- Warrior: Improved Defensive Stance (Rank 3) - 3,18
        --          Reduces all spell damage taken while in Defensive Stance by 2%/4%/6%.
		["MOD_DMG_TAKEN"] = {
			[1] = {
				["MELEE"] = true,
				["RANGED"] = true,
				["HOLY"] = true,
				["FIRE"] = true,
				["NATURE"] = true,
				["FROST"] = true,
				["SHADOW"] = true,
				["ARCANE"] = true,
				["rank"] = {
					-0.75,
				},
				["buff"] = BS["Shield Wall"],
			},
			[2] = {
				["MELEE"] = true,
				["RANGED"] = true,
				["HOLY"] = true,
				["FIRE"] = true,
				["NATURE"] = true,
				["FROST"] = true,
				["SHADOW"] = true,
				["ARCANE"] = true,
				["rank"] = {
					-0.1,
				},
				["stance"] = "Interface\\Icons\\Ability_Warrior_DefensiveStance",
			},
			[3] = {
				["MELEE"] = true,
				["RANGED"] = true,
				["HOLY"] = true,
				["FIRE"] = true,
				["NATURE"] = true,
				["FROST"] = true,
				["SHADOW"] = true,
				["ARCANE"] = true,
				["rank"] = {
					0.1,
				},
				["stance"] = "Interface\\Icons\\Ability_Racial_Avatar",
			},
			[4] = {
				["MELEE"] = true,
				["RANGED"] = true,
				["HOLY"] = true,
				["FIRE"] = true,
				["NATURE"] = true,
				["FROST"] = true,
				["SHADOW"] = true,
				["ARCANE"] = true,
				["rank"] = {
					0.05,
				},
				["buff"] = BS["Death Wish"],
			},
			[5] = {
				["MELEE"] = true,
				["RANGED"] = true,
				["HOLY"] = true,
				["FIRE"] = true,
				["NATURE"] = true,
				["FROST"] = true,
				["SHADOW"] = true,
				["ARCANE"] = true,
				["rank"] = {
					0.2,
				},
				["buff"] = BS["Recklessness"],
			},
			[6] = {
				["HOLY"] = true,
				["FIRE"] = true,
				["NATURE"] = true,
				["FROST"] = true,
				["SHADOW"] = true,
				["ARCANE"] = true,
				["tab"] = 3,
				["num"] = 18,
				["rank"] = {
					-0.02, -0.04, -0.06,
				},
			},
		},
		-- Warrior: Toughness (Rank 5) - 3,5
		--          Increases your armor value from items by 2%/4%/6%/8%/10%.
		["MOD_ARMOR"] = {
			[1] = {
				["tab"] = 3,
				["num"] = 5,
				["rank"] = {
					1.02, 1.04, 1.06, 1.08, 1.1,
				},
			},
		},
		-- Warrior: Vitality (Rank 5) - 3,21
		--          Increases your total Stamina by 1%/2%/3%/4%/5% and your total Strength by 2%/4%/6%/8%/10%.
		["MOD_STA"] = {
			[1] = {
				["tab"] = 3,
				["num"] = 21,
				["rank"] = {
					0.01, 0.02, 0.03, 0.04, 0.05,
				},
			},
		},
		-- Warrior: Vitality (Rank 5) - 3,21
		--          Increases your total Stamina by 1%/2%/3%/4%/5% and your total Strength by 2%/4%/6%/8%/10%.
		["MOD_STR"] = {
			[1] = {
				["tab"] = 3,
				["num"] = 21,
				["rank"] = {
					0.02, 0.04, 0.06, 0.08, 0.1,
				},
			},
		},
		-- Warrior: Shield Mastery (Rank 3) - 3,16
		--          Increases the amount of damage absorbed by your shield by 10%/20%/30%.
		["MOD_BLOCK_VALUE"] = {
			[1] = {
				["tab"] = 3,
				["num"] = 16,
				["rank"] = {
					0.1, 0.2, 0.3,
				},
			},
		},
	},
	["ALL"] = {
		-- Priest: Power Infusion - Buff
		--         Infuses the target with power, increasing their spell damage and healing by 20%. Lasts 15 sec.
		["MOD_SPELL_DMG"] = {
			[1] = {
				["rank"] = {
					0.2,
				},
				["buff"] = BS["Power Infusion"],
			},
		},
		-- Priest: Power Infusion - Buff
		--         Infuses the target with power, increasing their spell damage and healing by 20%. Lasts 15 sec.
		["MOD_HEALING"] = {
			[1] = {
				["rank"] = {
					0.2,
				},
				["buff"] = BS["Power Infusion"],
			},
		},
		-- Night Elf : Quickness - Racial
		--             Dodge chance increased by 1%.
		["ADD_DODGE"] = {
			[1] = {
				["rank"] = {
					1,
				},
				["race"] = "NightElf",
			},
		},
		-- Paladin: Lay on Hands (Rank 1/2) - Buff
		--          Armor increased by 15%/30%.
		-- Priest: Inspiration (Rank 1/2/3) - Buff
		--         Increases armor by 8%/16%/25%.
		-- Shaman: Ancestral Fortitude (Rank 1/2/3) - Buff
		--         Increases your armor value by 8%/16%/25%.
		["MOD_ARMOR"] = {
			[1] = {
				["rank"] = {
					1.15, 1.30,
				},
				["buff"] = BS["Lay on Hands"],
			},
			[2] = {
				["rank"] = {
					1.08, 1.16, 1.25,
				},
				["buff"] = BS["Inspiration"],
			},
			[3] = {
				["rank"] = {
					1.08, 1.16, 1.25,
				},
				["buff"] = BS["Ancestral Fortitude"],
			},
		},
		-- Tauren: Endurance - Racial
		--         Total Health increased by 5%.
		["MOD_HEALTH"] = {
			[1] = {
				["rank"] = {
					1.05,
				},
				["race"] = "Tauren",
			},
		},
		-- Blessing of Kings - Buff
		-- Increases stats by 10%.
		-- Greater Blessing of Kings - Buff
		-- Increases stats by 10%.
		["MOD_STR"] = {
			[1] = {
				["rank"] = {
					0.1,
				},
				["buff"] = BS["Blessing of Kings"],
			},
			[2] = {
				["rank"] = {
					0.1,
				},
				["buff"] = BS["Greater Blessing of Kings"],
			},
		},
		-- Blessing of Kings - Buff
		-- Increases stats by 10%.
		-- Greater Blessing of Kings - Buff
		-- Increases stats by 10%.
		["MOD_AGI"] = {
			[1] = {
				["rank"] = {
					0.1,
				},
				["buff"] = BS["Blessing of Kings"],
			},
			[2] = {
				["rank"] = {
					0.1,
				},
				["buff"] = BS["Greater Blessing of Kings"],
			},
		},
		-- Blessing of Kings - Buff
		-- Increases stats by 10%.
		-- Greater Blessing of Kings - Buff
		-- Increases stats by 10%.
		["MOD_STA"] = {
			[1] = {
				["rank"] = {
					0.1,
				},
				["buff"] = BS["Blessing of Kings"],
			},
			[2] = {
				["rank"] = {
					0.1,
				},
				["buff"] = BS["Greater Blessing of Kings"],
			},
		},
		-- Blessing of Kings - Buff
		-- Increases stats by 10%.
		-- Greater Blessing of Kings - Buff
		-- Increases stats by 10%.
		["MOD_INT"] = {
			[1] = {
				["rank"] = {
					0.1,
				},
				["buff"] = BS["Blessing of Kings"],
			},
			[2] = {
				["rank"] = {
					0.1,
				},
				["buff"] = BS["Greater Blessing of Kings"],
			},
		},
		-- Blessing of Kings - Buff
		-- Increases stats by 10%.
		-- Greater Blessing of Kings - Buff
		-- Increases stats by 10%.
		["MOD_SPI"] = {
			[1] = {
				["rank"] = {
					0.1,
				},
				["buff"] = BS["Blessing of Kings"],
			},
			[2] = {
				["rank"] = {
					0.1,
				},
				["buff"] = BS["Greater Blessing of Kings"],
			},
		},
	},
}

function StatLogic:GetStatMod(stat, school)
	local statModInfo = StatModInfo[stat]
	local mod = statModInfo.initialValue
	-- if school is required for this statMod but not given
	if statModInfo.school and not school then return mod end
	-- Class specific mods
	if type(StatModTable[class][stat]) == "table" then
		for _, case in ipairs(StatModTable[class][stat]) do
			local ok = true
			if school and not case[school] then ok = nil end
			if ok and case.condition and not loadstring("return "..case.condition)() then ok = nil end
			if ok and case.buff and not GetPlayerBuffName(case.buff) then ok = nil end
			if ok and case.stance and case.stance ~= GetStanceIcon() then ok = nil end
			if ok then
				local r, _
				-- if talant field
				if case.tab and case.num then
					_, _, _, _, r = GetTalentInfo(case.tab, case.num)
				-- no talant but buff is given
				elseif case.buff then
					r = GetPlayerBuffRank(case.buff)
				-- no talant but all other given conditions are statisfied
				elseif case.condition or case.stance then
					r = 1
				end
				if r and r ~= 0 then
					if statModInfo.initialValue == 0 then
						mod = mod + case.rank[r]
					else
						mod = mod * case.rank[r]
					end
				end
			end
		end
	end
	-- Non class specific mods
	if type(StatModTable["ALL"][stat]) == "table" then
		for _, case in ipairs(StatModTable["ALL"][stat]) do
			local ok = true
			if school and not case[school] then ok = nil end
			if ok and case.condition and not loadstring("return "..case.condition)() then ok = nil end
			if ok and case.buff and not GetPlayerBuffName(case.buff) then ok = nil end
			if ok and case.stance and case.stance ~= GetStanceIcon() then ok = nil end
			if ok and case.race and case.race ~= race then ok = nil end
			if ok then
				local r
				-- there are no talants in non class specific mods
				-- check buff
				if case.buff then
					r = GetPlayerBuffRank(case.buff)
				-- no talant but all other given conditions are statisfied
				elseif case.condition or case.stance or case.race then
					r = 1
				end
				if r and r ~= 0 then
					if statModInfo.initialValue == 0 then
						mod = mod + case.rank[r]
					else
						mod = mod * case.rank[r]
					end
				end
			end
		end
	end
	
	return mod + statModInfo.finalAdjust
end


--=================--
-- Stat Conversion --
--=================--
local ClassNameToID = {
	["WARRIOR"] = 1,
	["PALADIN"] = 2,
	["HUNTER"] = 3,
	["ROGUE"] = 4,
	["PRIEST"] = 5,
	["SHAMAN"] = 6,
	["MAGE"] = 7,
	["WARLOCK"] = 8,
	["DRUID"] = 9,
}

function StatLogic:GetReductionFromArmor(armor, attackerLevel)
	local levelModifier = attackerLevel
	if ( levelModifier > 59 ) then
		levelModifier = levelModifier + (4.5 * (levelModifier - 59))
	end
	local temp = armor / (85 * levelModifier + 400)
	local armorReduction = temp / (1 + temp)
	-- caps at 75%
	if armorReduction > 0.75 then
		armorReduction = 0.75
	end
	if armorReduction < 0 then
		armorReduction = 0
	end
	return armorReduction
end

function StatLogic:GetEffectFromDefense(defense, attackerLevel)
	return (defense - attackerLevel * 5) * 0.04
end


----------------------------------
-- GetEffectFromRating Function --
----------------------------------
--[[ Rating ID
CR_WEAPON_SKILL = 1;
CR_DEFENSE_SKILL = 2;
CR_DODGE = 3;
CR_PARRY = 4;
CR_BLOCK = 5;
CR_HIT_MELEE = 6;
CR_HIT_RANGED = 7;
CR_HIT_SPELL = 8;
CR_CRIT_MELEE = 9;
CR_CRIT_RANGED = 10;
CR_CRIT_SPELL = 11;
CR_HIT_TAKEN_MELEE = 12;
CR_HIT_TAKEN_RANGED = 13;
CR_HIT_TAKEN_SPELL = 14;
CR_CRIT_TAKEN_MELEE = 15;
CR_CRIT_TAKEN_RANGED = 16;
CR_CRIT_TAKEN_SPELL = 17;
CR_HASTE_MELEE = 18;
CR_HASTE_RANGED = 19;
CR_HASTE_SPELL = 20;
CR_WEAPON_SKILL_MAINHAND = 21;
CR_WEAPON_SKILL_OFFHAND = 22;
CR_WEAPON_SKILL_RANGED = 23;
--]]

-- Level 60 rating base
local RatingBase = {
	[CR_WEAPON_SKILL] = 2.5,
	[CR_DEFENSE_SKILL] = 1.5,
	[CR_DODGE] = 12,
	[CR_PARRY] = 20,
	[CR_BLOCK] = 5,
	[CR_HIT_MELEE] = 10,
	[CR_HIT_RANGED] = 10,
	[CR_HIT_SPELL] = 8,
	[CR_CRIT_MELEE] = 14,
	[CR_CRIT_RANGED] = 14,
	[CR_CRIT_SPELL] = 14,
	[CR_HIT_TAKEN_MELEE] = 10, -- hit avoidance
	[CR_HIT_TAKEN_RANGED] = 10,
	[CR_HIT_TAKEN_SPELL] = 8,
	[CR_CRIT_TAKEN_MELEE] = 25, -- resilience
	[CR_CRIT_TAKEN_RANGED] = 25,
	[CR_CRIT_TAKEN_SPELL] = 25,
	[CR_HASTE_MELEE] = 20 / 3,
	[CR_HASTE_RANGED] = 20 / 3,
	[CR_HASTE_SPELL] = 20 / 3,
	[CR_WEAPON_SKILL_MAINHAND] = 2.5,
	[CR_WEAPON_SKILL_OFFHAND] = 2.5,
	[CR_WEAPON_SKILL_RANGED] = 2.5,
}

-- Formula reverse engineered by Whitetooth@Cenarius (hotdogee@bahamut.twbbs.org)

--[[---------------------------------
{	:GetEffectFromRating(rating, id, level)
-------------------------------------
-- Description
	Gets the effective chance from ratings
-- Args
	rating
	    number - rating value
	id
	    number - rating id (global defined in PaperDollFrame.lua)
	[level]
	    number - player level
-- Examples
	StatLogic:GetEffectFromRating(10, CR_DODGE, 70)
}
-----------------------------------]]
function StatLogic:GetEffectFromRating(rating, id, level)
	-- check for invalid input
	if type(rating) ~= "number" or id < 1 or id > 23 then return 0 end
	-- defaults to player level if not given
	level = level or UnitLevel("player")
	if level >= 60 then
		return rating/RatingBase[id]*((-3/82)*level+(131/41))
	elseif level >= 10 then
		return rating/RatingBase[id]/((1/52)*level-(8/52))
	else
		return rating/RatingBase[id]/((1/52)*10-(8/52))
	end
end

---------------------------
-- GetAPFromStr Function --
---------------------------
local APPerStr = {
	2, 2, 1, 1, 1, 2, 1, 1, 2,
	--["WARRIOR"] = 2,
	--["PALADIN"] = 2,
	--["HUNTER"] = 1,
	--["ROGUE"] = 1,
	--["PRIEST"] = 1,
	--["SHAMAN"] = 2,
	--["MAGE"] = 1,
	--["WARLOCK"] = 1,
	--["DRUID"] = 2,
}

function StatLogic:GetAPPerStr(class)
	-- if class is a class string, convert to class id
	if type(class) == "string" and ClassNameToID[strupper(class)] ~= nil then
		class = ClassNameToID[strupper(class)]
	-- if class is invalid input, default to player class
	elseif type(class) ~= "number" or class < 1 or class > 9 then
		local _
		_, class = UnitClass("player")
		class = ClassNameToID[class]
	end
	return APPerStr[class]
end

---------------------------
-- GetBlockValueFromStr Function --
---------------------------
local BlockValuePerStr = {
	0.04, 0.04, 0, 0, 0, 0.04, 0, 0, 0,
	--["WARRIOR"] = 0.04,
	--["PALADIN"] = 0.04,
	--["HUNTER"] = 0,
	--["ROGUE"] = 0,
	--["PRIEST"] = 0,
	--["SHAMAN"] = 0.04,
	--["MAGE"] = 0,
	--["WARLOCK"] = 0,
	--["DRUID"] = 0,
}

function StatLogic:GetBlockValuePerStr(class)
	-- if class is a class string, convert to class id
	if type(class) == "string" and ClassNameToID[strupper(class)] ~= nil then
		class = ClassNameToID[strupper(class)]
	-- if class is invalid input, default to player class
	elseif type(class) ~= "number" or class < 1 or class > 9 then
		local _
		_, class = UnitClass("player")
		class = ClassNameToID[class]
	end
	return BlockValuePerStr[class]
end

---------------------------
-- GetAPFromAgi Function --
---------------------------
local APPerAgi = {
	0, 0, 1, 1, 0, 0, 0, 0, 0,
	--["WARRIOR"] = 0,
	--["PALADIN"] = 0,
	--["HUNTER"] = 1,
	--["ROGUE"] = 1,
	--["PRIEST"] = 0,
	--["SHAMAN"] = 0,
	--["MAGE"] = 0,
	--["WARLOCK"] = 0,
	--["DRUID"] = 0,
}

function StatLogic:GetAPPerAgi(class)
	-- if class is a class string, convert to class id
	if type(class) == "string" and ClassNameToID[strupper(class)] ~= nil then
		class = ClassNameToID[strupper(class)]
	-- if class is invalid input, default to player class
	elseif type(class) ~= "number" or class < 1 or class > 9 then
		local _
		_, class = UnitClass("player")
		class = ClassNameToID[class]
	end
	-- Check druid cat form
	if (class == 9) and GetPlayerBuffName(BS["Cat Form"]) then
		return 1
	end
	return APPerAgi[class]
end

---------------------------
-- GetRAPFromAgi Function --
---------------------------
local RAPPerAgi = {
	1, 0, 1, 1, 0, 0, 0, 0, 0,
	--["WARRIOR"] = 1,
	--["PALADIN"] = 0,
	--["HUNTER"] = 1,
	--["ROGUE"] = 1,
	--["PRIEST"] = 0,
	--["SHAMAN"] = 0,
	--["MAGE"] = 0,
	--["WARLOCK"] = 0,
	--["DRUID"] = 0,
}

function StatLogic:GetRAPPerAgi(class)
	-- if class is a class string, convert to class id
	if type(class) == "string" and ClassNameToID[strupper(class)] ~= nil then
		class = ClassNameToID[strupper(class)]
	-- if class is invalid input, default to player class
	elseif type(class) ~= "number" or class < 1 or class > 9 then
		local _
		_, class = UnitClass("player")
		class = ClassNameToID[class]
	end
	return RAPPerAgi[class]
end

------------------------------
-- GetDodgeFromAgi Function --
------------------------------
-- AgiToDodgeBase 
-- Derived by Whitetooth@Cenarius (hotdogee@bahamut.twbbs.org)
local AgiToDodgeBase = {
	0.7580, 0.6520, -5.4500, -0.5900, 3.1830, 1.6750, 3.4575, 2.0350, -1.8720,
	--["WARRIOR"] = 0.7580,
	--["PALADIN"] = 0.6520,
	--["HUNTER"] = -5.4500,
	--["ROGUE"] = -0.5900,
	--["PRIEST"] = 3.1830,
	--["SHAMAN"] = 1.6750,
	--["MAGE"] = 3.4575,
	--["WARLOCK"] = 2.0350,
	--["DRUID"] = -1.8720,
}

-- Calculates dodge per agi for your class and your current level
function StatLogic:GetDodgePerAgi(class)
	local _, agility = UnitStat("player", 2)
	-- if class is a class string, convert to class id
	if type(class) == "string" and ClassNameToID[strupper(class)] ~= nil then
		class = ClassNameToID[strupper(class)]
	-- if class is invalid input, default to player class
	elseif type(class) ~= "number" or class < 1 or class > 9 then
		local _
		_, class = UnitClass("player")
		class = ClassNameToID[class]
	end
	-- dodgeFromAgi is %
	local dodgeFromAgi = GetDodgeChance() - self:GetStatMod("ADD_DODGE") - self:GetEffectFromRating(GetCombatRating(CR_DODGE), CR_DODGE, UnitLevel("player")) - self:GetEffectFromDefense(GetTotalDefense("player"), UnitLevel("player"))
	return (dodgeFromAgi - AgiToDodgeBase[class]) / agility
end

--[[---------------------------------
{	:GetDodgeFromAgi(agi)
-------------------------------------
-- Description
	Gets your dodge chance from agility
-- Args
	agi
	    number - agi value
-- Examples
	StatLogic:GetDodgeFromAgi(10)
}
-----------------------------------]]
function StatLogic:GetDodgeFromAgi(agi)
	-- check for invalid input
	self:argCheck(agi, 2, "number")
	-- Calculate
	return agi * self:GetDodgePerAgi()
end


-----------------------------
-- GetCritFromAgi Function --
-----------------------------
-- Numbers reverse engineered by Whitetooth@Cenarius (hotdogee@bahamut.twbbs.org)
local CritPerAgi = {
	 [1] = {0.2500, 0.2174, 0.2840, 0.4476, 0.0909, 0.1663, 0.0771, 0.1500, 0.2020, },
	 [2] = {0.2381, 0.2070, 0.2834, 0.4290, 0.0909, 0.1663, 0.0771, 0.1500, 0.2020, },
	 [3] = {0.2381, 0.2070, 0.2711, 0.4118, 0.0909, 0.1583, 0.0771, 0.1429, 0.1923, },
	 [4] = {0.2273, 0.1976, 0.2530, 0.3813, 0.0865, 0.1583, 0.0735, 0.1429, 0.1923, },
	 [5] = {0.2174, 0.1976, 0.2430, 0.3677, 0.0865, 0.1511, 0.0735, 0.1429, 0.1836, },
	 [6] = {0.2083, 0.1890, 0.2337, 0.3550, 0.0865, 0.1511, 0.0735, 0.1364, 0.1836, },
	 [7] = {0.2083, 0.1890, 0.2251, 0.3321, 0.0865, 0.1511, 0.0735, 0.1364, 0.1756, },
	 [8] = {0.2000, 0.1812, 0.2171, 0.3217, 0.0826, 0.1446, 0.0735, 0.1364, 0.1756, },
	 [9] = {0.1923, 0.1812, 0.2051, 0.3120, 0.0826, 0.1446, 0.0735, 0.1304, 0.1683, },
	[10] = {0.1923, 0.1739, 0.1984, 0.2941, 0.0826, 0.1385, 0.0701, 0.1304, 0.1553, },
	[11] = {0.1852, 0.1739, 0.1848, 0.2640, 0.0826, 0.1385, 0.0701, 0.1250, 0.1496, },
	[12] = {0.1786, 0.1672, 0.1670, 0.2394, 0.0790, 0.1330, 0.0701, 0.1250, 0.1496, },
	[13] = {0.1667, 0.1553, 0.1547, 0.2145, 0.0790, 0.1330, 0.0701, 0.1250, 0.1443, },
	[14] = {0.1613, 0.1553, 0.1441, 0.1980, 0.0790, 0.1279, 0.0701, 0.1200, 0.1443, },
	[15] = {0.1563, 0.1449, 0.1330, 0.1775, 0.0790, 0.1231, 0.0671, 0.1154, 0.1346, },
	[16] = {0.1515, 0.1449, 0.1267, 0.1660, 0.0757, 0.1188, 0.0671, 0.1111, 0.1346, },
	[17] = {0.1471, 0.1403, 0.1194, 0.1560, 0.0757, 0.1188, 0.0671, 0.1111, 0.1303, },
	[18] = {0.1389, 0.1318, 0.1117, 0.1450, 0.0757, 0.1147, 0.0671, 0.1111, 0.1262, },
	[19] = {0.1351, 0.1318, 0.1060, 0.1355, 0.0727, 0.1147, 0.0671, 0.1071, 0.1262, },
	[20] = {0.1282, 0.1242, 0.0998, 0.1271, 0.0727, 0.1073, 0.0643, 0.1034, 0.1122, },
	[21] = {0.1282, 0.1208, 0.0962, 0.1197, 0.0727, 0.1073, 0.0643, 0.1000, 0.1122, },
	[22] = {0.1250, 0.1208, 0.0910, 0.1144, 0.0727, 0.1039, 0.0643, 0.1000, 0.1092, },
	[23] = {0.1190, 0.1144, 0.0872, 0.1084, 0.0699, 0.1039, 0.0643, 0.0968, 0.1063, },
	[24] = {0.1163, 0.1115, 0.0829, 0.1040, 0.0699, 0.1008, 0.0617, 0.0968, 0.1063, },
	[25] = {0.1111, 0.1087, 0.0797, 0.0980, 0.0699, 0.0978, 0.0617, 0.0909, 0.1010, },
	[26] = {0.1087, 0.1060, 0.0767, 0.0936, 0.0673, 0.0950, 0.0617, 0.0909, 0.1010, },
	[27] = {0.1064, 0.1035, 0.0734, 0.0903, 0.0673, 0.0950, 0.0617, 0.0909, 0.0985, },
	[28] = {0.1020, 0.1011, 0.0709, 0.0865, 0.0673, 0.0924, 0.0617, 0.0882, 0.0962, },
	[29] = {0.1000, 0.0988, 0.0680, 0.0830, 0.0649, 0.0924, 0.0593, 0.0882, 0.0962, },
	[30] = {0.0962, 0.0945, 0.0654, 0.0792, 0.0649, 0.0875, 0.0593, 0.0833, 0.0878, },
	[31] = {0.0943, 0.0925, 0.0637, 0.0768, 0.0649, 0.0875, 0.0593, 0.0833, 0.0859, },
	[32] = {0.0926, 0.0925, 0.0614, 0.0741, 0.0627, 0.0853, 0.0593, 0.0811, 0.0859, },
	[33] = {0.0893, 0.0887, 0.0592, 0.0715, 0.0627, 0.0831, 0.0571, 0.0811, 0.0841, },
	[34] = {0.0877, 0.0870, 0.0575, 0.0691, 0.0627, 0.0831, 0.0571, 0.0789, 0.0824, },
	[35] = {0.0847, 0.0836, 0.0556, 0.0664, 0.0606, 0.0792, 0.0571, 0.0769, 0.0808, },
	[36] = {0.0833, 0.0820, 0.0541, 0.0643, 0.0606, 0.0773, 0.0551, 0.0750, 0.0792, },
	[37] = {0.0820, 0.0820, 0.0524, 0.0628, 0.0606, 0.0773, 0.0551, 0.0732, 0.0777, },
	[38] = {0.0794, 0.0791, 0.0508, 0.0609, 0.0586, 0.0756, 0.0551, 0.0732, 0.0777, },
	[39] = {0.0781, 0.0776, 0.0493, 0.0592, 0.0586, 0.0756, 0.0551, 0.0714, 0.0762, },
	[40] = {0.0758, 0.0750, 0.0481, 0.0572, 0.0586, 0.0723, 0.0532, 0.0698, 0.0709, },
	[41] = {0.0735, 0.0737, 0.0470, 0.0556, 0.0568, 0.0707, 0.0532, 0.0682, 0.0696, },
	[42] = {0.0725, 0.0737, 0.0457, 0.0542, 0.0568, 0.0707, 0.0532, 0.0682, 0.0696, },
	[43] = {0.0704, 0.0713, 0.0444, 0.0528, 0.0551, 0.0693, 0.0532, 0.0667, 0.0685, },
	[44] = {0.0694, 0.0701, 0.0433, 0.0512, 0.0551, 0.0679, 0.0514, 0.0667, 0.0673, },
	[45] = {0.0676, 0.0679, 0.0421, 0.0497, 0.0551, 0.0665, 0.0514, 0.0638, 0.0651, },
	[46] = {0.0667, 0.0669, 0.0413, 0.0486, 0.0534, 0.0652, 0.0514, 0.0625, 0.0641, },
	[47] = {0.0649, 0.0659, 0.0402, 0.0474, 0.0534, 0.0639, 0.0498, 0.0625, 0.0641, },
	[48] = {0.0633, 0.0639, 0.0391, 0.0464, 0.0519, 0.0627, 0.0498, 0.0612, 0.0631, },
	[49] = {0.0625, 0.0630, 0.0382, 0.0454, 0.0519, 0.0627, 0.0498, 0.0600, 0.0621, },
	[50] = {0.0610, 0.0612, 0.0373, 0.0440, 0.0519, 0.0605, 0.0482, 0.0588, 0.0585, },
	[51] = {0.0595, 0.0604, 0.0366, 0.0431, 0.0505, 0.0594, 0.0482, 0.0577, 0.0577, },
	[52] = {0.0588, 0.0596, 0.0358, 0.0422, 0.0505, 0.0583, 0.0482, 0.0577, 0.0569, },
	[53] = {0.0575, 0.0580, 0.0350, 0.0412, 0.0491, 0.0583, 0.0467, 0.0566, 0.0561, },
	[54] = {0.0562, 0.0572, 0.0341, 0.0404, 0.0491, 0.0573, 0.0467, 0.0556, 0.0561, },
	[55] = {0.0549, 0.0557, 0.0334, 0.0394, 0.0478, 0.0554, 0.0467, 0.0545, 0.0546, },
	[56] = {0.0543, 0.0550, 0.0328, 0.0386, 0.0478, 0.0545, 0.0454, 0.0536, 0.0539, },
	[57] = {0.0532, 0.0544, 0.0321, 0.0378, 0.0466, 0.0536, 0.0454, 0.0526, 0.0531, },
	[58] = {0.0521, 0.0530, 0.0314, 0.0370, 0.0466, 0.0536, 0.0454, 0.0517, 0.0525, },
	[59] = {0.0510, 0.0524, 0.0307, 0.0364, 0.0454, 0.0528, 0.0441, 0.0517, 0.0518, },
	[60] = {0.0500, 0.0512, 0.0301, 0.0355, 0.0454, 0.0512, 0.0441, 0.0500, 0.0493, },
	[61] = {0.0469, 0.0491, 0.0297, 0.0334, 0.0443, 0.0496, 0.0435, 0.0484, 0.0478, },
	[62] = {0.0442, 0.0483, 0.0290, 0.0322, 0.0444, 0.0486, 0.0432, 0.0481, 0.0472, },
	[63] = {0.0418, 0.0472, 0.0284, 0.0307, 0.0441, 0.0470, 0.0424, 0.0470, 0.0456, },
	[64] = {0.0397, 0.0456, 0.0279, 0.0296, 0.0433, 0.0456, 0.0423, 0.0455, 0.0447, },
	[65] = {0.0377, 0.0446, 0.0273, 0.0286, 0.0426, 0.0449, 0.0422, 0.0448, 0.0438, },
	[66] = {0.0360, 0.0437, 0.0270, 0.0276, 0.0419, 0.0437, 0.0411, 0.0435, 0.0430, },
	[67] = {0.0344, 0.0425, 0.0264, 0.0268, 0.0414, 0.0427, 0.0412, 0.0436, 0.0424, },
	[68] = {0.0329, 0.0416, 0.0259, 0.0262, 0.0412, 0.0417, 0.0408, 0.0424, 0.0412, },
	[69] = {0.0315, 0.0408, 0.0254, 0.0256, 0.0410, 0.0408, 0.0404, 0.0414, 0.0406, },
	[70] = {0.0303, 0.0400, 0.0250, 0.0250, 0.0400, 0.0400, 0.0400, 0.0405, 0.0400, },
	[71] = {0.0297, 0.0393, 0.0246, 0.0244, 0.0390, 0.0392, 0.0396, 0.0396, 0.0394, },
	[72] = {0.0292, 0.0385, 0.0242, 0.0238, 0.0381, 0.0384, 0.0393, 0.0387, 0.0388, },
	[73] = {0.0287, 0.0378, 0.0238, 0.0233, 0.0372, 0.0377, 0.0389, 0.0379, 0.0383, },
}

--[[---------------------------------
{	:GetCritFromAgi(agi, class, level)
-------------------------------------
-- Description
	Gets the crit chance from agility
-- Args
	agi
	    number - agi value
	[class]
	    string - english class name
	    number - class id
	[level]
	    number - player level
-- Examples
	StatLogic:GetCritFromAgi(10, "WARRIOR", 70)
}
-----------------------------------]]
function StatLogic:GetCritFromAgi(agi, class, level)
	-- check for invalid input
	if type(agi) ~= "number" then return end
	-- if level is invalid input, default to player level
	if type(level) ~= "number" or level < 1 or level > 73 then
		level = UnitLevel("player")
	end
	-- if class is a class string, convert to class id
	if type(class) == "string" and ClassNameToID[strupper(class)] ~= nil then
		class = ClassNameToID[strupper(class)]
	-- if class is invalid input, default to player class
	elseif type(class) ~= "number" or class < 1 or class > 9 then
		local _
		_, class = UnitClass("player")
		class = ClassNameToID[class]
	end
	-- Calculate
	return agi * CritPerAgi[level][class]
end


----------------------------------
-- GetSpellCritFromInt Function --
----------------------------------
-- Numbers reverse engineered by Whitetooth@Cenarius (hotdogee@bahamut.twbbs.org)
local SpellCritPerInt = {
	 [1] = {0.0000, 0.0832, 0.0699, 0.0000, 0.1710, 0.1333, 0.1637, 0.1500, 0.1431, },
	 [2] = {0.0000, 0.0793, 0.0666, 0.0000, 0.1636, 0.1272, 0.1574, 0.1435, 0.1369, },
	 [3] = {0.0000, 0.0793, 0.0666, 0.0000, 0.1568, 0.1217, 0.1516, 0.1375, 0.1312, },
	 [4] = {0.0000, 0.0757, 0.0635, 0.0000, 0.1505, 0.1217, 0.1411, 0.1320, 0.1259, },
	 [5] = {0.0000, 0.0757, 0.0635, 0.0000, 0.1394, 0.1166, 0.1364, 0.1269, 0.1211, },
	 [6] = {0.0000, 0.0724, 0.0608, 0.0000, 0.1344, 0.1120, 0.1320, 0.1222, 0.1166, },
	 [7] = {0.0000, 0.0694, 0.0608, 0.0000, 0.1297, 0.1077, 0.1279, 0.1179, 0.1124, },
	 [8] = {0.0000, 0.0694, 0.0583, 0.0000, 0.1254, 0.1037, 0.1240, 0.1138, 0.1124, },
	 [9] = {0.0000, 0.0666, 0.0583, 0.0000, 0.1214, 0.1000, 0.1169, 0.1100, 0.1086, },
	[10] = {0.0000, 0.0666, 0.0559, 0.0000, 0.1140, 0.1000, 0.1137, 0.1065, 0.0984, },
	[11] = {0.0000, 0.0640, 0.0559, 0.0000, 0.1045, 0.0933, 0.1049, 0.0971, 0.0926, },
	[12] = {0.0000, 0.0616, 0.0538, 0.0000, 0.0941, 0.0875, 0.0930, 0.0892, 0.0851, },
	[13] = {0.0000, 0.0594, 0.0499, 0.0000, 0.0875, 0.0800, 0.0871, 0.0825, 0.0807, },
	[14] = {0.0000, 0.0574, 0.0499, 0.0000, 0.0784, 0.0756, 0.0731, 0.0767, 0.0750, },
	[15] = {0.0000, 0.0537, 0.0466, 0.0000, 0.0724, 0.0700, 0.0671, 0.0717, 0.0684, },
	[16] = {0.0000, 0.0537, 0.0466, 0.0000, 0.0684, 0.0666, 0.0639, 0.0688, 0.0656, },
	[17] = {0.0000, 0.0520, 0.0451, 0.0000, 0.0627, 0.0636, 0.0602, 0.0635, 0.0617, },
	[18] = {0.0000, 0.0490, 0.0424, 0.0000, 0.0597, 0.0596, 0.0568, 0.0600, 0.0594, },
	[19] = {0.0000, 0.0490, 0.0424, 0.0000, 0.0562, 0.0571, 0.0538, 0.0569, 0.0562, },
	[20] = {0.0000, 0.0462, 0.0399, 0.0000, 0.0523, 0.0538, 0.0505, 0.0541, 0.0516, },
	[21] = {0.0000, 0.0450, 0.0388, 0.0000, 0.0502, 0.0518, 0.0487, 0.0516, 0.0500, },
	[22] = {0.0000, 0.0438, 0.0388, 0.0000, 0.0470, 0.0500, 0.0460, 0.0493, 0.0477, },
	[23] = {0.0000, 0.0427, 0.0368, 0.0000, 0.0453, 0.0474, 0.0445, 0.0471, 0.0463, },
	[24] = {0.0000, 0.0416, 0.0358, 0.0000, 0.0428, 0.0459, 0.0422, 0.0446, 0.0437, },
	[25] = {0.0000, 0.0396, 0.0350, 0.0000, 0.0409, 0.0437, 0.0405, 0.0429, 0.0420, },
	[26] = {0.0000, 0.0387, 0.0341, 0.0000, 0.0392, 0.0424, 0.0390, 0.0418, 0.0409, },
	[27] = {0.0000, 0.0387, 0.0333, 0.0000, 0.0376, 0.0412, 0.0372, 0.0398, 0.0394, },
	[28] = {0.0000, 0.0370, 0.0325, 0.0000, 0.0362, 0.0394, 0.0338, 0.0384, 0.0384, },
	[29] = {0.0000, 0.0362, 0.0318, 0.0000, 0.0348, 0.0383, 0.0325, 0.0367, 0.0366, },
	[30] = {0.0000, 0.0347, 0.0304, 0.0000, 0.0333, 0.0368, 0.0312, 0.0355, 0.0346, },
	[31] = {0.0000, 0.0340, 0.0297, 0.0000, 0.0322, 0.0354, 0.0305, 0.0347, 0.0339, },
	[32] = {0.0000, 0.0333, 0.0297, 0.0000, 0.0311, 0.0346, 0.0294, 0.0333, 0.0325, },
	[33] = {0.0000, 0.0326, 0.0285, 0.0000, 0.0301, 0.0333, 0.0286, 0.0324, 0.0318, },
	[34] = {0.0000, 0.0320, 0.0280, 0.0000, 0.0289, 0.0325, 0.0278, 0.0311, 0.0309, },
	[35] = {0.0000, 0.0308, 0.0269, 0.0000, 0.0281, 0.0314, 0.0269, 0.0303, 0.0297, },
	[36] = {0.0000, 0.0303, 0.0264, 0.0000, 0.0273, 0.0304, 0.0262, 0.0295, 0.0292, },
	[37] = {0.0000, 0.0297, 0.0264, 0.0000, 0.0263, 0.0298, 0.0254, 0.0284, 0.0284, },
	[38] = {0.0000, 0.0287, 0.0254, 0.0000, 0.0256, 0.0289, 0.0248, 0.0277, 0.0276, },
	[39] = {0.0000, 0.0282, 0.0250, 0.0000, 0.0249, 0.0283, 0.0241, 0.0268, 0.0269, },
	[40] = {0.0000, 0.0273, 0.0241, 0.0000, 0.0241, 0.0272, 0.0235, 0.0262, 0.0256, },
	[41] = {0.0000, 0.0268, 0.0237, 0.0000, 0.0235, 0.0267, 0.0230, 0.0256, 0.0252, },
	[42] = {0.0000, 0.0264, 0.0237, 0.0000, 0.0228, 0.0262, 0.0215, 0.0248, 0.0244, },
	[43] = {0.0000, 0.0256, 0.0229, 0.0000, 0.0223, 0.0254, 0.0211, 0.0243, 0.0240, },
	[44] = {0.0000, 0.0256, 0.0225, 0.0000, 0.0216, 0.0248, 0.0206, 0.0236, 0.0233, },
	[45] = {0.0000, 0.0248, 0.0218, 0.0000, 0.0210, 0.0241, 0.0201, 0.0229, 0.0228, },
	[46] = {0.0000, 0.0245, 0.0215, 0.0000, 0.0206, 0.0235, 0.0197, 0.0224, 0.0223, },
	[47] = {0.0000, 0.0238, 0.0212, 0.0000, 0.0200, 0.0231, 0.0192, 0.0220, 0.0219, },
	[48] = {0.0000, 0.0231, 0.0206, 0.0000, 0.0196, 0.0226, 0.0188, 0.0214, 0.0214, },
	[49] = {0.0000, 0.0228, 0.0203, 0.0000, 0.0191, 0.0220, 0.0184, 0.0209, 0.0209, },
	[50] = {0.0000, 0.0222, 0.0197, 0.0000, 0.0186, 0.0215, 0.0179, 0.0204, 0.0202, },
	[51] = {0.0000, 0.0219, 0.0194, 0.0000, 0.0183, 0.0210, 0.0176, 0.0200, 0.0198, },
	[52] = {0.0000, 0.0216, 0.0192, 0.0000, 0.0178, 0.0207, 0.0173, 0.0195, 0.0193, },
	[53] = {0.0000, 0.0211, 0.0186, 0.0000, 0.0175, 0.0201, 0.0170, 0.0191, 0.0191, },
	[54] = {0.0000, 0.0208, 0.0184, 0.0000, 0.0171, 0.0199, 0.0166, 0.0186, 0.0186, },
	[55] = {0.0000, 0.0203, 0.0179, 0.0000, 0.0166, 0.0193, 0.0162, 0.0182, 0.0182, },
	[56] = {0.0000, 0.0201, 0.0177, 0.0000, 0.0164, 0.0190, 0.0154, 0.0179, 0.0179, },
	[57] = {0.0000, 0.0198, 0.0175, 0.0000, 0.0160, 0.0187, 0.0151, 0.0176, 0.0176, },
	[58] = {0.0000, 0.0191, 0.0170, 0.0000, 0.0157, 0.0182, 0.0149, 0.0172, 0.0173, },
	[59] = {0.0000, 0.0189, 0.0168, 0.0000, 0.0154, 0.0179, 0.0146, 0.0168, 0.0169, },
	[60] = {0.0000, 0.0185, 0.0164, 0.0000, 0.0151, 0.0175, 0.0143, 0.0165, 0.0164, },
	[61] = {0.0000, 0.0157, 0.0157, 0.0000, 0.0148, 0.0164, 0.0143, 0.0159, 0.0162, },
	[62] = {0.0000, 0.0153, 0.0154, 0.0000, 0.0145, 0.0159, 0.0143, 0.0154, 0.0157, },
	[63] = {0.0000, 0.0148, 0.0150, 0.0000, 0.0143, 0.0152, 0.0143, 0.0148, 0.0150, },
	[64] = {0.0000, 0.0143, 0.0144, 0.0000, 0.0139, 0.0147, 0.0142, 0.0143, 0.0146, },
	[65] = {0.0000, 0.0140, 0.0141, 0.0000, 0.0137, 0.0142, 0.0142, 0.0138, 0.0142, },
	[66] = {0.0000, 0.0136, 0.0137, 0.0000, 0.0134, 0.0138, 0.0138, 0.0135, 0.0137, },
	[67] = {0.0000, 0.0133, 0.0133, 0.0000, 0.0132, 0.0134, 0.0133, 0.0130, 0.0133, },
	[68] = {0.0000, 0.0131, 0.0130, 0.0000, 0.0130, 0.0131, 0.0131, 0.0127, 0.0131, },
	[69] = {0.0000, 0.0128, 0.0128, 0.0000, 0.0127, 0.0128, 0.0128, 0.0125, 0.0128, },
	[70] = {0.0000, 0.0125, 0.0125, 0.0000, 0.0125, 0.0125, 0.0125, 0.0122, 0.0125, },
	[71] = {0.0000, 0.0122, 0.0123, 0.0000, 0.0123, 0.0122, 0.0122, 0.0119, 0.0122, },
	[72] = {0.0000, 0.0120, 0.0120, 0.0000, 0.0121, 0.0120, 0.0119, 0.0116, 0.0120, },
	[73] = {0.0000, 0.0118, 0.0118, 0.0000, 0.0119, 0.0117, 0.0117, 0.0114, 0.0118, },
}

--[[---------------------------------
{	:GetSpellCritFromInt(int, class, level)
-------------------------------------
-- Description
	Gets the spell crit chance from intellect
-- Args
	int
	    number - int value
	[class]
	    string - english class name
	    number - class id
	[level]
	    number - player level
-- Examples
	StatLogic:GetSpellCritFromInt(10, "MAGE", 70)
}
-----------------------------------]]
function StatLogic:GetSpellCritFromInt(int, class, level)
	-- check for invalid input
	if type(int) ~= "number" then return end
	-- if level is invalid input, default to player level
	if type(level) ~= "number" or level < 1 or level > 73 then
		level = UnitLevel("player")
	end
	-- if class is a class string, convert to class id
	if type(class) == "string" and ClassNameToID[strupper(class)] ~= nil then
		class = ClassNameToID[strupper(class)]
	-- if class is invalid input, default to player class
	elseif type(class) ~= "number" or class < 1 or class > 9 then
		local _
		_, class = UnitClass("player")
		class = ClassNameToID[class]
	end
	-- Calculate
	-- defaults to player level if not given
	return int * SpellCritPerInt[level][class]
end

----------------------------------
-- GetNormalManaRegenFromSpi Function --
----------------------------------
-- Numbers reverse engineered by Whitetooth@Cenarius (hotdogee@bahamut.twbbs.org)
local NormalManaRegenPerSpi = {
	0, 0.1, 0.1, 0, 0.125, 0, 0.1, 0.125, 0.1, 0, 0.1125,
}
--[[---------------------------------
{	:GetNormalManaRegenFromSpi(spi, class)
-------------------------------------
-- Description
	Gets the mana regen rate per 5 seconds while NOT casting from spirit
-- Args
	int
	    number - int value
	[class]
	    string - english class name
	    number - class id
-- Examples
	StatLogic:GetNormalManaRegenFromSpi(10, "MAGE")
}
-----------------------------------]]
function StatLogic:GetNormalManaRegenFromSpi(spi, class)
	-- check for invalid input
	if type(spi) ~= "number" then return end
	-- if class is a class string, convert to class id
	if type(class) == "string" and ClassNameToID[strupper(class)] ~= nil then
		class = ClassNameToID[strupper(class)]
	-- if class is invalid input, default to player class
	elseif type(class) ~= "number" or class < 1 or class > 9 then
		local _
		_, class = UnitClass("player")
		class = ClassNameToID[class]
	end
	-- Calculate
	-- defaults to player level if not given
	return spi * NormalManaRegenPerSpi[class] * 5
end

----------------------------------
-- GetHealthRegenFromSpi Function --
----------------------------------
-- Numbers reverse engineered by Whitetooth@Cenarius (hotdogee@bahamut.twbbs.org)
local HealthRegenPerSpi = {
	0.5, 0.125, 0.125, 0.333333, 0.041667, 0, 0.071429, 0.041667, 0.045455, 0, 0.0625,
}
--[[---------------------------------
{	:GetHealthRegenFromSpi(spi, class)
-------------------------------------
-- Description
	Gets the health regen rate per 5 seconds when out of combat from spirit
-- Args
	int
	    number - int value
	[class]
	    string - english class name
	    number - class id
-- Examples
	StatLogic:GetHealthRegenFromSpi(10, "MAGE")
}
-----------------------------------]]
function StatLogic:GetHealthRegenFromSpi(spi, class)
	-- check for invalid input
	if type(spi) ~= "number" then return end
	-- if class is a class string, convert to class id
	if type(class) == "string" and ClassNameToID[strupper(class)] ~= nil then
		class = ClassNameToID[strupper(class)]
	-- if class is invalid input, default to player class
	elseif type(class) ~= "number" or class < 1 or class > 9 then
		local _
		_, class = UnitClass("player")
		class = ClassNameToID[class]
	end
	-- Calculate
	-- defaults to player level if not given
	return spi * HealthRegenPerSpi[class] * 5
end

-- ================== --
-- Stat Summarization --
-- ================== --
--[[---------------------------------
{	:GetSum(item, table)
-------------------------------------
-- Description
	Gets a sum of stats from item
-- Args
	item
	    string - link or name of target item
	 or number - itemID of target item
	 or table - tooltip of target item
	[table]
	    table - the sum of stat values are writen to this table if provided
-- Examples
	StatLogic:GetSum(21417) -- [Ring of Unspoken Names]
	StatLogic:GetSum("item:28040:2717")
	StatLogic:GetSum("item:19019:117") -- TF
	StatLogic:GetSum("item:3185:0:0:0:0:0:1957") -- Acrobatic Staff of Frozen Wrath ID:3185:0:0:0:0:0:1957
	StatLogic:GetSum(24396)
	SetTip("item:3185:0:0:0:0:0:1957")
}
-----------------------------------]]

function StatLogic:GetSum(item, table)
	-- Locale check
	if GetLocale() ~= L["Locale"] then return end
	local _
	-- Check item
	if (type(item) == "string") or (type(item) == "number") then
	elseif type(item) == "table" and type(item.GetItem) == "function" then
		-- Get the link
		_, item = item:GetItem()
	else
		return
	end
	-- Check if item is in local cache
	local name, link, _, _, reqLv, _, _, _, itemType = GetItemInfo(item)
	if not name then return end

	-- Initialize table
	table = table or {}
	setmetatable(table, statTableMetatable)

	-- Clear table values
	for k in pairs(table) do
		table[k] = nil
	end

	-- Get data from cache if available
	if cache[link] then
		copyTable(table, cache[link])
		return table
	end

	-- Set metadata
	table.itemType = itemType
	-- Don't scan Relics because they don't have general stats
	if itemType == "INVTYPE_RELIC" then
		cache[link] = copy(table)
		return table
	end

	-- Start parsing
	tip:ClearLines() -- this is required or SetX won't work the second time its called
	tip:SetHyperlink(link)
	print(link)
	for i = 2, tip:NumLines() do
		local text = tip[i]:GetText()

		-- Trim spaces
		text = strtrim(text)
		-- Strip color codes
		if strsub(text, -2) == "|r" then
			text = strsub(text, 1, -3)
		end
		if strfind(strsub(text, 1, 10), "|c%x%x%x%x%x%x%x%x") then
			text = strsub(text, 11)
		end

		local _, g, b = tip[i]:GetTextColor()
		-- Fast Exclude --
		-- Exclude obvious strings that do not need to be checked, also exclude lines that are not white and green and normal (normal for Frozen Wrath bonus)
		if not (L.Exclude[text] or L.Exclude[strutf8sub(text, 1, L.ExcludeLen)] or strsub(text, 1, 1) == '"' or g < 0.8 or (b < 0.99 and b > 0.1)) then
			--print(text.." = ")
			local found
			local value, stat, idTable, id
			-----------------------
			-- WholeLine Lookups --
			-----------------------
			-- Mainly used for enchants or stuff without numbers:
			-- "Mithril Spurs"
			local idTable = L.WholeTextLookup[text]
			if idTable == false then
				found = true
				print("|cffadadad".."  WholeText Exclude: "..text)
			elseif idTable then
				found = true
				for id, value in pairs(L.WholeTextLookup[text]) do
					-- sum stat
					table[id] = (table[id] or 0) + value
					print("|cffff5959".."  WholeText: ".."|cffffc259"..text..", ".."|cffffff59"..tostring(id).."="..tostring(value))
				end
			end
			----------------------------
			-- Single Plus Stat Check --
			----------------------------
			-- depending on locale, L.SinglePlusStatCheck may be
			-- +19 Stamina = "^%+(%d+) ([%a ]+%a)$"
			-- Stamina +19 = "^([%a ]+%a) %+(%d+)$"
			-- +19 耐力 = "^%+(%d+) (.-)$"
			if not found then
				local _, _, value, statText = strfind(text, L.SinglePlusStatCheck)
				if value then
					if tonumber(statText) then
						value, statText = statText, value
					end
					idTable = L.StatIDLookup[statText]
					if idTable == false then
						found = true
						print("|cffadadad".."  SinglePlus Exclude: "..text)
					elseif idTable then
						found = true
						local debugText = "|cffff5959".."  SinglePlus: ".."|cffffc259"..text
						for _, id in ipairs(idTable) do
							--print("  '"..value.."', '"..id.."'")
							-- sum stat
							table[id] = (table[id] or 0) + tonumber(value)
							debugText = debugText..", ".."|cffffff59"..tostring(id).."="..tostring(value)
						end
						print(debugText)
					else
						-- pattern match but not found in L.StatIDLookup, keep looking
					end
				end
			end
			-----------------------------
			-- Single Equip Stat Check --
			-----------------------------
			-- depending on locale, L.SingleEquipStatCheck may be
			-- "^Equip: (.-) by u?p? ?t?o? ?(%d+) ?(.-)%.$"
			if not found then
				local _, _, statText1, value, statText2 = strfind(text, L.SingleEquipStatCheck)
				if value then
					local statText = statText1..statText2
					idTable = L.StatIDLookup[statText]
					if idTable == false then
						found = true
						print("|cffadadad".."  SingleEquip Exclude: "..text)
					elseif idTable then
						found = true
						local debugText = "|cffff5959".."  SingleEquip: ".."|cffffc259"..text
						for _, id in ipairs(idTable) do
							--print("  '"..value.."', '"..id.."'")
							-- sum stat
							table[id] = (table[id] or 0) + tonumber(value)
							debugText = debugText..", ".."|cffffff59"..tostring(id).."="..tostring(value)
						end
						print(debugText)
					else
						-- pattern match but not found in L.StatIDLookup, keep looking
					end
				end
			end
			-- PreScan for special cases, that will fit wrongly into DeepScan
			-- PreScan also has exclude patterns
			if not found then
				for pattern, id in pairs(L.PreScanPatterns) do
					found, _, value = strfind(text, pattern)
					if found then
						--found = true
						if id ~= false then
							local debugText = "|cffff5959".."  PreScan: ".."|cffffc259"..text
							--print("  '"..value.."' = '"..id.."'")
							-- sum stat
							table[id] = (table[id] or 0) + tonumber(value)
							debugText = debugText..", ".."|cffffff59"..tostring(id).."="..tostring(value)
							print(debugText)
						else
							print("|cffadadad".."  PreScan Exclude: "..text)
						end
						break
					end
				end
				if found then

				end
			end
			--------------
			-- DeepScan --
			--------------
			--[[
			-- Strip leading "Equip: ", "Socket Bonus: "
			["Equip: "] = "Equip: ",
			["Socket Bonus: "] = "Socket Bonus: ",
			-- Strip trailing "."
			["."] = ".",
			["DeepScanSeparators"] = {
				" and ", -- "Critical Rating +6 and Dodge Rating +5": Assassin's Fire Opal ID:30565
				"/", -- "+10 Defense Rating/+10 Stamina/+15 Block Value": ZG Enchant
				" & ", -- "+26 Healing Spells & 2% Reduced Threat": Bracing Earthstorm Diamond ID:25897
				", ", -- "+6 Spell Damage, +5 Spell Crit Rating": Potent Ornate Topaz ID: 28123
				"%. ", -- "Equip: Increases attack power by 81 when fighting Undead.  It also allows the acquisition of Scourgestones on behalf of the Argent Dawn.": Seal of the Dawn
			},
			["DeepScanPatterns"] = {
				"^(.-) by u?p? ?t?o? ?(%d+) ?(.-)$", -- "xxx by up to 22 xxx" (scan first)
				"^(.-) ?%+(%d+) ?(.-)$", -- "xxx xxx +22" or "+22 xxx xxx" or "xxx +22 xxx" (scan 2ed)
				"^(.-) ?([%d%.]+) ?(.-)$", -- 22.22 xxx xxx (scan last)
			},
			--]]
			if not found then
				-- Get a local copy
				local text = text
				-- Trim spaces
				text = strtrim(text)
				-- Strip leading "Equip: ", "Socket Bonus: "
				text = gsub(text, L["Equip: "], "")
				text = gsub(text, L["Socket Bonus: "], "")
				-- Strip trailing "."
				if strutf8sub(text, -1) == L["."] then
					text = strutf8sub(text, 1, -2)
				end
				-- Replace separators with @
				for _, sep in ipairs(L.DeepScanSeparators) do
					if strfind(text, sep) then
						text = gsub(text, sep, "@")
					end
				end
				-- Split text using @
				text = {strsplit("@", text)}
				for _, text in ipairs(text) do
					local foundWholeText = false
					-- Trim spaces
					text = strtrim(text)
					print("|cff008080".."'"..text.."'")
					-- Whole Text Lookup
					local idTable = L.WholeTextLookup[text]
					if idTable == false then
						foundWholeText = true
						found = true
						print("|cffadadad".."  DeepScan Exclude: "..text)
					elseif idTable then
						foundWholeText = true
						found = true
						for id, value in pairs(L.WholeTextLookup[text]) do
							-- sum stat
							table[id] = (table[id] or 0) + value
							print("|cffff5959".."  DeepScan: ".."|cffffc259"..text..", ".."|cffffff59"..tostring(id).."="..tostring(value))
						end
					else
						-- pattern match but not found in L.WholeTextLookup, keep looking
					end
					if not foundWholeText then
						-- Pattern scan
						for _, pattern in ipairs(L.DeepScanPatterns) do
							local _, _, statText1, value, statText2 = strfind(text, pattern)
							if value then
								local statText = statText1..statText2
								idTable = L.StatIDLookup[statText]
								if idTable == false then
									found = true
									print("|cffadadad".."  DeepScan Exclude: "..text)
									break
								elseif idTable then
									found = true
									local debugText = "|cffff5959".."  DeepScan: ".."|cffffc259"..text
									for _, id in ipairs(idTable) do
										--print("  '"..value.."', '"..id.."'")
										-- sum stat
										table[id] = (table[id] or 0) + tonumber(value)
										debugText = debugText..", ".."|cffffff59"..tostring(id).."="..tostring(value)
									end
									print(debugText)
									break
								else
									-- pattern match but not found in L.StatIDLookup, keep looking
									print("  DeepScan Lookup Fail: |cffff0000'"..statText.."'|r, pattern = |cff72ff59'"..pattern.."'")
								end
							end
						end
					end
				end
			end

			if not found then
				print("  No Match: |cffff0000'"..text.."'")
				if DEBUG and RatingBuster then
					RatingBuster.db.profile.test = text
				end
			end
		else
			--print("Excluded: "..text)
		end
	end
	cache[link] = copy(table)
	setmetatable(table, statTableMetatable)
	return table
end

--[[---------------------------------
{	:GetDiff(item, table1, table2)
-------------------------------------
-- Description
	Gets the stat diffrence from item and equipped items
-- Args
	item
	    string - link or name of target item
	 or number - itemID of target item
	 or table - tooltip of target item
	[table1]
	    table - stat difference of item and equipped item 1 are writen to this table if provided
	[table2]
	    table - stat difference of item and equipped item 2 are writen to this table if provided
-- Examples
	StatLogic:GetDiff(21417, {}) -- Ring of Unspoken Names
	StatLogic:GetDiff(21452) -- Staff of the Ruins
}
-----------------------------------]]
local getSlotID = {
	INVTYPE_AMMO           = 0,
	INVTYPE_GUNPROJECTILE  = 0,
	INVTYPE_BOWPROJECTILE  = 0,
	INVTYPE_HEAD           = 1,
	INVTYPE_NECK           = 2,
	INVTYPE_SHOULDER       = 3,
	INVTYPE_BODY           = 4,
	INVTYPE_CHEST          = 5,
	INVTYPE_ROBE           = 5,
	INVTYPE_WAIST          = 6,
	INVTYPE_LEGS           = 7,
	INVTYPE_FEET           = 8,
	INVTYPE_WRIST          = 9,
	INVTYPE_HAND           = 10,
	INVTYPE_FINGER         = {11,12},
	INVTYPE_TRINKET        = {13,14},
	INVTYPE_CLOAK          = 15,
	INVTYPE_WEAPON         = {16,17},
	INVTYPE_2HWEAPON       = 16+17,
	INVTYPE_WEAPONMAINHAND = 16,
	INVTYPE_WEAPONOFFHAND  = 17,
	INVTYPE_SHIELD         = 17,
	INVTYPE_HOLDABLE       = 17,
	INVTYPE_RANGED         = 18,
	INVTYPE_RANGEDRIGHT    = 18,
	INVTYPE_RELIC          = 18,
	INVTYPE_GUN            = 18,
	INVTYPE_CROSSBOW       = 18,
	INVTYPE_WAND           = 18,
	INVTYPE_THROWN         = 18,
	INVTYPE_TABARD         = 19,
}

SLTest = {}

function StatLogic:GetDiff(item, diff1, diff2)
	-- Locale check
	if GetLocale() ~= L["Locale"] then return end
	-- Get item sum, results are written into diff1 table
	diff1 = self:GetSum(item, diff1)
	if not diff1 then return end
	local itemType = diff1.itemType
	-- Get equip location slot id for use in GetInventoryItemLink
	local slotID = getSlotID[itemType]
	-- Don't do bags
	if not slotID then return end

	--debug
	copyTable(SLTest, diff1)

	-- 1h weapon, check if player can dual wield, check for 2h equipped
	if itemType == "INVTYPE_WEAPON" then
		local eqItem1Sum = GetInventoryItemLink("player", 16)
		if eqItem1Sum then
			eqItem1Sum = self:GetSum(eqItem1Sum)
		else
			eqItem1Sum = newStatTable()
		end
		diff1 = diff1 - eqItem1Sum

		-- If player can Dual Wield, calculate offhand difference
		if IsUsableSpell(BS["Dual Wield"]) then
			-- If 2h is equipped, copy diff1 to diff2
			if eqItem1Sum.itemType == "INVTYPE_2HWEAPON" then
				diff2 = diff2 or new()
				copyTable(diff2, diff1)
			else
				diff2 = self:GetSum(item, diff2)
				local eqItem2Sum = GetInventoryItemLink("player", 17)
				if eqItem2Sum then
					eqItem2Sum = self:GetSum(eqItem2Sum)
				else
					eqItem2Sum = newStatTable()
				end
				diff2 = diff2 - eqItem2Sum
				del(eqItem2Sum)
			end
			del(eqItem1Sum)
			return diff1, diff2
		else
			del(eqItem1Sum)
			return diff1
		end
	-- Ring or trinket
	elseif type(slotID) == "table" then
		-- Get slot link
		local eqItem1Sum = GetInventoryItemLink("player", slotID[1])
		-- Check slot link to see if slot has an item equipped
		if eqItem1Sum then
			eqItem1Sum = self:GetSum(eqItem1Sum)
		-- Else get an empty table
		else
			eqItem1Sum = newStatTable()
		end
		-- Calculate the stat difference, results are written into diff1 table
		diff1 = diff1 - eqItem1Sum
		-- Slot2
		diff2 = self:GetSum(item, diff2)
		local eqItem2Sum = GetInventoryItemLink("player", slotID[2])
		if eqItem2Sum then
			eqItem2Sum = self:GetSum(eqItem2Sum)
		else
			eqItem2Sum = newStatTable()
		end
		diff2 = diff2 - eqItem2Sum
		-- Cleanup
		del(eqItem1Sum)
		del(eqItem2Sum)
		return diff1, diff2
	-- 2h weapon, so we calculate the difference with equipped main hand and off hand
	elseif itemType == "INVTYPE_2HWEAPON" then
		local eqItem1Sum = GetInventoryItemLink("player", 16)
		if eqItem1Sum then
			eqItem1Sum = self:GetSum(eqItem1Sum)
		else
			eqItem1Sum = newStatTable()
		end
		local eqItem2Sum = GetInventoryItemLink("player", 17)
		if eqItem2Sum then
			eqItem2Sum = self:GetSum(eqItem2Sum)
		else
			eqItem2Sum = newStatTable()
		end
		diff1 = diff1 - eqItem1Sum - eqItem2Sum
		-- Cleanup
		del(eqItem1Sum)
		del(eqItem2Sum)
		return diff1
	-- Off hand slot, check if we have 2h equipped
	elseif slotID == 17 then
		local eqItem1Sum = GetInventoryItemLink("player", 16)
		if eqItem1Sum then
			eqItem1Sum = self:GetSum(eqItem1Sum)
		else
			eqItem1Sum = newStatTable()
		end
		-- If 2h is equipped
		if eqItem1Sum.itemType == "INVTYPE_2HWEAPON" then
			diff1 = diff1 - eqItem1Sum
		else
			local eqItem1Sum = GetInventoryItemLink("player", 17)
			if eqItem1Sum then
				eqItem1Sum = self:GetSum(eqItem1Sum)
			else
				eqItem1Sum = newStatTable()
			end
			diff1 = diff1 - eqItem1Sum
			del(eqItem1Sum)
		end
		del(eqItem1Sum)
		return diff1
	-- Single slot item
	else
		local eqItem1Sum = GetInventoryItemLink("player", slotID)
		if eqItem1Sum then
			eqItem1Sum = self:GetSum(eqItem1Sum)
		else
			eqItem1Sum = newStatTable()
		end
		diff1 = diff1 - eqItem1Sum
		-- Cleanup
		del(eqItem1Sum)
		return diff1
	end
end


-- StatLogic:Bench(1000)
---------
-- self:GetSum(link, table)
-- 1000 times: 0.82 sec without cache
-- 1000 times: 0.04 sec with cache
---------
-- ItemBonusLib:ScanItemLink(link)
-- 1000 times: 1.58 sec
---------
function StatLogic:Bench(k)
	local t1 = GetTime()
	local link = GetInventoryItemLink("player", 12)
	local table = {}
	for i = 1, k, 1 do
		---------------------------------------------------------------------------
		--self:SplitDoJoin("+24 Agility/+4 Stamina, +4 Dodge and +4 Spell Crit/+5 Spirit", {"/", " and ", ","})
		---------------------------------------------------------------------------
		self:GetSum(link)
		--ItemBonusLib:ScanItemLink(link)
		---------------------------------------------------------------------------
		--ItemRefTooltip:SetScript("OnTooltipSetItem", function(frame, ...) RatingBuster:Print("OnTooltipSetItem") end)
		---------------------------------------------------------------------------
	end
	return GetTime() - t1
end


function StatLogic:PatternTest()
	patternTable = {
		"(%a[%a ]+%a) ?%d* ?%a* by u?p? ?t?o? ?(%d+) ?a?n?d? ?", -- xxx xxx by 22 (scan first)
		"(%a[%a ]+) %+(%d+) ?a?n?d? ?", -- xxx xxx +22 (scan 2ed)
		"(%d+) ([%a ]+) ?a?n?d? ?", -- 22 xxx xxx (scan last)
	}
	textTable = {
		"Spell Damage +6 and Spell Hit Rating +5",
		"+3 Stamina, +4 Critical Strike Rating",
		"+26 Healing Spells & 2% Reduced Threat",
		"+3 Stamina/+4 Critical Strike Rating",
		"Socket Bonus: 2 mana per 5 sec.",
		"Equip: Increases damage and healing done by magical spells and effects by up to 150.",
		"Equip: Increases the spell critical strike rating of all party members within 30 yards by 28.",
		"Equip: Increases damage and healing done by magical spells and effects of all party members within 30 yards by up to 33.",
		"Equip: Increases healing done by magical spells and effects of all party members within 30 yards by up to 62.",
		"Equip: Increases your spell damage by up to 120 and your healing by up to 300.",
		"Equip: Restores 11 mana per 5 seconds to all party members within 30 yards.",
		"Equip: Increases healing done by spells and effects by up to 300.",
		"Equip: Increases attack power by 420 in Cat, Bear, Dire Bear, and Moonkin forms only.",
	}
	for _, text in ipairs(textTable) do
		DEFAULT_CHAT_FRAME:AddMessage(text.." = ")
		for _, pattern in ipairs(patternTable) do
			local found
			for k, v in gmatch(text, pattern) do
				found = true
				DEFAULT_CHAT_FRAME:AddMessage("  '"..k.."', '"..v.."'")
			end
			if found then
				DEFAULT_CHAT_FRAME:AddMessage("  using: "..pattern)
				DEFAULT_CHAT_FRAME:AddMessage("----------------------------")
				break
			end
		end
	end
end

AceLibrary:Register(StatLogic, MAJOR_VERSION, MINOR_VERSION, activate)

_G.StatLogic = AceLibrary("StatLogic-1.0")
